/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.authentication;

import java.net.Socket;
import java.math.BigInteger;
import java.util.*;
import org.apache.log4j.Logger;
import com.sshtools.j2ssh.transport.InvalidStateException;
import com.sshtools.j2ssh.transport.MessageAlreadyRegisteredException;
import com.sshtools.j2ssh.transport.Service;
import com.sshtools.j2ssh.transport.ServiceOperationException;
import com.sshtools.j2ssh.transport.ServiceState;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.j2ssh.transport.TransportProtocolException;

/**
 *  <p>
 *
 *  Implements the SSH Authentication protocol as a Transport layer service.
 *  This class can authenticate with any authentication object implementing the
 *  SshAuthentication interface and returns the success, partial success or
 *  failure result synchronously.</p> <p>
 *
 *  The authentication process is as follows:<br>
 *  <br>
 *  The authentication service is started<br>
 *  The server MAY reply with a banner message (which can be retreived using
 *  <code>
 *  getBannerMessage</code><br>
 *  The user MAY request the list of available authorizations by calling <code>getAvailableAuths</code>
 *  <br>
 *  The user MUST request authentication by calling <code>authenticate</code>
 *  for each method they wish to try (with the Service instance to start)<br>
 *  The server responds with a success message and the desired service is
 *  started<br>
 *
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class AuthenticationProtocol extends Service {
    private List availableAuths = null;
    private Service service;
    private byte[] exchangeHash;
    private static Logger log = Logger.getLogger(AuthenticationProtocol.class);
    private AuthenticationProtocolState state = new AuthenticationProtocolState();
    private boolean isNoneRequest = false;
    private SshAuthentication currentAuth = null;
    private String banner;


    /**
     *  Constructor for the AuthenticationProtocol
     */
    public AuthenticationProtocol() {
        super("ssh-userauth");
    }


    /**
     *  <p>
     *
     *  Called by the framework when the service is initialized. We do not
     *  currently perform any processing here.</p>
     *
     *@exception  ServiceOperationException
     */
    public void onServiceInit() throws ServiceOperationException { }


    /**
     *  <p>
     *
     *  Called by the framework if the remote computer disconnects.</p>
     *
     *@param  reason  The reason for disconnection
     */
    public void onDisconnect(String reason) {
        stop();
    }


    /**
     *  <p>
     *
     *  Abstract method implementation called when the service is accepted by
     *  the server. We register client side message notifications here.</p>
     *
     *@exception  ServiceOperationException  Thrown if a message registration
     *      error occurs
     */
    protected void onServiceAccept() throws ServiceOperationException {

        try {
            log.info("Registering messages");

            getTransportLayer().registerMessage(new Integer(SshMsgUserAuthFailure.SSH_MSG_USERAUTH_FAILURE),
                    SshMsgUserAuthFailure.class,
                    getMessageStore());

            getTransportLayer().registerMessage(new Integer(SshMsgUserAuthSuccess.SSH_MSG_USERAUTH_SUCCESS),
                    SshMsgUserAuthSuccess.class,
                    getMessageStore());

            getTransportLayer().registerMessage(new Integer(SshMsgUserAuthBanner.SSH_MSG_USERAUTH_BANNER),
                    SshMsgUserAuthBanner.class,
                    getMessageStore());

        } catch (MessageAlreadyRegisteredException e) {
            throw new ServiceOperationException(
                    "Required message already registered by another service or protocol");
        }

    }


    /**
     *  <p>
     *
     *  Abstract method implementation called when the service is request by a
     *  remote client.</p> <p>
     *
     *  NOTE: The Authentication Protocol currently does not implement and
     *  processing here; we should register server side messages here.</p>
     */
    protected void onServiceRequest() { }


    /**
     *  <p>
     *
     *  Called by the framework when a exception occurs in a child thread</p>
     *
     *@param  e  The excpetion thrown
     */
    protected void onThreadException(Exception e) {

        stop();

    }


    /**
     *  <p>
     *
     *  Gets the current state of the authentication protocol.</p>
     *
     *@return    The AuthenticationProtocolState
     */
    public AuthenticationProtocolState getState() {
        return state;
    }


    /**
     *  <p>
     *
     *  Gets the authentication banner message received from the server. This
     *  may be null.</p>
     *
     *@return    The banner message
     */
    public String getBannerMessage() {
        return banner;
    }


    /**
     *  <p>
     *
     *  Called by the Service framework when a registered message has been
     *  received.</p>
     *
     *@param  msg                             The message received
     *@exception  ServiceOperationException
     *@exception  TransportProtocolException
     */
    protected void onMessageReceived(SshMessage msg)
             throws ServiceOperationException, TransportProtocolException {

        switch (msg.getMessageId()) {
            case SshMsgUserAuthFailure.SSH_MSG_USERAUTH_FAILURE:
            {
                onMsgUserAuthFailure((SshMsgUserAuthFailure) msg);
                break;
            }
            case SshMsgUserAuthSuccess.SSH_MSG_USERAUTH_SUCCESS:
            {
                onMsgUserAuthSuccess((SshMsgUserAuthSuccess) msg);
                break;
            }
            case SshMsgUserAuthBanner.SSH_MSG_USERAUTH_BANNER:
            {
                onMsgUserAuthBanner((SshMsgUserAuthBanner) msg);
                break;
            }
            default:
            {
                throw new ServiceOperationException("Unregistered message received!");
            }
        }
    }


    /**
     *  <p>
     *
     *  Gets the available authentication methods returned by the server.</p>
     *  <p>
     *
     *  NOTE: The authentication protocol states that the server may return
     *  authentication methods that are not valid for the user.</p>
     *
     *@param  username                        The username to request
     *      authentication methods for
     *@param  serviceName                     The service name to start
     *@return                                 The list of authentication methods
     *@exception  TransportProtocolException  Description of the Exception
     */
    public List getAvailableAuths(String username, String serviceName)
             throws TransportProtocolException {

        log.info("Requesting authentication methods");

        if (availableAuths == null) {

            SshMsgUserAuthRequest msg = new SshMsgUserAuthRequest(username,
                    serviceName, "none");

            isNoneRequest = true;

            getTransportLayer().sendMessage(msg, this);

            // Wait for the state to change before returning i.e. we have a list
            // of available authentication methods
            state.waitForState(AuthenticationProtocolState.READY);

        }

        isNoneRequest = false;

        return availableAuths;
    }


    /**
     *  <p>
     *
     *  Authenticate a user with the given authentication method.</p>
     *
     *@param  auth                                 The authentication method
     *      instance to try.
     *@param  serviceToStart                       The service instance to
     *      start.
     *@return                                      The result of the
     *      authentication; this is an AuthenticationProtocolState value.
     *@exception  AuthenticationProtocolException
     *@exception  ServiceOperationException
     *@exception  TransportProtocolException       Description of the Exception
     */
    public int authenticate(SshAuthentication auth, Service serviceToStart)
             throws TransportProtocolException,
            AuthenticationProtocolException,
            ServiceOperationException {

        auth.authenticate(getTransportLayer(), serviceToStart.getServiceName());

        int ret = state.waitForStateUpdate();

        if (ret == AuthenticationProtocolState.COMPLETE) {
            ServiceState state = new ServiceState();

            try {
                state.setState(ServiceState.SERVICE_ACCEPTED);
                serviceToStart.start(getTransportLayer(), state, exchangeHash);
                return ret;
            } catch (InvalidStateException ise) {
                onThreadException(ise);
            }
        }

        return ret;
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_USERAUTH_FAILURE message.</p>
     *
     *@param  msg  The message received.
     */
    private void onMsgUserAuthFailure(SshMsgUserAuthFailure msg) {

        if (msg.getPartialSuccess() && !isNoneRequest) {
            log.info("Previous authentication method succeeded but additional method(s) are required");
        } else {
            log.info("Previous authentication method failed; try an alternative method");
        }

        // Record the list of authenticaitons available
        availableAuths = msg.getAvailableAuthentications();

        // If this is a none request set our state to ready

        try {
            if (isNoneRequest) {
                state.updateState(AuthenticationProtocolState.READY);
                return;
            }

            // Update the state object with the correct state
            if (msg.getPartialSuccess()) {
                state.updateState(AuthenticationProtocolState.PARTIAL);
            } else {
                state.updateState(AuthenticationProtocolState.FAILED);
            }

        } catch (InvalidStateException ise) {

        }
    }



    /**
     *  <p>
     *
     *  Handles the SSH_MSG_USERAUTH_SUCCESS message.</p>
     *
     *@param  msg  The message received
     */
    private void onMsgUserAuthSuccess(SshMsgUserAuthSuccess msg) {

        log.debug("Authentication succeeded");

        // Update the state
        try {
            state.updateState(AuthenticationProtocolState.COMPLETE);
        } catch (InvalidStateException ise) {

        }
        // We can now stop the authentication service
        stop();

    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_USERAUTH_BANNER message. The user should be shown
     *  the banner message.</p>
     *
     *@param  msg  The message received.
     */
    private void onMsgUserAuthBanner(SshMsgUserAuthBanner msg) {

        banner = msg.getBanner();

    }

}
