/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.authentication;

import java.awt.Frame;
import com.sshtools.j2ssh.transport.TransportProtocol;
import com.sshtools.j2ssh.transport.TransportProtocolException;

/**
 *  <p>
 *
 *  Implements the Password Authentication for the SSH Authenticaiton Protocol
 *  </p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class PasswordAuthentication implements SshAuthentication {

    private TransportProtocol transport;
    private String username;
    private String password;


    /**
     *  Constructs the PasswordAuthentication object
     */
    public PasswordAuthentication() { }


    /**
     *  <p>
     *
     *  Displays a modal login prompt.</p>
     *
     *@param  parent  The parent frame
     *@return         True if the instance is ready to authenticate otherwise
     *      false
     */
    public boolean showAuthenticationDialog(Frame parent) {

        // Create the password authentication dialog
        PasswordAuthenticationDialog dialog = new PasswordAuthenticationDialog(parent);

        // Show the dialog
        if (dialog.showPromptForPassword(username)) {
            username = dialog.getUsername();
            password = dialog.getPassword();
            return true;
        }

        return false;
    }


    /**
     *  <p>
     *
     *  Sets the username for the authentication.</p>
     *
     *@param  username  The username
     */
    public void setUsername(String username) {
        this.username = username;
    }


    /**
     *  <p>
     *
     *  Sets the password for the authentication.</p>
     *
     *@param  password  The user's password
     */
    public void setPassword(String password) {
        this.password = password;
    }


    /**
     *  <p>
     *
     *  Returns the SSH User Authentication method name.</p>
     *
     *@return    The method name
     */
    public String getMethodName() {

        return "password";
    }


    /**
     *  <p>
     *
     *  Sends the password authentication over the transport protocol.</p>
     *
     *@param  transport                            The connected transport
     *      protocol
     *@param  serviceToStart                       The service to start after an
     *      authentication success
     *@exception  TransportProtocolException
     *@exception  AuthenticationProtocolException  Description of the Exception
     */
    public void authenticate(TransportProtocol transport, String serviceToStart)
             throws TransportProtocolException,
            AuthenticationProtocolException {

        this.transport = transport;

        if (username == null || password == null) {
            throw new AuthenticationProtocolException("Username and password cannot be null!");
        }

        // Send a password authentication request
        SshMsgUserAuthPwdRequest msg = new SshMsgUserAuthPwdRequest(username, password, serviceToStart);

        transport.sendMessage(msg, this);

    }

}

