/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.authentication;

import java.util.*;
import org.apache.log4j.Logger;
import com.sshtools.j2ssh.configuration.AuthenticationConfiguration;
import com.sshtools.j2ssh.configuration.AuthenticationMethod;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.configuration.SshAPIConfiguration;
import com.sshtools.j2ssh.transport.AlgorithmNotSupportedException;

/**
 *  <p>
 *
 *  Factory object to create instances of SSH Authentication objects. Additional
 *  authentication methods can be defined in the API configuration file ssh.xml.
 *  </p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshAuthenticationFactory {

    private static Map auths;
    private static Logger log = Logger.getLogger(SshAuthenticationFactory.class);
    /**
     *  The Password authentication method name
     */
    public final static String AUTH_PASSWORD = "password";

    static {

        auths = new HashMap();

        log.info("Loading supported authentication methods");

        log.debug("Password implemented by " + PasswordAuthentication.class.getName());
        auths.put(AUTH_PASSWORD, PasswordAuthentication.class.getName());

        // Now load external methods from configuration file
        SshAPIConfiguration config = ConfigurationLoader.getAPIConfiguration();

        if (config != null) {

            AuthenticationConfiguration authConfig = config.getAuthenticationConfiguration();

            List addons = authConfig.getAuthenticationMethod();

            Iterator it = addons.iterator();

            // Add the methods to our supported list
            while (it.hasNext()) {

                AuthenticationMethod method = (AuthenticationMethod) it.next();

                String name = method.getMethodName();

                // Make sure it follows the extensibility rules
                if (name.lastIndexOf("@") > 0) {
                    log.debug(name + " implemented by " + method.getImplementationClass());
                    auths.put(name, method.getImplementationClass());
                } else {

                    log.error("Unable to load " + name
                            + "; method name does not conform to extensibility rules");
                }

            }

        }
    }


    /**
     *  Constructor for SshAuthenticationFactory
     */
    protected SshAuthenticationFactory() { }


    /**
     *  Gets the supported authentication methods
     *
     *@return    A list containing the authentication methods
     */
    public static List getSupportedMethods() {

        // Get the list of ciphers
        ArrayList list = new ArrayList(auths.keySet());

        // Return the list
        return list;
    }


    /**
     *  <p>
     *
     *  Creates a new instance of the SshAuthentication object which implements
     *  methodName.</p>
     *
     *@param  methodName                          The method to instansiate
     *@return                                     The instance
     *@exception  AlgorithmNotSupportedException  Thrown if the method is not
     *      supported
     */
    public static SshAuthentication newInstance(String methodName)
             throws AlgorithmNotSupportedException {

        String className = "";
        try {
            className = (String) auths.get(methodName);

            if (className != null) {
                return (SshAuthentication) Class.forName(className).newInstance();
            } else {
                throw new AlgorithmNotSupportedException(methodName + " is not supported");
            }
        } catch (Exception e) {
            throw new AlgorithmNotSupportedException("Could not locate " + className);
        }

    }
}
