/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.authentication;

import java.io.*;
import java.util.*;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  <Implements the SSH_MSG_USERAUTH_FAILURE message/</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshMsgUserAuthFailure extends SshMessage {

    private List auths;
    private boolean partialSuccess;

    /**
     *  The message id
     */
    protected final static int SSH_MSG_USERAUTH_FAILURE = 51;


    /**
     *  <p>
     *
     *  Constructs the message.</p>
     *
     *@param  auths                        The authentication methods available
     *@param  partialSuccess               True if a partial success
     *@exception  InvalidMessageException
     */
    public SshMsgUserAuthFailure(String auths, boolean partialSuccess)
             throws InvalidMessageException {
        super(SSH_MSG_USERAUTH_FAILURE);

        loadListFromDelimString(auths);

        this.partialSuccess = partialSuccess;

    }


    /**
     *  <p>
     *
     *  Gets the message name for debugging.</p>
     *
     *@return
     */
    public String getMessageName() {
        return "SSH_MSG_USERAUTH_FAILURE";
    }


    /**
     *  <p>
     *
     *  Constructs the message.</p>
     */
    public SshMsgUserAuthFailure() {
        super(SSH_MSG_USERAUTH_FAILURE);
    }


    /**
     *  <p>
     *
     *  Loads the lists of available authentications from a comma delimited
     *  string.</p>
     *
     *@param  list  Description of the Parameter
     */
    private void loadListFromDelimString(String list) {

        StringTokenizer tok = new StringTokenizer(list, ",");

        auths = new ArrayList();

        while (tok.hasMoreElements()) {

            auths.add(tok.nextElement());
        }

    }


    /**
     *  <p>
     *
     *  Constructs the message from a byte array.</p>
     *
     *@param  bar                          The data being read
     *@exception  InvalidMessageException
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            String auths = bar.readString();
            partialSuccess = (bar.read() != 0 ? true : false);

            loadListFromDelimString(auths);

        } catch (IOException ioe) {

            throw new InvalidMessageException("Invalid message data");
        }
    }


    /**
     *  <p>
     *
     *  Constructs a byte array containing the message.</p>
     *
     *@param  baw                                                       The byte
     *      array being written to
     *@exception  com.sshtools.j2ssh.transport.InvalidMessageException
     *      Description of the Exception
     */
    protected void constructByteArray(ByteArrayWriter baw) throws com.sshtools.j2ssh.transport.InvalidMessageException {

        try {

            String sep = "";
            String authMethods = "";
            Iterator it = auths.iterator();

            while (it.hasNext()) {

                authMethods = sep + (String) it.next();
                sep = ",";

            }

            baw.writeString(authMethods);

            baw.write((partialSuccess ? 1 : 0));

        } catch (IOException ioe) {

            throw new InvalidMessageException("Invalid message data");
        }
    }


    /**
     *  Gets the available Authentications methods
     *
     *@return    The list of available authentications
     */
    public List getAvailableAuthentications() {
        return auths;
    }


    /**
     *  <p>
     *
     *  Gets the partial success flag.</p>
     *
     *@return    The partial success value
     */
    public boolean getPartialSuccess() {
        return partialSuccess;
    }

}
