/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.client;

import java.io.*;
import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;
import java.net.Socket;
import java.util.*;
import org.apache.log4j.Level;
import com.sshtools.j2ssh.configuration.SshConnectionProperties;
import com.sshtools.j2ssh.SshException;
import com.sshtools.j2ssh.authentication.AuthenticationProtocol;
import com.sshtools.j2ssh.authentication.AuthenticationProtocolException;
import com.sshtools.j2ssh.authentication.SshAuthentication;
import com.sshtools.j2ssh.connection.Channel;
import com.sshtools.j2ssh.connection.ConnectionProtocol;
import com.sshtools.j2ssh.connection.WindowSpaceNotAvailableException;
import com.sshtools.j2ssh.transport.HostKeyVerification;
import com.sshtools.j2ssh.transport.ServiceOperationException;
import com.sshtools.j2ssh.transport.TransportProtocolClient;
import com.sshtools.j2ssh.transport.TransportProtocolException;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;
/**
 *  <p>
 *
 *  The SessionClient object provides authenticated remote session access
 *  through the SSH connection protocol.</p> <p>
 *
 *  Using the methods contained it it possible to start the execution of the
 *  users shell, execute a program or start an SSH subsystem. Users should refer
 *  to [SSH_CONN] for further technical information.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public abstract class SessionClient extends ConnectionProtocol {

    private Channel session = null;
    private static Logger log = Logger.getLogger(SessionClient.class);
    private List authMethods;
    private Socket socket;
    private TransportProtocolClient transport;
    private AuthenticationProtocol authentication;
    private String authenticationBanner = "";
    private Integer exitCode;


    /**
     *  Constructor for the SessionClient object
     */
    public SessionClient() { }


    /**
     *  Called by the framework when the service is started
     *
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    protected abstract void initSession() throws TransportProtocolException,
            ServiceOperationException;



    /**
     *  Provides the list of available authentication methods for the current
     *  user
     *
     *@return    A list containing the method names as specified in
     *      [SSH-USERAUTH]
     */
    protected List getAvailableAuthMethods() {
        return authMethods;
    }


    /**
     *  Gets the exit code of the comman exectued. This may be null if nothing
     *  has been returned from the remote side.
     *
     *@return
     */
    protected Integer getExitCode() {
        return exitCode;
    }


    /**
     *  Requests that the remote server start the users shell
     *
     *@return                                 The success of the operation
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    public boolean startShell() throws TransportProtocolException,
            ServiceOperationException {

        log.debug("Requesting users shell");

        // Send the request for a shell, we want a reply
        return sendChannelRequest(session, "shell", true, null);
    }


    /**
     *  Sets the environment variable for the command or shell to be started
     *
     *@param  name                            The environment variable name
     *@param  value                           The environment variable value
     *@return                                 The success of the request
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    public boolean setEnvironmentVariable(String name, String value)
             throws TransportProtocolException, ServiceOperationException {

        log.debug("Requesting environment variable to be set [" + name + "=" + value + "]");

        try {
            ByteArrayWriter baw = new ByteArrayWriter();

            baw.writeString(name);
            baw.writeString(value);

            return sendChannelRequest(session, "env", true, baw.toByteArray());
        } catch (IOException ioe) {

            throw new ServiceOperationException("Could not set environment variable;" +
                    " error writing request data");
        }
    }


    /**
     *  Changes the terminal dimensions to the newly provided settings
     *
     *@param  term                            The pseudo terminal instance
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    public void changeTerminalDimensions(PseudoTerminal term)
             throws TransportProtocolException, ServiceOperationException {

        log.debug("Changing terminal dimensions");

        try {
            ByteArrayWriter baw = new ByteArrayWriter();

            baw.writeInt(term.getColumns());
            baw.writeInt(term.getRows());
            baw.writeInt(term.getWidth());
            baw.writeInt(term.getHeight());

            sendChannelRequest(session, "window-change", false, baw.toByteArray());

        } catch (IOException ioe) {
            throw new ServiceOperationException("Failed to change terminal dimensions; error writing request data");
        }
    }


    /**
     *  Requests that the remote server allocate a psuedo terminal for the
     *  session.
     *
     *@param  term                            The terminal answerback mode
     *@return                                 The success of the operation
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    public boolean requestPseudoTerminal(PseudoTerminal term)
             throws TransportProtocolException, ServiceOperationException {

        log.info("Requesting pseudo terminal");
        log.debug("Terminal Type is " + term.getTerm());

        // This requests a pseudo terminal
        try {
            ByteArrayWriter baw = new ByteArrayWriter();
            baw.writeString(term.getTerm());
            baw.writeInt(term.getColumns());
            baw.writeInt(term.getRows());
            baw.writeInt(term.getWidth());
            baw.writeInt(term.getHeight());
            baw.writeString(term.getEncodedTerminalModes());

            return sendChannelRequest(session, "pty-req", true, baw.toByteArray());
        } catch (IOException ioe) {
            throw new ServiceOperationException("Failed to write channel request data to ByteArrayWriter");
        }

    }


    /**
     *  Requests that the remote server start the execution of the given
     *  command.
     *
     *@param  command                         The command to execute
     *@return                                 The success of the operation
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    public boolean executeCommand(String command)
             throws TransportProtocolException, ServiceOperationException {
        log.info("Requesting command execution");
        log.debug("Command is " + command);

        try {
            ByteArrayWriter baw = new ByteArrayWriter();

            baw.writeString(command);

            return sendChannelRequest(session, "exec", true, baw.toByteArray());
        } catch (IOException ioe) {

            throw new ServiceOperationException("Failed to request command execution; error writing request data");
        }

    }


    /**
     *  Requests that the remote server executes the predefined subsystem (for
     *  example SFTP).
     *
     *@param  subsystem                       The predefined subsystem to start
     *@return                                 The success of the operation
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    public boolean startSubsystem(String subsystem)
             throws TransportProtocolException, ServiceOperationException {

        log.info("Starting " + subsystem + " subsystem");

        try {
            ByteArrayWriter baw = new ByteArrayWriter();

            baw.writeString(subsystem);

            return sendChannelRequest(session, "subsystem", true, baw.toByteArray());
        } catch (IOException ioe) {

            throw new ServiceOperationException("Failed to start the " + subsystem +
                    " subsystem; could not write request data");
        }
    }


    /**
     *  Called by the framework when the remote server has sent EOF for the
     *  given channel
     *
     *@param  channel  The channel for which EOF was received
     */
    protected void onChannelEOF(Channel channel) { }


    /**
     *  Called by the framework when the remote server has closed the specified
     *  channel. At this point the channel must be closed and the session is
     *  disconnected (calling onDisconnect)
     *
     *@param  channel  The channel to be closed
     */
    protected void onChannelClose(Channel channel) {

        transport.disconnect("Closing session");
        onDisconnect("The session was closed");

    }


    /**
     *  Called when the session is disconnecting
     *
     *@param  reason  The disconnection reason
     */
    protected abstract void onDisconnect(String reason);


    /**
     *  Called by the framework when the remote server wishes to open a channel
     *
     *@param  channel      The channel for the request
     *@param  requestData  The extended channel open data
     *@return              Return true if the channel is to bo opened, otherwise
     *      false
     */
    protected boolean onChannelOpen(Channel channel, byte[] requestData) {

        return false;
    }


    /**
     *  Connects to the remote host synchronously
     *
     *@param  properties                      The connection properties
     *@param  hostVerification                The host verification instance
     *@exception  ServiceOperationException   Throw if any service exception
     *      occurs
     *@exception  TransportProtocolException  May be thrown by transport
     *      protocol methods used
     */
    public void connectSynchronously(SshConnectionProperties properties, HostKeyVerification hostVerification)
             throws TransportProtocolException,
            ServiceOperationException {

        try {
            SessionClient.this.socket = new Socket(properties.getHost(), properties.getPort());
        } catch (IOException ioe) {
            throw new ServiceOperationException("Unable to connect socket to " + properties.getHost());
        }

        // Start the transport protocol
        SessionClient.this.transport =
                new TransportProtocolClient(
                SessionClient.this.socket,
                properties,
                hostVerification);

        // Start the authentication protocol
        SessionClient.this.authentication =
                (AuthenticationProtocol)
                SessionClient.this.transport.requestService("ssh-userauth");

        // Get the available authentication methods for the user

        SessionClient.this.authMethods =
                SessionClient.this.authentication.getAvailableAuths(
                properties.getUsername(),
                SessionClient.this.getServiceName());

        // If we've received an authentication banner the record it
        SessionClient.this.authenticationBanner =
                SessionClient.this.authentication.getBannerMessage();

        // Inform the subclasses that the connection is complete
        SessionClient.this.onConnectionComplete();

    }


    /**
     *  Connects to the remote computer asynchronously. Calls the abstract
     *  method <code>onConnectionComplete</code> when the connection has been
     *  completed.
     *
     *@param  properties        The properties to connect
     *@param  hostVerification  The host verification instance
     */
    public void connectAsynchronously(final SshConnectionProperties properties,
            final HostKeyVerification hostVerification) {

        Thread thread = new Thread(
            new Runnable() {
                public void run() {

                    try {
                        connectSynchronously(properties, hostVerification);
                    } catch (SshException e) {
                        onThreadException(e);
                    }
                }
            });

        thread.start();

    }


    /**
     *  Called after the connect has completed. At this point there is a list of
     *  suitable authenticaiton methods available for the user and possibly an
     *  authenticaiton banner message to display
     */
    protected abstract void onConnectionComplete();


    /**
     *  Authenticates the user using the authentication method provided.
     *
     *@param  auth                                 The authenticaiton method
     *      instance
     *@return                                      The authentication result
     *@exception  AuthenticationProtocolException
     *@exception  ServiceOperationException
     *@exception  TransportProtocolException
     */
    public int authenticate(SshAuthentication auth)
             throws TransportProtocolException,
            AuthenticationProtocolException,
            ServiceOperationException {

        // Do the authentication
        int result = authentication.authenticate(auth, this);

        return result;
    }


    /**
     *  Closes the session
     *
     *@exception  ServiceOperationException
     */
    public void close() throws TransportProtocolException,
                                      ServiceOperationException {

     if (session != null) {
          closeChannel(session);
      }

      transport.disconnect("Closing session");

      onDisconnect("Disconnected");

      stop();

    }


    /**
     *  Returns the authenticaiton banner supplied by the remote server.
     *
     *@return
     */
    public String getAuthenticationBanner() {
        return authenticationBanner;
    }


    /**
     *  Called when a SSH_MSG_GLOBAL_REQUEST is received
     *
     *@param  requestName  The request name
     *@param  wantReply    Does the remote computer wnat a reply?
     *@param  requestData  The request data
     */
    protected void onGlobalRequest(String requestName, boolean wantReply, byte[] requestData) { }


    /**
     *  Called when a SSH_MSG_REQUEST_FAILURE is recieved
     */
    protected void onGlobalRequestFailure() { }


    /**
     *  Called when the SSH_MSG_REQUEST_SUCCESS message is received
     *
     *@param  requestData  The request data
     */
    protected void onGlobalRequestSuccess(byte[] requestData) { }



    /**
     *  Called by the framework when channel data arrives. This then proceeds to
     *  validate window space and then call the correct handler for the channel
     *
     *@param  channel  The channel receiving the data
     *@param  data     The data received
     */
    protected void onChannelData(Channel channel, byte[] data) {

        // If were running out of window space then increase it
        if (channel.getLocalWindowSize() < 128) {
            log.debug("Increasing window space");

            try {
                sendChannelWindowAdjust(channel, 32768 - channel.getLocalWindowSize());
                channel.increaseLocalWindowSize(32768 - channel.getLocalWindowSize());
            } catch (TransportProtocolException tpe) {
                onThreadException(tpe);
            }
        }

        if (channel.equals(session)) {
            onSessionData(data);
        }
    }


    /**
     *  Called when session data arrives
     *
     *@param  data  The unprocessed session data
     */
    protected abstract void onSessionData(byte[] data);


    /**
     *  Called by the framework when ext channel data arrives.
     *
     *@param  channel  The channel for which the data has arrived
     *@param  data     The extended data
     */
    protected void onChannelExtData(Channel channel, byte[] data) {

        // If were running out of window space then increase it
        if (channel.getLocalWindowSize() < 128) {
            log.debug("Increasing window space");

            try {
                sendChannelWindowAdjust(channel, 32768 - channel.getLocalWindowSize());
                channel.increaseLocalWindowSize(32768 - channel.getLocalWindowSize());
            } catch (TransportProtocolException tpe) {

            }
        }

        if (channel.equals(session)) {
            onSessionData(data);
        }
    }


    /**
     *  Called when extended session data arrives
     *
     *@param  data  The extended session data
     */
    protected abstract void onSessionExtData(byte[] data);


    /**
     *  Called by the framework when a channel request arrives
     *
     *@param  channel      The channel for which the request was made
     *@param  requestType  The request type
     *@param  wantReply    Whether the remote side wants an explicit reply
     *@param  requestData  The request data
     */
    protected void onChannelRequest(Channel channel, String requestType,
            boolean wantReply, byte[] requestData) {

        if (requestType.equals("exit-status")) {
            exitCode = new Integer(ByteArrayReader.previewInt(requestData, 0));
        }
    }


    /**
     *  Called by the framework when the service has been accepted by the remote
     *  host
     *
     *@exception  ServiceOperationException  Thrown
     */
    protected void onServiceAccept() throws ServiceOperationException {

        // The connection protocol has been accepted so open up a session
        // channel with a reasonable amount of window space
        try {
            session = openChannel("session", "session", 32768, 32768, null);

            // call the abstract method to allow subclasses to perform service specific
            // initialization
            initSession();
        } catch (TransportProtocolException tpe) {
            throw new ServiceOperationException("Failed to send session channel open request");
        }

        if (session == null) {
            throw new ServiceOperationException("Failed to open session channel");
        }
    }


    /**
     *  Called by the framwork when the remote computer requests a service. This
     *  method ignores the requests on a client implementation
     */
    protected void onServiceRequest() {
        // We are a client so just ignore these messages
    }


    /**
     *  Sends session data to the remote server
     *
     *@param  data                                  The data to send
     *@exception  WindowSpaceNotAvailableException
     *@exception  TransportProtocolException
     */
    public void sendSessionData(byte[] data)
             throws WindowSpaceNotAvailableException,
            TransportProtocolException {

        if (session.getRemoteWindowSize() < data.length) {
            throw new WindowSpaceNotAvailableException();
        }

        sendChannelData(session, data);

    }

}
