/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.configuration;

import java.io.*;
import org.apache.log4j.Logger;
import java.security.*;


/**
 *  <p>
 *
 *  Performs one time configuration loading for the JSSH project.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class ConfigurationLoader {

    private static Logger log = Logger.getLogger(ConfigurationLoader.class);
    private static SshAPIConfiguration config;
    private static String sshtoolsHome;

    static {

        try {

            // Get the sshtools.home system property
            sshtoolsHome = System.getProperty("sshtools.home");

            if (sshtoolsHome == null) {

                log.info("sshtools.home not set; defaulting to java.home/lib");

                // Not sshtools.home so lets try the java.home instead
                sshtoolsHome = System.getProperty("java.home");

                if (!sshtoolsHome.endsWith(File.separator)) {
                    sshtoolsHome += File.separator;
                }

                sshtoolsHome += "lib";

            }
	    else {

               if(!sshtoolsHome.endsWith(File.separator))
                   sshtoolsHome += File.separator;

               sshtoolsHome += "conf";

            }


            if (!sshtoolsHome.endsWith(File.separator))
                sshtoolsHome += File.separator;


            String apiConfig = sshtoolsHome + "ssh.xml";

            log.info("Loading api configuration from " + apiConfig);

            // Load the configuration file
            File f = new File(apiConfig);

            FileInputStream in = new FileInputStream(f);

            config = SshAPIConfiguration.unmarshal(in);

        } catch (Exception e) {
            log.fatal(e);
            config = null;
        }
    }


    /**
     *  Constructor for the ConfigurationLoader object
     */
    protected ConfigurationLoader() { }


    /**
     *  <p>
     *
     *  Static method to call so we can invoke the static block, just in case we
     *  don't want to wait for a static call to <code>getAPIConfiguration()
     * </code>.</p>
     */
    public static void init() {
        // Forces the static block so we can force loading
    }


    /**
     *  <p>
     *
     *  Gets the API Configuration instance.</p>
     *
     *@return
     */
    public static SshAPIConfiguration getAPIConfiguration() {

        return config;
    }


    /**
     *  <p>
     *
     *  Returns the sshtools home directory. If the system property is not found
     *  then the loader defaults to java.home/lib. This method will always
     *  return the path with a trailing File.separator
     *
     *@return
     */
    public static String getConfigurationLocation() {
        return sshtoolsHome;
    }
}
