/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.configuration;

import java.util.*;
import java.io.*;
import javax.xml.bind.StructureValidationException;
import javax.xml.bind.UnmarshalException;
import com.sshtools.j2ssh.transport.cipher.SshCipherFactory;
import com.sshtools.j2ssh.transport.compression.SshCompressionFactory;
import com.sshtools.j2ssh.transport.hmac.SshHmacFactory;
import com.sshtools.j2ssh.transport.publickey.SshPublicKeyFactory;
import com.sshtools.j2ssh.transport.kex.SshKeyExchangeFactory;

/**
 *  <p>
 *
 *  Represents an SSH connection. An instance of this class is passed to the
 *  connect methods of the transport protocol. It specifies all the preferred
 *  algorithms and methods</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshConnectionProperties {

    private String prefEncryption = SshCipherFactory.getDefaultCipher();
    private String prefDecryption = SshCipherFactory.getDefaultCipher();
    private String prefSendMac = SshHmacFactory.getDefaultHmac();
    private String prefRecvMac = SshHmacFactory.getDefaultHmac();
    private String prefSendComp = SshCompressionFactory.getDefaultCompression();
    private String prefRecvComp = SshCompressionFactory.getDefaultCompression();
    private String prefPK = SshPublicKeyFactory.getDefaultPublicKey();
    private String prefKex = SshKeyExchangeFactory.getDefaultKeyExchange();
    private String host;
    private String username;
    private int port = 22;
    private List authMethods = new ArrayList();
    private String term;



    /**
     *  Constructor for the SshConnectionProperties object
     *
     *@param  host      The host to connect to
     *@param  username  The name of the user
     */
    public SshConnectionProperties(String host, String username) {
        this.host = host;
        this.username = username;
    }


    /**
     *  Constructor for the SshConnectionProperties object
     */
    public SshConnectionProperties() { }


    /**
     *  Opens a connection file and loads the properties ready for connection
     *
     *@param  file                             The full path to the file
     *@throws  InvalidConnectionFileException  Thrown if the file is not a valid
     *      connection file
     */
    public void open(String file) throws InvalidConnectionFileException {

        try {

            File f = new File(file);

            FileInputStream in = new FileInputStream(f);

            SshConnection ssh;

            ssh = SshConnection.unmarshal(in);

            host = ssh.getHostname();
            username = ssh.getUsername();
            port = Integer.valueOf(ssh.getPort()).intValue();
            term = ssh.getTerminalType();

            PreferredEncryption encrypt = ssh.getPreferredEncryption();

            prefEncryption = encrypt.getClientToServer();
            prefDecryption = encrypt.getServerToClient();

            PreferredMac mac = ssh.getPreferredMac();

            prefSendMac = mac.getClientToServer();
            prefRecvMac = mac.getServerToClient();

            PreferredCompression comp = ssh.getPreferredCompression();

            prefSendComp = comp.getClientToServer();
            prefRecvComp = comp.getServerToClient();

            prefKex = ssh.getPreferredKeyExchange();

            prefPK = ssh.getPreferredPublicKey();

            List list = ssh.getAuthenticationMethod();

            authMethods = new ArrayList(list);

            in.close();
        } catch (FileNotFoundException fnfe) {
            throw new InvalidConnectionFileException(file + " was not found!");
        } catch (UnmarshalException ue) {
            throw new InvalidConnectionFileException(file + " was not a valid connection file");
        } catch (IOException ioe) {
            throw new InvalidConnectionFileException("IO error on " + file);
        } finally {

        }
    }


    /**
     *  Call this method to save the connection properties to file
     *
     *@param  file                             The file to save to
     *@throws  InvalidConnectionFileException
     */
    public void save(String file) throws InvalidConnectionFileException {

        try {

            File f = new File(file);

            FileOutputStream out = new FileOutputStream(f);

            SshConnection ssh = new SshConnection();

            ssh.setHostname(host);
            ssh.setUsername(username);
            ssh.setPort(String.valueOf(port));
            ssh.setTerminalType(term);

            PreferredEncryption encrypt = new PreferredEncryption();

            encrypt.setClientToServer(prefEncryption);
            encrypt.setServerToClient(prefDecryption);

            ssh.setPreferredEncryption(encrypt);

            PreferredMac mac = new PreferredMac();

            mac.setClientToServer(prefSendMac);
            mac.setServerToClient(prefRecvMac);

            ssh.setPreferredMac(mac);

            PreferredCompression comp = new PreferredCompression();

            comp.setClientToServer(prefSendComp);
            comp.setServerToClient(prefRecvComp);

            ssh.setPreferredCompression(comp);

            ssh.setPreferredKeyExchange(prefKex);

            ssh.setPreferredPublicKey(prefPK);

            ssh.getAuthenticationMethod().addAll(authMethods);

            ssh.validate();

            ssh.marshal(out);

            out.close();
        } catch (FileNotFoundException fnfe) {
            throw new InvalidConnectionFileException(file + " was not found!");
        } catch (StructureValidationException v) {
            throw new InvalidConnectionFileException(file + " was not a valid connection file!");
        } catch (IOException ioe) {
            throw new InvalidConnectionFileException("io error on " + file);
        } finally {

        }

    }


    /**
     *  Adds and authentication method to the properties. When a connection is
     *  saved, these methods are attempted without the need for the user to
     *  specify them
     *
     *@param  method  The method name to add
     */
    public synchronized void addAuthenticationMethod(String method) {

        if (!authMethods.contains(method)) {
            authMethods.add(new String(method));
        }
    }


    /**
     *  Removes an authentication method
     *
     *@param  method
     */
    public synchronized void removeAuthenticaitonMethod(String method) {

        authMethods.remove(method);

    }


    /**
     *  Gets the list of authentication methods for this connection
     *
     *@return
     */
    public List getAuthenticationMethods() {
        return (List) ((ArrayList) authMethods).clone();
    }


    /**
     *  Sets the terminal type
     *
     *@param  term
     */
    public void setTerminalType(String term) {
        this.term = term;
    }


    /**
     *  Gets the terminal type
     *
     *@return
     */
    public String getTerminalType() {
        return term;
    }


    /**
     *  Gets the host name for the connection
     *
     *@return    The host name
     */
    public String getHost() {
        return host;
    }


    /**
     *  Gets the port for this connection
     *
     *@return    The port value
     */
    public int getPort() {
        return port;
    }



    /**
     *  Sets the host name for this connection
     *
     *@param  host  The new host value
     */
    public void setHost(String host) {
        this.host = host;
    }


    /**
     *  Sets the username for the connection
     *
     *@param  username
     */
    public void setUsername(String username) {
        this.username = username;
    }


    /**
     *  Sets the port for the connection
     *
     *@param  port  The new port value
     */
    public void setPort(int port) {
        this.port = port;
    }


    /**
     *  Gets the username for the connection
     *
     *@return    The username value
     */
    public String getUsername() {
        return username;
    }


    /**
     *  Gets the preferred client->server encryption method for the connection
     *
     *@return    The prefEncryption value
     */
    public String getPrefCSEncryption() {
        return prefEncryption;
    }


    /**
     *  Sets the preferred key exchange for the connection
     *
     *@param  pref
     */
    public void setPrefKex(String pref) {
        prefKex = pref;
    }


    /**
     *  Sets the prefEncryption attribute of the SshConnectionProperties object
     *
     *@param  pref  The method name
     */
    public void setPrefCSEncryption(String pref) {
        prefEncryption = pref;
    }


    /**
     *  Gets the preferred server->client encryption for the conneciton
     *
     *@return    The method name
     */
    public String getPrefSCEncryption() {
        return prefDecryption;
    }


    /**
     *  Sets the preferred server->client encryption for the connection
     *
     *@param  pref  The method name
     */
    public void setPrefSCEncryption(String pref) {
        prefDecryption = pref;
    }


    /**
     *  Gets the preferred client->server message authentication
     *
     *@return    The method name
     */
    public String getPrefCSMac() {
        return prefSendMac;
    }


    /**
     *  Sets the preferred client->server message authentication
     *
     *@param  pref  The new prefCSMac value
     */
    public void setPrefCSMac(String pref) {
        prefSendMac = pref;
    }


    /**
     *  Gets the preferred server->client message authentication
     *
     *@return    The method name
     */
    public String getPrefSCMac() {
        return prefRecvMac;
    }


    /**
     *  Sets the preferred server->client message authentication
     *
     *@param  pref  The new prefSCMac value
     */
    public void setPrefSCMac(String pref) {
        prefRecvMac = pref;
    }


    /**
     *  Gets the preferred client->server compression
     *
     *@return    The method name
     */
    public String getPrefCSComp() {
        return prefSendComp;
    }


    /**
     *  Sets the preferred client->server compression
     *
     *@param  pref  The new prefCSComp value
     */
    public void setPrefCSComp(String pref) {
        prefSendComp = pref;
    }


    /**
     *  Gets the preferred server->client compression
     *
     *@return    The method name
     */
    public String getPrefSCComp() {
        return prefRecvComp;
    }


    /**
     *  Sets the preferred server->client compression
     *
     *@param  pref  The new prefSCComp value
     */
    public void setPrefSCComp(String pref) {
        prefRecvComp = pref;
    }


    /**
     *  Gets the preferred public key mechanism
     *
     *@return    The method name
     */
    public String getPrefPublicKey() {
        return prefPK;
    }


    /**
     *  Sets the preferred public key mechanism
     *
     *@param  pref  The method name
     */
    public void setPrefPublicKey(String pref) {
        prefPK = pref;
    }


    /**
     *  Gets the preferred key exchange method
     *
     *@return    The method value
     */
    public String getPrefKex() {
        return prefKex;
    }
}
