/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.connection;

/**
 *  <p>
 *
 *  Implements an SSH connection protocol channel.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class Channel {

    private String channelType;
    private String channelName;
    private long localWindowSize = 0;
    private long remoteWindowSize = 0;
    private long localChannelId;
    private long remoteChannelId;
    private static int channelIdGen = 0;
    private long localPacketSize;
    private long remotePacketSize;
    private boolean isClosed = false;
    private boolean isLocalEOF = false;
    private boolean isRemoteEOF = false;
    private byte[] channelOpenData;


    /**
     *  <p>
     *
     *  The constructor for the Channel when the local side request the channel
     *  to be opened.</p>
     *
     *@param  channelName  The channel name (this can be anything you want to
     *      identify with)
     *@param  channelType  The SSH channel type
     *@param  windowSize   The local window size for the channel
     *@param  packetSize   The maximum packet size
     */
    public Channel(String channelName, String channelType, long windowSize, long packetSize) {

        this.localChannelId = channelIdGen;
        channelIdGen++;

        this.channelName = channelName;
        this.channelType = channelType;
        this.localWindowSize = windowSize;
        this.localPacketSize = packetSize;

    }


    /**
     *  <p>
     *
     *  Constructor for the Channel when the channel request comes from the
     *  remote side.</p>
     *
     *@param  channelType       The SSH channel type
     *@param  remoteChannelId   The remote sides channel id
     *@param  remoteWindowSize  The remote sides window size
     *@param  remotePacketSize  The remote sides maximum packet size
     */
    public Channel(String channelType, long remoteChannelId, long remoteWindowSize, long remotePacketSize) {

        this.localChannelId = channelIdGen;
        channelIdGen++;

        this.channelName = "";
        this.channelType = channelType;
        this.remoteWindowSize = remoteWindowSize;
        this.remotePacketSize = remotePacketSize;
        this.remoteChannelId = remoteChannelId;

    }


    /**
     *  <p>
     *
     *  Sets the SSH_MSG_CHANNEL_OPEN request data.</p>
     *
     *@param  data
     */
    public void setChannelOpenData(byte[] data) {
        channelOpenData = data;
    }


    /**
     *  <p>
     *
     *  Gets the SSH_MSG_CHANNEL_OPEN message data.</p>
     *
     *@return
     */
    public byte[] getChannelOpenData() {
        return channelOpenData;
    }


    /**
     *  <p>
     *
     *  Gets the channel type.</p>
     *
     *@return
     */
    public String getChannelType() {
        return channelType;
    }


    /**
     *  <p>
     *
     *  Gets the channel name. This is a descriptive name that you can use to
     *  identify the channel to your own application.</p>
     *
     *@return
     */
    public String getChannelName() {
        return channelName;
    }


    /**
     *  <p>
     *
     *  Returns the local channel id.</p>
     *
     *@return
     */
    public long getLocalChannelId() {
        return localChannelId;
    }


    /**
     *  <p>
     *
     *  Returns the current local window size.</p>
     *
     *@return
     */
    public synchronized long getLocalWindowSize() {

        return localWindowSize;
    }


    /**
     *  <p>
     *
     *  Returns the current remote window size.</p>
     *
     *@return
     */
    public synchronized long getRemoteWindowSize() {

        return remoteWindowSize;
    }


    /**
     *  <p>
     *
     *  Returns the remote channel id.</p>
     *
     *@return
     */
    public long getRemoteChannelId() {
        return remoteChannelId;
    }


    /**
     *  <p>
     *
     *  Increases the local window size.</p>
     *
     *@param  count  The number of bytes to add.
     */
    public synchronized void increaseLocalWindowSize(long count) {

        localWindowSize += count;

    }


    /**
     *  <p>
     *
     *  Increases the remote window size.</p>
     *
     *@param  count  The number of bytes to add.
     */
    protected synchronized void increaseRemoteWindowSize(long count) {
        remoteWindowSize += count;
    }


    /**
     *  <p>
     *
     *  Consumes remote window space.</p>
     *
     *@param  count  The number of bytes to consume.
     */
    protected synchronized void consumeRemoteWindow(long count) {

        remoteWindowSize -= count;

    }


    /**
     *  <p>
     *
     *  Consumes local window space.</p>
     *
     *@param  count  The number of bytes to consume.
     */
    protected synchronized void consumeLocalWindow(long count) {

        localWindowSize -= count;

    }


    /**
     *  <p>
     *
     *  Sets the remote channel id.</p>
     *
     *@param  channelId
     */
    protected void setRemoteChannelId(long channelId) {
        remoteChannelId = channelId;
    }


    /**
     *  <p>
     *
     *  Gets the remote maximum packet size.</p>
     *
     *@return
     */
    public long getRemotePacketSize() {
        return remotePacketSize;
    }


    /**
     *  <p>
     *
     *  Gets the local maximum packet size.</p>
     *
     *@return
     */
    public long getLocalPacketSize() {
        return localPacketSize;
    }


    /**
     *  <p>
     *
     *  Sets the remote maximum packet size.</p>
     *
     *@param  packetSize
     */
    protected void setRemotePacketSize(long packetSize) {
        remotePacketSize = packetSize;
    }


    /**
     *  <p>
     *
     *  Sets the local maximum packet size.</p>
     *
     *@param  packetSize
     */
    protected void setLocalPacketSize(long packetSize) {
        localPacketSize = packetSize;
    }


    /**
     *  <p>
     *
     *  Closes the channel.</p>
     */
    protected void close() {
        isClosed = true;
    }


    /**
     *  <p>
     *
     *  Returns true if the channel is closed.</p>
     *
     *@return
     */
    public boolean isClosed() {
        return isClosed;
    }


    /**
     *  <p>
     *
     *  Sets the local side as End Of File. No data can be received.</p>
     */
    protected void setLocalEOF() {
        isLocalEOF = true;
    }


    /**
     *  <p>
     *
     *  Returns true if the local side is EOF.</p>
     *
     *@return
     */
    public boolean isLocalEOF() {
        return isLocalEOF;
    }


    /**
     *  <p>
     *
     *  Sets the remote side as End Of File. No data can be sent.</p>
     */
    protected void setRemoteEOF() {
        isRemoteEOF = true;
    }


    /**
     *  <p>
     *
     *  Returns true if the remote side is EOF.</p>
     *
     *@return
     */
    public boolean isRemoteEOF() {
        return isRemoteEOF;
    }

}
