/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.connection;
import java.net.Socket;
import java.util.*;
import java.io.IOException;
import java.net.UnknownHostException;
import org.apache.log4j.Logger;
import com.sshtools.j2ssh.transport.MessageAlreadyRegisteredException;
import com.sshtools.j2ssh.transport.Service;
import com.sshtools.j2ssh.transport.ServiceOperationException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.j2ssh.transport.TransportProtocol;
import com.sshtools.j2ssh.transport.TransportProtocolException;

/**
 *  <p>
 *
 *  Implementation for the SSH connection protocol as a transport layer service.
 *  This class provides method for sending all the available connection protocol
 *  message as well as abstract methods for handling the same messages.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public abstract class ConnectionProtocol extends Service {

    private Map channels = new HashMap();
    private static Logger log = Logger.getLogger(ConnectionProtocol.class);
    TransportProtocol transport;


    /**
     *  Constructor for the SshConnectionProtocol object
     */
    public ConnectionProtocol() {
        super("ssh-connection");
    }


    /**
     *  <p>
     *
     *  Called by the framework once the service has been accepted or requested.
     *  </p>
     *
     *@exception  ServiceOperationException
     */
    public void onServiceInit() throws ServiceOperationException {

        try {
            transport = getTransportLayer();

            log.info("Registering connection protocol messages");

            transport.registerMessage(
                    new Integer(SshMsgChannelOpenConfirmation.SSH_MSG_CHANNEL_OPEN_CONFIRMATION),
                    SshMsgChannelOpenConfirmation.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelOpenFailure.SSH_MSG_CHANNEL_OPEN_FAILURE),
                    SshMsgChannelOpenFailure.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelOpen.SSH_MSG_CHANNEL_OPEN),
                    SshMsgChannelOpen.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelClose.SSH_MSG_CHANNEL_CLOSE),
                    SshMsgChannelClose.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelEOF.SSH_MSG_CHANNEL_EOF),
                    SshMsgChannelEOF.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelData.SSH_MSG_CHANNEL_DATA),
                    SshMsgChannelData.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelExtendedData.SSH_MSG_CHANNEL_EXTENDED_DATA),
                    SshMsgChannelExtendedData.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelFailure.SSH_MSG_CHANNEL_FAILURE),
                    SshMsgChannelFailure.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelRequest.SSH_MSG_CHANNEL_REQUEST),
                    SshMsgChannelRequest.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelSuccess.SSH_MSG_CHANNEL_SUCCESS),
                    SshMsgChannelSuccess.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgChannelWindowAdjust.SSH_MSG_CHANNEL_WINDOW_ADJUST),
                    SshMsgChannelWindowAdjust.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgGlobalRequest.SSH_MSG_GLOBAL_REQUEST),
                    SshMsgGlobalRequest.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgRequestFailure.SSH_MSG_REQUEST_FAILURE),
                    SshMsgRequestFailure.class,
                    getMessageStore());

            transport.registerMessage(
                    new Integer(SshMsgRequestSuccess.SSH_MSG_REQUEST_SUCCESS),
                    SshMsgRequestSuccess.class,
                    getMessageStore());
        } catch (MessageAlreadyRegisteredException e) {
            throw new ServiceOperationException("Could not register all messages! " + e.getMessage());
        }

    }


    /**
     *  <p>
     *
     *  Gets the Channel instance for the channelId provided.</p>
     *
     *@param  channelId  The local channel id
     *@return            The Channel instance
     */
    private Channel getChannel(long channelId) {

        return (Channel) channels.get(new Long(channelId));
    }


    /**
     *  <p>
     *
     *  Removes a channel.</p>
     *
     *@param  channel  The channel to remove
     */
    private void removeChannel(Channel channel) {

        channels.remove(new Long(channel.getLocalChannelId()));
    }


    /**
     *  <p>
     *
     *  Sends the connection protocol message SSH_MSG_GLOBAL_REQUEST.</p>
     *
     *@param  requestName                     The request name
     *@param  wantReply                       Whether a reply is needed
     *@param  requestData                     The request specific data
     *@exception  TransportProtocolException
     */
    protected void sendGlobalRequest(String requestName, boolean wantReply, byte[] requestData)
             throws TransportProtocolException {

        SshMsgGlobalRequest msg = new SshMsgGlobalRequest(requestName, wantReply, requestData);

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Sends the connection protocol message SSH_MSG_REQUEST_SUCCESS.</p>
     *
     *@param  requestData                     The request specific data
     *@exception  TransportProtocolException
     */
    protected void sendGlobalRequestSuccess(byte[] requestData)
             throws TransportProtocolException {

        SshMsgRequestSuccess msg = new SshMsgRequestSuccess(requestData);

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Sends the connection protocol message SSH_MSG_REQUEST_FAILURE.</p>
     *
     *@exception  TransportProtocolException
     */
    protected void sendGlobalRequestFailure()
             throws TransportProtocolException {

        SshMsgRequestFailure msg = new SshMsgRequestFailure();

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Syncrhonously opens a channel by sending the SSH_MSG_CHANNEL_OPEN
     *  message and waits for either the SSH_MSG_CHANNEL_OPEN_FAILURE or
     *  SSH_MSG_CHANNEL_OPEN_CONFIRMATION messages. If the request succeeds the
     *  Channel instance os returned, otherwise the method returns null.</p>
     *
     *@param  channelName                     The descriptive name for the
     *      channel
     *@param  channelType                     The SSH channel type (i.e.
     *      ;session')
     *@param  initialWindowSize               The initial window size required
     *@param  maximumPacketSize               The maximum packet size required
     *@param  channelData                     Channel specific request data
     *@return                                 Description of the Return Value
     *@exception  TransportProtocolException
     *@exception  ServiceOperationException   Description of the Exception
     */
    protected Channel openChannel(String channelName, String channelType, long initialWindowSize,
            long maximumPacketSize, byte[] channelData)
             throws TransportProtocolException,
            ServiceOperationException {

        // Create a new channel
        Channel channel = new Channel(channelName, channelType, initialWindowSize, maximumPacketSize);

        //channel.increaseLocalWindowSize(initialWindowSize);

        // Create the message
        SshMsgChannelOpen msg = new SshMsgChannelOpen(channel.getChannelType(),
                channel.getLocalChannelId(), channel.getLocalWindowSize(),
                maximumPacketSize, channelData);

        // Send the message
        transport.sendMessage(msg, this);

        // Wait for the next message
        int messageIdFilter[] = new int[2];
        messageIdFilter[0] =
                SshMsgChannelOpenConfirmation.SSH_MSG_CHANNEL_OPEN_CONFIRMATION;

        messageIdFilter[1] =
                SshMsgChannelOpenFailure.SSH_MSG_CHANNEL_OPEN_FAILURE;

        SshMessage result = waitForSingleMessage(messageIdFilter);

        if (result.getMessageId() ==
                SshMsgChannelOpenConfirmation.SSH_MSG_CHANNEL_OPEN_CONFIRMATION) {

            // Set the properties
            channel.setRemoteChannelId(((SshMsgChannelOpenConfirmation) result).getSenderChannel());
            channel.setRemotePacketSize(((SshMsgChannelOpenConfirmation) result).getMaximumPacketSize());
            channel.increaseRemoteWindowSize(((SshMsgChannelOpenConfirmation) result).getInitialWindowSize());
            channel.setChannelOpenData(((SshMsgChannelOpenConfirmation) result).getChannelData());
            channels.put(new Long(channel.getLocalChannelId()), channel);

            return channel;
        } else {
            return null;
        }
    }


    /**
     *  <p>
     *
     *  Sends the SSH_MSG_CHANNEL_OPEN_CONFIRMATION message.</p>
     *
     *@param  channel                         The channel to confirm open
     *@param  channelData                     Channel specific data
     *@exception  TransportProtocolException
     */
    protected void sendChannelOpenConfirmation(Channel channel, byte[] channelData)
             throws TransportProtocolException {

        SshMsgChannelOpenConfirmation msg = new SshMsgChannelOpenConfirmation(
                channel.getRemoteChannelId(),
                channel.getLocalChannelId(),
                channel.getLocalWindowSize(),
                channel.getLocalPacketSize(),
                channelData);

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Sends the SSH_MSG_CHANNEL_OPEN_FAILURE message.</p>
     *
     *@param  channel                         The channel to inform of failure
     *@param  reasonCode                      The reason code
     *@param  additionalInfo                  Additional information
     *@param  languageTag                     The language tag
     *@exception  TransportProtocolException
     */
    protected void sendChannelOpenFailure(Channel channel, long reasonCode,
            String additionalInfo,
            String languageTag)
             throws TransportProtocolException {

        SshMsgChannelOpenFailure msg = new SshMsgChannelOpenFailure(channel.getRemoteChannelId(),
                reasonCode,
                additionalInfo,
                languageTag);

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Closes the channel by sending the SSH_MSG_CHANNEL_CLOSE message.</p>
     *
     *@param  channel                         The channel to close
     *@exception  TransportProtocolException
     */
    protected void closeChannel(Channel channel) throws TransportProtocolException {

        SshMsgChannelClose msg = new SshMsgChannelClose(channel.getLocalChannelId());

        transport.sendMessage(msg, this);

        channel.close();

    }


    /**
     *  <p>
     *
     *  Sets the local channel as End Of File by sending the SSH_MSG_CHANNEL_EOF
     *  message.</p>
     *
     *@param  channel                         The channel to set EOF
     *@exception  TransportProtocolException
     */
    protected void sendChannelEOF(Channel channel) throws TransportProtocolException {

        SshMsgChannelEOF msg = new SshMsgChannelEOF(channel.getRemoteChannelId());

        transport.sendMessage(msg, this);

        channel.setLocalEOF();

    }


    /**
     *  <p>
     *
     *  Sends data to the channel using the SSH_MSG_CHANNEL_DATA.</p>
     *
     *@param  channel                               The channel to send data
     *      over
     *@param  data                                  The data to send
     *@exception  TransportProtocolException
     *@exception  WindowSpaceNotAvailableException  Description of the Exception
     */
    protected void sendChannelData(Channel channel, byte[] data)
             throws WindowSpaceNotAvailableException, TransportProtocolException {

        // Consume the window space
        if (channel.getRemoteWindowSize() > data.length) {

            channel.consumeRemoteWindow(data.length);

            SshMsgChannelData msg = new SshMsgChannelData(
                    channel.getRemoteChannelId(),
                    data);

            transport.sendMessage(msg, this);

        } else {
            throw new WindowSpaceNotAvailableException();
        }

    }


    /**
     *  <p>
     *
     *  Sends data to the channel using the SSH_MSG_CHANNEL_EXT_DATA.</p>
     *
     *@param  channel                               The channel to send data
     *      through
     *@param  data                                  The data channel
     *@exception  TransportProtocolException
     *@exception  WindowSpaceNotAvailableException  Description of the Exception
     */
    protected void sendChannelExtData(Channel channel, byte[] data)
             throws WindowSpaceNotAvailableException,
            TransportProtocolException {

        // Consume the window space
        if (channel.getRemoteWindowSize() > data.length) {

            channel.consumeRemoteWindow(data.length);

            SshMsgChannelExtendedData msg = new SshMsgChannelExtendedData(
                    channel.getRemoteChannelId(),
                    SshMsgChannelExtendedData.SSH_EXTENDED_DATA_STDERR,
                    data);

            transport.sendMessage(msg, this);
        } else {
            throw new WindowSpaceNotAvailableException();
        }
    }


    /**
     *  <p>
     *
     *  Sends the SSH_MSG_CHANNEL_FAILURE message.</p>
     *
     *@param  channel                         The failed channel
     *@exception  TransportProtocolException
     */
    protected void sendChannelFailure(Channel channel) throws TransportProtocolException {

        SshMsgChannelFailure msg = new SshMsgChannelFailure(channel.getRemoteChannelId());

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Sends the SSH_MSG_CHANNEL_WINDOW_ADJUST message.</p>
     *
     *@param  channel                         The channel to send
     *@param  bytesToAdd                      The number of bytes to add to the
     *      window space
     *@exception  TransportProtocolException
     */
    protected void sendChannelWindowAdjust(Channel channel, long bytesToAdd) throws TransportProtocolException {

        SshMsgChannelWindowAdjust msg = new SshMsgChannelWindowAdjust(
                channel.getRemoteChannelId(),
                bytesToAdd);

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Sends the SSH_MSG_CHANNEL_SUCCES message.</p>
     *
     *@param  channel                         The channel
     *@exception  TransportProtocolException
     */
    protected void sendChannelSuccess(Channel channel) throws TransportProtocolException {

        SshMsgChannelSuccess msg = new SshMsgChannelSuccess(channel.getRemoteChannelId());

        transport.sendMessage(msg, this);

    }


    /**
     *  <p>
     *
     *  Sends the SSH_MSG_CHANNEL_REQUEST message.</p>
     *
     *@param  channel                         The channel for the request
     *@param  requestType                     The channel request type
     *@param  wantReply                       Whether a reply is needed
     *@param  requestData                     The request specific data
     *@return                                 Description of the Return Value
     *@exception  TransportProtocolException
     *@exception  ServiceOperationException   Description of the Exception
     *@excpetion                              ServiceOperationException
     */
    protected boolean sendChannelRequest(Channel channel, String requestType,
            boolean wantReply, byte[] requestData)
             throws TransportProtocolException,
            ServiceOperationException {

        log.debug("Sending " + requestType + "request for the "
                + channel.getChannelType() + " channel");

        SshMsgChannelRequest msg = new SshMsgChannelRequest(channel.getRemoteChannelId(),
                requestType,
                wantReply,
                requestData);

        transport.sendMessage(msg, this);

        // If the user requests a reply then wait for the message and return result
        if (wantReply) {

            // Set up our message filter
            int messageIdFilter[] = new int[2];
            messageIdFilter[0] = SshMsgChannelSuccess.SSH_MSG_CHANNEL_SUCCESS;
            messageIdFilter[1] = SshMsgChannelFailure.SSH_MSG_CHANNEL_FAILURE;

            // Wait for either success or failure
            SshMessage reply = waitForSingleMessage(messageIdFilter);

            switch (reply.getMessageId()) {

                case SshMsgChannelSuccess.SSH_MSG_CHANNEL_SUCCESS:
                {
                    log.debug("Channel success");
                    return true;
                }
                case SshMsgChannelFailure.SSH_MSG_CHANNEL_FAILURE:
                {
                    log.debug("Channel failure");
                    return false;
                }
            }

        }

        return false;
    }



    /**
     *  <p>
     *
     *  Implements the abstract method to handle the registered message
     *  notifications.</p>
     *
     *@param  msg                             The message received
     *@exception  ServiceOperationException
     *@exception  TransportProtocolException
     */
    public void onMessageReceived(SshMessage msg) throws ServiceOperationException, TransportProtocolException {

        // Route the message to the correct handling function
        switch (msg.getMessageId()) {
            case SshMsgGlobalRequest.SSH_MSG_GLOBAL_REQUEST:
            {
                onMsgGlobalRequest((SshMsgGlobalRequest) msg);
                break;
            }
            case SshMsgRequestFailure.SSH_MSG_REQUEST_FAILURE:
            {
                onMsgRequestFailure((SshMsgRequestFailure) msg);
                break;
            }
            case SshMsgRequestSuccess.SSH_MSG_REQUEST_SUCCESS:
            {
                onMsgRequestSuccess((SshMsgRequestSuccess) msg);
                break;
            }
            case SshMsgChannelOpen.SSH_MSG_CHANNEL_OPEN:
            {
                onMsgChannelOpen((SshMsgChannelOpen) msg);
                break;
            }
            case SshMsgChannelClose.SSH_MSG_CHANNEL_CLOSE:
            {
                onMsgChannelClose((SshMsgChannelClose) msg);
                break;
            }
            case SshMsgChannelEOF.SSH_MSG_CHANNEL_EOF:
            {
                onMsgChannelEOF((SshMsgChannelEOF) msg);
                break;
            }
            case SshMsgChannelData.SSH_MSG_CHANNEL_DATA:
            {
                onMsgChannelData((SshMsgChannelData) msg);
                break;
            }
            case SshMsgChannelExtendedData.SSH_MSG_CHANNEL_EXTENDED_DATA:
            {
                onMsgChannelExtendedData((SshMsgChannelExtendedData) msg);
                break;
            }
            case SshMsgChannelRequest.SSH_MSG_CHANNEL_REQUEST:
            {
                onMsgChannelRequest((SshMsgChannelRequest) msg);
                break;
            }
            case SshMsgChannelWindowAdjust.SSH_MSG_CHANNEL_WINDOW_ADJUST:
            {
                onMsgChannelWindowAdjust((SshMsgChannelWindowAdjust) msg);
                break;
            }
            default:
            {
                // If we never registered it why are we getting it?
                throw new ServiceOperationException("Unregistered message received!");
            }
        }

    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_GLOBAL_REQUEST message. Calls the abstract method
     *  <code>onGlobalRequest</code>.</p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException
     */
    private void onMsgGlobalRequest(SshMsgGlobalRequest msg)
             throws ServiceOperationException {

        onGlobalRequest(msg.getRequestName(), msg.getWantReply(),
                msg.getRequestData());

    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_REQUEST_SUCCESS message calling the abstract method
     *  <code>onGlobalRequestSuccess</code>.</p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException
     */
    private void onMsgRequestSuccess(SshMsgRequestSuccess msg) throws ServiceOperationException {

        onGlobalRequestSuccess(msg.getRequestData());
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_REQUEST_FAILURE message calling the abstract method
     *  <code>onGlobalRequestFailure</code>.</p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException
     */
    private void onMsgRequestFailure(SshMsgRequestFailure msg) throws ServiceOperationException {

        onGlobalRequestFailure();
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_CHANNEL_OPEN message calling the abstract method
     *  <code>onChannelOpen</code> to determine whether the channel should be
     *  opened or rejected.</p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException
     */
    private void onMsgChannelOpen(SshMsgChannelOpen msg) throws ServiceOperationException {

        Channel channel = new Channel(msg.getChannelType(), msg.getSenderChannelId(),
                msg.getInitialWindowSize(), msg.getMaximumPacketSize());

        if (onChannelOpen(channel, msg.getChannelData())) {
            channels.put(new Long(channel.getLocalChannelId()), channel);
        }
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_CHANNEL_CLOSE message. This method sets the channel
     *  object for the closed channel to a closed state and calls the abstract
     *  method <code>onChannelClose</code>.</p>
     *
     *@param  msg                             The message received
     *@exception  ServiceOperationException   Thrown if the remote computer trys
     *      to access an non existent channel
     *@exception  TransportProtocolException
     */
    private void onMsgChannelClose(SshMsgChannelClose msg)
             throws ServiceOperationException, TransportProtocolException {

        Channel channel = getChannel(msg.getRecipientChannel());

        // If we have not already closed it then inform the subclasses
        if (channel == null) {
            throw new ServiceOperationException("Remote computer tried to close a " +
                    "non existent channel!");
        }
        closeChannel(channel);

        onChannelClose(channel);

        // Remove the channel
        removeChannel(channel);
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_CHANNEL_EOF message. This method sets the channel
     *  objects remote side as EOF (No more data will can be sent) and then
     *  calls the abstract method <code>onChannelEOF</code>.</p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException  Thrown if the remote computer trys
     *      to access an non existent channel
     */
    private void onMsgChannelEOF(SshMsgChannelEOF msg) throws ServiceOperationException {

        Channel channel = getChannel(msg.getRecipientChannel());

        if (channel == null) {
            throw new ServiceOperationException("Remote side tried to set a non " +
                    "existent channel to EOF!");
        }
        channel.setRemoteEOF();

        onChannelEOF(channel);
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_CHANNEL_DATA message. This method consumes window
     *  space for the data's channel and calls the abstract method <code>onChannelData
     *  </code>. Any data received above the current window space is ignored.
     *  </p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException  Thrown if the remote computer trys
     *      to access an non existent channel
     */
    private void onMsgChannelData(SshMsgChannelData msg) throws ServiceOperationException {

        // Get the channel data
        byte[] data = msg.getChannelData();

        // Get the data's channel
        Channel channel = getChannel(msg.getRecipientChannel());

        if (channel == null) {
            throw new ServiceOperationException("Remote computer sent data for non existent channel");
        }

        if (channel.getLocalWindowSize() > data.length) {

            // The data is within the window space currently allowed
            channel.consumeLocalWindow(msg.getChannelData().length);
            onChannelData(channel, data);
        } else {

            // We can ignore any extra data that is over and above the window
            // size
            data = new byte[(int) channel.getLocalWindowSize()];

            System.arraycopy(msg.getChannelData(), 0, data, 0, (int) channel.getLocalWindowSize());

            channel.consumeLocalWindow(data.length);

            onChannelData(channel, data);

        }
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_CHANNEL_EXT_DATA message. This method consumes
     *  window space for the data's channel and calls the abstract method <code>onChannelExtData
     *  </code>. Any data received above the current window space is ignored.
     *  </p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException  Thrown if the remote computer trys
     *      to access an non existent channel
     */
    private void onMsgChannelExtendedData(SshMsgChannelExtendedData msg)
             throws ServiceOperationException {

        byte[] data = msg.getChannelData();

        Channel channel = getChannel(msg.getRecipientChannel());

        if (channel == null) {
            throw new ServiceOperationException("Remote computer sent data for non existent channel");
        }

        if (channel.getLocalWindowSize() > data.length) {

            channel.consumeLocalWindow(data.length);
            onChannelData(channel, data);

        } else {

            // We can ignore any extra data that is over and above the window
            // size
            data = new byte[(int) channel.getLocalWindowSize()];

            System.arraycopy(msg.getChannelData(), 0, data, 0, (int) channel.getLocalWindowSize());

            channel.consumeLocalWindow(data.length);

            onChannelExtData(channel, data);

        }
    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_CHANNEL_WINDOW_ADJUST message. This method increases
     *  the remote sides window space by the number of bytes specified in the
     *  message.</p>
     *
     *@param  msg                            Description of the Parameter
     *@exception  ServiceOperationException  Thrown if the remote computer trys
     *      to access an non existent channel
     */
    private void onMsgChannelWindowAdjust(SshMsgChannelWindowAdjust msg) throws
            ServiceOperationException {

        log.debug(String.valueOf(msg.getBytesToAdd()) + " bytes added to remote window");

        Channel channel = getChannel(msg.getRecipientChannel());

        if (channel == null) {
            throw new ServiceOperationException("Remote computer tried to increase " +
                    "window space for a non existent channel!");
        }

        channel.increaseRemoteWindowSize(msg.getBytesToAdd());

    }


    /**
     *  <p>
     *
     *  Handles the SSH_MSG_CHANNEL_REQUEST message. This method calls the
     *  abstract method <code>onChannelRequest</code>.</p>
     *
     *@param  msg                            The message received
     *@exception  ServiceOperationException  Thrown if the remote computer trys
     *      to access an non existent channel
     */
    private void onMsgChannelRequest(SshMsgChannelRequest msg)
             throws ServiceOperationException {

        Channel channel = getChannel(msg.getRecipientChannel());

        if (channel == null) {
            throw new ServiceOperationException("Remote computer tried to make a request for " +
                    "a non existence channel!");
        }

        onChannelRequest(channel, msg.getRequestType(), msg.getWantReply(), msg.getChannelData());
    }



    /**
     *  <p>
     *
     *  Implement this method to handle the SSH_MSG_GLOBAL_REQUEST message.</p>
     *
     *@param  requestName                    The name of the request
     *@param  wantReply                      True is the remote computer wants a
     *      reply. Use <code>sendGlobalRequestSuccess</code> or <code>sendGlobalRequestFailure</code>
     *@param  requestData                    Request specific data
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract void onGlobalRequest(String requestName,
            boolean wantReply, byte[] requestData) throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implement this method to handle the SSH_MSG_REQUEST_SUCCESS message.</p>
     *
     *@param  requestData                    Request specific data
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract void onGlobalRequestSuccess(byte[] requestData)
             throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implements this method to handle the SSH_MSG_REQUEST_FAILURE message.
     *  </p>
     *
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract void onGlobalRequestFailure() throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implements this method to handle the SSH_MSG_CHANNEL_OPEN message. Use
     *  <code>sendChannelOpenConfirmation</code> or <code>sendChannelOpenFailure
     *  </code> methods to respond.</p>
     *
     *@param  channel                        The channel to be opened
     *@param  channelData                    Channel specific data
     *@return                                Return true if channel has been
     *      opened
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract boolean onChannelOpen(Channel channel, byte[] channelData)
             throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implement this method to handle the SSH_MSG_CHANNEL_CLOSE message.</p>
     *
     *@param  channel                        The channel being closed
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract void onChannelClose(Channel channel) throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implement this method to handle the SSH_MSG_CHANNEL_EOF message.</p>
     *
     *@param  channel                        The channel being set to EOF
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract void onChannelEOF(Channel channel) throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implement this method to handle the SSH_MSG_CHANNEL_REQUEST message.</p>
     *
     *@param  channel                        The channel for the request
     *@param  requestType                    The request type
     *@param  wantReply                      true if the remote side requires a
     *      reply Use <code>sendChannelRequestSuccess</code> or <code>sendChannelRequestFailure</code>
     *@param  requestData                    The request specific data
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract void onChannelRequest(Channel channel, String requestType,
            boolean wantReply, byte[] requestData) throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implement this method to handle the SSH_MSG_CHANNEL_DATA message.</p>
     *
     *@param  channel                        The data's channel
     *@param  data                           The data
     *@exception  ServiceOperationException  Throw for ant exceptions
     */
    protected abstract void onChannelData(Channel channel, byte[] data) throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Implement this methos to handle the SSH_MSG_CHANNEL_EXT_DATA message.
     *  </p>
     *
     *@param  channel                        The data's channel
     *@param  data                           The data
     *@exception  ServiceOperationException  Throw for any exceptions
     */
    protected abstract void onChannelExtData(Channel channel, byte[] data) throws ServiceOperationException;

}
