/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.connection;

import java.io.IOException;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the SSH_MSG_CHANNEL_DATA message.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshMsgChannelData extends SshMessage {

    /**
     *  The message id
     */
    public final static int SSH_MSG_CHANNEL_DATA = 94;

    // The recipient channel id
    private long recipientChannel;

    // The channel data
    private byte[] channelData;


    /**
     *  Constructor for the SshMsgChannelData object
     *
     *@param  recipientChannel  The reciepient channel id
     *@param  channelData       The channel data
     */
    public SshMsgChannelData(long recipientChannel, byte[] channelData) {
        super(SSH_MSG_CHANNEL_DATA);

        this.recipientChannel = recipientChannel;
        this.channelData = channelData;

    }


    /**
     *  <p>
     *
     *  Gets the message name for debugging.</p>
     *
     *@return
     */
    public String getMessageName() {
        return "SSH_MSG_CHANNEL_DATA";
    }


    /**
     *  <p>
     *
     *  Constructor for the SshMsgChannelData object.</p>
     */
    public SshMsgChannelData() {
        super(SSH_MSG_CHANNEL_DATA);
    }


    /**
     *  <p>
     *
     *  Gets the recipient channel id.</p>
     *
     *@return    The recipient channel id
     */
    public long getRecipientChannel() {
        return recipientChannel;
    }


    /**
     *  <p>
     *
     *  Gets the channel data.</p>
     *
     *@return    The channel data
     */
    public byte[] getChannelData() {
        return channelData;
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to construct the message from a byte
     *  array.</p>
     *
     *@param  bar                          The byte array being read
     *@exception  InvalidMessageException  Thrown if the data is invalid
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            recipientChannel = bar.readInt();

            if (bar.available() > 0) {
                channelData = bar.readBinaryString();
            }
        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to construct a byte array containing the
     *  message data.</p>
     *
     *@param  baw                          The byte array being written
     *@exception  InvalidMessageException  Thrown if the data cannot be written
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {

            baw.writeInt(recipientChannel);

            if (channelData != null) {
                baw.writeBinaryString(channelData);
            } else {
                baw.writeInt(0);
            }

        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }
    }
}
