/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.connection;

import java.io.IOException;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the SSH_MSG_CHANNEL_OPEN message.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshMsgChannelOpen extends SshMessage {

    private String channelType;
    private long senderChannelId;
    private long initialWindowSize;
    private long maximumPacketSize;
    private byte[] channelData;

    /**
     *  The message id
     */
    protected final static int SSH_MSG_CHANNEL_OPEN = 90;


    /**
     *  Constructor for the SshMsgChannelOpen object
     *
     *@param  channelType        The channel type (i.e. 'session')
     *@param  senderChannelId    The senders channel id
     *@param  initialWindowSize  The initial data window space
     *@param  maximumPacketSize  The maximum packet size
     *@param  channelData        The channel specific data
     */
    public SshMsgChannelOpen(String channelType, long senderChannelId, long initialWindowSize,
            long maximumPacketSize, byte[] channelData) {
        super(SSH_MSG_CHANNEL_OPEN);

        this.channelType = channelType;
        this.senderChannelId = senderChannelId;
        this.initialWindowSize = initialWindowSize;
        this.maximumPacketSize = maximumPacketSize;
        this.channelData = channelData;
    }


    /**
     *  Constructor for the SshMsgChannelOpen object
     */
    public SshMsgChannelOpen() {
        super(SSH_MSG_CHANNEL_OPEN);
    }


    /**
     *  Gets the channel specific data
     *
     *@return    The channel data value
     */
    public byte[] getChannelData() {
        return channelData;
    }


    /**
     *  Gets the message name for debugging
     *
     *@return
     */
    public String getMessageName() {
        return "SSH_MSG_CHANNEL_OPEN";
    }


    /**
     *  Gets the channel type
     *
     *@return    The channel type
     */
    public String getChannelType() {
        return channelType;
    }


    /**
     *  Gets the sender channel id
     *
     *@return    The sender channel id
     */
    public long getSenderChannelId() {
        return senderChannelId;
    }


    /**
     *  Gets the initial window size
     *
     *@return    The initial window size
     */
    public long getInitialWindowSize() {
        return initialWindowSize;
    }


    /**
     *  Gets the maximum packet size
     *
     *@return    The maximum packet size
     */
    public long getMaximumPacketSize() {
        return maximumPacketSize;
    }


    /**
     *  Abstract method implementation to construct the message from a byte
     *  array.
     *
     *@param  bar                          The byte array being read
     *@exception  InvalidMessageException  Thrown of the data cannot be read
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            channelType = bar.readString();
            senderChannelId = bar.readInt();
            initialWindowSize = bar.readInt();
            maximumPacketSize = bar.readInt();

            if (bar.available() > 0) {
                channelData = new byte[bar.available()];
                bar.read(channelData);
            }
        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }

    }


    /**
     *  Abstract method implementation to construct a byte array containing the
     *  message data.
     *
     *@param  baw                          The byte array being written
     *@exception  InvalidMessageException  Thrown of the data cannot be written
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {

            baw.writeString(channelType);
            baw.writeInt(senderChannelId);
            baw.writeInt(initialWindowSize);
            baw.writeInt(maximumPacketSize);
            if (channelData != null) {
                baw.write(channelData);
            }
        } catch (IOException ioe) {
            throw new InvalidMessageException("Could not write message data");
        }

    }
}
