/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.connection;

import java.io.IOException;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the SSH_MSG_CHANNEL_OPEN_FAILURE message.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshMsgChannelOpenFailure extends SshMessage {

    /**
     *  The message id
     */
    protected final static int SSH_MSG_CHANNEL_OPEN_FAILURE = 92;
    /**
     *  A Channel Failure Reason Code
     */
    protected final static long SSH_OPEN_ADMINISTRATIVELY_PROHIBITED = 1;
    /**
     *  A Channel Failure Reason Code
     */
    protected final static long SSH_OPEN_CONNECT_FAILED = 2;
    /**
     *  A Channel Failure Reason Code
     */
    protected final static long SSH_OPEN_UNKNOWN_CHANNEL_TYPE = 3;
    /**
     *  A Channel Failure Reason Code
     */
    protected final static long SSH_OPEN_RESOURCE_SHORTAGE = 4;

    private long recipientChannel;
    private long reasonCode;
    private String additional;
    private String languageTag;


    /**
     *  Constructor for the SshMsgChannelOpenFailure object
     *
     *@param  recipientChannel  The recipient channel id
     *@param  reasonCode        The reason code for the failure
     *@param  additional        Additional textual information
     *@param  languageTag       The language tag
     */
    public SshMsgChannelOpenFailure(long recipientChannel, long reasonCode,
            String additional, String languageTag) {

        super(SSH_MSG_CHANNEL_OPEN_FAILURE);
        this.recipientChannel = recipientChannel;
        this.reasonCode = reasonCode;
        this.additional = additional;
        this.languageTag = languageTag;

    }


    /**
     *  Constructor for the SshMsgChannelOpenFailure object
     */
    public SshMsgChannelOpenFailure() {
        super(SSH_MSG_CHANNEL_OPEN_FAILURE);
    }


    /**
     *  Gets the message name for debugging
     *
     *@return
     */
    public String getMessageName() {
        return "SSH_MSG_CHANNEL_OPEN_FAILURE";
    }


    /**
     *  Gets the recipient channel id
     *
     *@return    The recipient channel id
     */
    public long getRecipientChannel() {
        return recipientChannel;
    }


    /**
     *  Gets the reason code
     *
     *@return    The reason code
     */
    public long getReasonCode() {
        return reasonCode;
    }


    /**
     *  Gets the additional text
     *
     *@return    The additional text
     */
    public String getAdditionalText() {
        return additional;
    }


    /**
     *  Gets the language tag
     *
     *@return    The language tag
     */
    public String getLanguageTag() {
        return languageTag;
    }


    /**
     *  Abstract method implementation to construct the message from a byte
     *  array.
     *
     *@param  bar                          The byte array being read
     *@exception  InvalidMessageException  Thrown if the data cannot be read
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {
            recipientChannel = bar.readInt();
            reasonCode = bar.readInt();
            additional = bar.readString();
            languageTag = bar.readString();

        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }

    }


    /**
     *  Abstract method implementation to construct a byte array containing the
     *  message.
     *
     *@param  baw                          The byte array being written
     *@exception  InvalidMessageException  Thrown if the data cannot be written
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {
            baw.writeInt(recipientChannel);
            baw.writeInt(reasonCode);
            baw.writeString(additional);
            baw.writeString(languageTag);

        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }
    }
}
