/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.connection;

import java.io.IOException;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the SSH_MSG_CHANNEL_REQUEST message.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshMsgChannelRequest extends SshMessage {

    /**
     *  The message id
     */
    protected final static int SSH_MSG_CHANNEL_REQUEST = 98;

    private long recipientChannel;
    private String requestType;
    private boolean wantReply;
    private byte[] channelData;


    /**
     *  Constructor for the SshMsgChannelRequest object
     *
     *@param  recipientChannel  The recipient channel id
     *@param  requestType       The channel request type
     *@param  wantReply         True if you want the remote side to reply
     *@param  channelData       The channel request specific data
     */
    public SshMsgChannelRequest(long recipientChannel, String requestType,
            boolean wantReply, byte[] channelData) {

        super(SSH_MSG_CHANNEL_REQUEST);

        this.recipientChannel = recipientChannel;
        this.requestType = requestType;
        this.wantReply = wantReply;
        this.channelData = channelData;

    }


    /**
     *  Gets the message name for debugging
     *
     *@return
     */
    public String getMessageName() {
        return "SSH_MSG_CHANNEL_REQUEST";
    }


    /**
     *  Constructor for the SshMsgChannelRequest object
     */
    public SshMsgChannelRequest() {
        super(SSH_MSG_CHANNEL_REQUEST);
    }


    /**
     *  Gets the recipient channel id
     *
     *@return    The recipient channel id
     */
    public long getRecipientChannel() {
        return recipientChannel;
    }


    /**
     *  Gets the request type
     *
     *@return    The request type
     */
    public String getRequestType() {
        return requestType;
    }


    /**
     *  Indicates whether the sender wants an explicit reply
     *
     *@return    The want reply value
     */
    public boolean getWantReply() {
        return wantReply;
    }


    /**
     *  Gets the channel request specific data
     *
     *@return    The channel request data
     */
    public byte[] getChannelData() {
        return channelData;
    }


    /**
     *  Abstract method implementation to construct the message from a byte
     *  array.
     *
     *@param  bar                                                       The byte
     *      array being read
     *@exception  com.sshtools.j2ssh.transport.InvalidMessageException
     *      Description of the Exception
     */
    protected void constructMessage(ByteArrayReader bar) throws com.sshtools.j2ssh.transport.InvalidMessageException {

        try {

            recipientChannel = bar.readInt();
            requestType = bar.readString();
            wantReply = (bar.read() == 0 ? false : true);

            if (bar.available() > 0) {
                channelData = new byte[bar.available()];
                bar.read(channelData);
            }
        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }
    }


    /**
     *  Abstract method implementation to construct a byte array containing the
     *  message data.
     *
     *@param  baw                                                       The byte
     *      array being written
     *@exception  com.sshtools.j2ssh.transport.InvalidMessageException
     *      Description of the Exception
     */
    protected void constructByteArray(ByteArrayWriter baw) throws com.sshtools.j2ssh.transport.InvalidMessageException {

        try {

            baw.writeInt(recipientChannel);
            baw.writeString(requestType);
            baw.write((wantReply ? 1 : 0));
            if (channelData != null) {
                baw.write(channelData);
            }
        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }

    }
}
