/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.connection;

import java.io.IOException;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the SSH_MSG_GLOBAL_REQUEST message.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshMsgGlobalRequest extends SshMessage {

    private String requestName;
    private boolean wantReply;
    private byte[] requestData;

    /**
     *  The message id
     */
    protected final static int SSH_MSG_GLOBAL_REQUEST = 80;


    /**
     *  Constructor for the SshMsgGlobalRequest object
     *
     *@param  requestName  The request name
     *@param  wantReply    True if a reply is required
     *@param  requestData  The request specific data
     */
    public SshMsgGlobalRequest(String requestName, boolean wantReply, byte[] requestData) {
        super(SSH_MSG_GLOBAL_REQUEST);

        this.requestName = requestName;
        this.wantReply = wantReply;
        this.requestData = requestData;

    }


    /**
     *  Gets the message name for debugging
     *
     *@return
     */
    public String getMessageName() {
        return "SSH_MSG_GLOBAL_REQUEST";
    }


    /**
     *  Constructor for the SshMsgGlobalRequest object
     */
    public SshMsgGlobalRequest() {
        super(SSH_MSG_GLOBAL_REQUEST);
    }


    /**
     *  Abstract method implementation to construct the message from a byte
     *  array
     *
     *@param  bar                          The byte array being read
     *@exception  InvalidMessageException  Thrown if the data cannot be read
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            requestName = bar.readString();
            wantReply = (bar.read() == 0 ? false : true);

            if (bar.available() > 0) {
                requestData = new byte[bar.available()];
                bar.read(requestData);
            }
        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }
    }


    /**
     *  Abstract method implementation to construct a byte array containing the
     *  message data.
     *
     *@param  baw                          The byte array being written to.
     *@exception  InvalidMessageException  Thrown of the data cannot be written
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {

            baw.writeString(requestName);
            baw.write((wantReply ? 1 : 0));

            if (requestData != null) {
                baw.write(requestData);
            }
        } catch (IOException ioe) {
            throw new InvalidMessageException("Invalid message data");
        }
    }


    /**
     *  Gets the request data
     *
     *@return    The request data
     */
    public byte[] getRequestData() {
        return requestData;
    }


    /**
     *  Gets the request name
     *
     *@return    The request name
     */
    public String getRequestName() {
        return requestName;
    }


    /**
     *  Indicates whether the sender wants a reply
     *
     *@return    The want reply value
     */
    public boolean getWantReply() {
        return wantReply;
    }
}
