/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.*;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.StructureValidationException;
import java.util.*;
import org.apache.log4j.Logger;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;


/**
 *  <p>
 *
 *  Implement this class to define specific handling of server host
 *  authorization events. The default hosts file is hosts.xml and should be
 *  located in JSSH_HOME.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public abstract class HostKeyVerification {

    private HostAuthorizations hosts;
    private static String defaultHostFile;
    private String hostFile;

    private static Logger log = Logger.getLogger(HostKeyVerification.class);

    static {

        log.info("Determining default host file");
        // Get the sshtools.home system property
        defaultHostFile = ConfigurationLoader.getConfigurationLocation();

        // Set the default host file name to our hosts.xml
        defaultHostFile += "hosts.xml";

        log.info("Defaulting host file to " + defaultHostFile);

    }


    /**
     *  <p>
     *
     *  Constructs the object loading the default hosts file.</p>
     *
     *@exception  InvalidHostFileException  Description of the Exception
     */

    public HostKeyVerification() throws InvalidHostFileException {

        this(defaultHostFile);
        hostFile = defaultHostFile;

    }


    /**
     *  <p>
     *
     *  Constructs the object loading the host file specified.</p>
     *
     *@param  hostFileName
     *@exception  InvalidHostFileException  Description of the Exception
     *@throws  InvalidHostFileException
     */
    public HostKeyVerification(String hostFileName) throws InvalidHostFileException {

        try {

            File f = new File(hostFileName);

            FileInputStream in = new FileInputStream(f);

            hosts = HostAuthorizations.unmarshal(in);

            hostFile = hostFileName;

            in.close();
        } catch (IOException ioe) {
            throw new InvalidHostFileException("Could not open or read " + hostFileName);
        } catch (UnmarshalException ue) {
            throw new InvalidHostFileException("The format of the host file is incorrect\n" + ue.getMessage());
        }

    }


    /**
     *  <p>
     *
     *  Abstract method called by the framework when a host key has been
     *  supplied that is not currently recorded in the hosts file.</p>
     *
     *@param  host                The name of the host.
     *@param  hostKeyFingerprint  The fingerprint of the host key supplied.
     */
    public abstract void onUnknownHost(String host, String hostKeyFingerprint);


    /**
     *  <p>
     *
     *  Abstract method called by the framework when a host key has been
     *  supplied that does not match the host key recorded in the host file.</p>
     *
     *@param  host            The name of the host.
     *@param  allowedHostKey  The host key currently allowed.
     *@param  actualHostKey   The host key provided.
     */
    public abstract void onHostKeyMismatch(String host, String allowedHostKey,
            String actualHostKey);


    /**
     *  <p>
     *
     *  Abstract method called by the framework when a connection has been made
     *  to a denied host.</p> <p>
     *
     *  NOTE: This currently does not supply the fingerprint of the denied host
     *  for a reason; if the host has been denied, most probably by an
     *  administrator then we should not be providing the host key for the user
     *  to subsequently allow the host.</p>
     *
     *@param  host  The name of the denied host.
     */
    public abstract void onDeniedHost(String host);


    /**
     *  <p>
     *
     *  Allows the host access if the host provides the host key specified.</p>
     *
     *@param  host                          The name of the host.
     *@param  hostKeyFingerprint            The fingerprint of the acceptable
     *      host key.
     *@param  always                        Specifies whether to always allow
     *      the host access (true) or only allow access this time.
     *@exception  InvalidHostFileException  Description of the Exception
     */
    public void allowHost(String host, String hostKeyFingerprint, boolean always)
             throws InvalidHostFileException {

        log.debug("Allowing " + host + " with fingerprint " + hostKeyFingerprint);

        // Get the current list
        List allowList = hosts.getAllowHost();

        // Get the host if its in the list
        AllowHost allowHost = getAllowedHost(host);

        if (allowHost == null) {

            // The host is not currently allowed so create a new entry
            allowHost = new AllowHost();
            allowHost.setHostName(host);
            allowHost.setFingerprint(hostKeyFingerprint);

            // Add the host to the allowed host list
            hosts.getAllowHost().add(allowHost);
        } else {
            // The host already is in the list so set the fingerprint of the host
            allowHost.setFingerprint(hostKeyFingerprint);
        }

        // If we always want to allow then save the host file with the
        // new details
        if (always) {
            saveHostFile();
        }
    }


    /**
     *  <p>
     *
     *  Saves the current host file.</p>
     *
     *@exception  InvalidHostFileException  Description of the Exception
     */
    private void saveHostFile() throws InvalidHostFileException {

        log.info("Saving " + defaultHostFile);

        try {

            File f = new File(hostFile);

            FileOutputStream out = new FileOutputStream(f);

            hosts.validate();

            hosts.marshal(out);

            out.close();

        } catch (IOException e) {
            throw new InvalidHostFileException("Could not write to " + hostFile);
        } catch (StructureValidationException sve) {
            throw new InvalidHostFileException("Host file validation error");
        }

    }


    /**
     *  <p>
     *
     *  Denys the host access.</p>
     *
     *@param  host                          The name of the host.
     *@param  always                        Specifies whether to always deny the
     *      specified host (true) or just deny the host acceess this time
     *      (false)
     *@exception  InvalidHostFileException  Description of the Exception
     */
    public void denyHost(String host, boolean always)
             throws InvalidHostFileException {

        log.debug(host + " is denied access");
        // Get the denied host from the list
        DenyHost denyHost = getDeniedHost(host);

        // If we already have it denied then return
        if (denyHost != null) {
            return;
        }

        // Record the newly denied host
        denyHost = new DenyHost();
        denyHost.setHostName(host);
        hosts.getDenyHost().add(denyHost);

        // Save it if need be
        if (always) {
            saveHostFile();
        }
    }


    /**
     *  <p>
     *
     *  Called by the framework to verify a host key</p>
     *
     *@param  host         The name of the host
     *@param  fingerprint  The fingerprint of the host key
     *@return              The result of the host key verification
     */
    public boolean verifyHost(String host, String fingerprint) {

        log.info("Verifying " + host + " host key");
        log.debug("Fingerprint: " + fingerprint);

        // See if the host is denied
        DenyHost denied = getDeniedHost(host);

        if (denied != null) {
            return false;
        }

        // Try the allowed hosts
        AllowHost allowed = getAllowedHost(host);

        if (allowed != null) {

            // The host is allowed so check the fingerprint
            if (allowed.getFingerprint().compareToIgnoreCase(fingerprint) == 0) {
                return true;
            }

            // The host key does not match the recorded so call the abstract
            // method so that the user can decide
            onHostKeyMismatch(host, allowed.getFingerprint(), fingerprint);

            // Recheck the after the users input
            return checkFingerprint(host, fingerprint);
        } else {

            // The host is unknown os ask the user
            onUnknownHost(host, fingerprint);

            // Recheck ans return the result
            return checkFingerprint(host, fingerprint);
        }

    }


    /**
     *  <p>
     *
     *  Compares the fingerprint against the allowed hosts.</p>
     *
     *@param  host         The name of the host.
     *@param  fingerprint  The fingerprint of the host key.
     *@return              The result of the check
     */
    private boolean checkFingerprint(String host, String fingerprint) {

        AllowHost allowed = getAllowedHost(host);

        if (allowed != null) {

            if (allowed.getFingerprint().compareToIgnoreCase(fingerprint) == 0) {
                return true;
            }
        }

        return false;
    }


    /**
     *  <p>
     *
     *  Return an AllowHost object from the currently allowed hosts.</p>
     *
     *@param  host  The name of the host.
     *@return
     */
    private AllowHost getAllowedHost(String host) {

        AllowHost theHost;
        Iterator it = hosts.getAllowHost().iterator();

        while (it.hasNext()) {

            theHost = (AllowHost) it.next();

            if (theHost.getHostName().compareToIgnoreCase(host) == 0) {
                return theHost;
            }
        }

        return null;
    }


    /**
     *  <p>
     *
     *  Return a DenyHost object from the currently denied hosts list.</p>
     *
     *@param  host  The name of the host
     *@return
     */
    private DenyHost getDeniedHost(String host) {

        DenyHost theHost;
        Iterator it = hosts.getDenyHost().iterator();

        while (it.hasNext()) {

            theHost = (DenyHost) it.next();

            if (theHost.getHostName().compareToIgnoreCase(host) == 0) {
                return theHost;
            }
        }

        return null;
    }
}
