/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;
import org.apache.log4j.*;
import java.util.*;


/**
 *  <p>
 *
 *  The service class provides a mechanisnm for the ssh services provided by the
 *  transport layer. It is itself a thread and contains a message loop in which
 *  all registered messages will be routed to onMessageReceived()
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public abstract class Service implements Runnable {

    private SshMessageStore messageStore = new SshMessageStore();
    private Thread thread = new Thread(this);
    private boolean keepRunning = true;
    private static Logger log = Logger.getLogger(Service.class);
    private ServiceState state;
    private byte[] exchangeHash;

    /**
     *  The transport layer to channel service messages through
     */
    private TransportProtocol transport;

    /**
     *  The name of this service
     */
    private String serviceName;


    /**
     *  Constructor for the SshService object
     *
     *@param  serviceName  The service name
     */
    public Service(String serviceName) {
        this.serviceName = serviceName;
    }


    /**
     *  <p>
     *
     *  This method is called by the framework to initialize the service after
     *  it has been created. Subclasses should implement the abstract method
     *  onServiceInit() which is called from this method. Depending upon the
     *  state of the service passed to the init method one of the abstract
     *  methods onServiceAccept() or onServiceRequest() is then called.</p>
     *
     *@param  transport                      The transport layer object used for
     *      communication
     *@param  state                          The state of the service
     *      (Accepted/Requested)
     *@param  exchangeHash                   The exchange hash
     *@exception  ServiceOperationException  Can be thrown from abstract method
     *      implementations
     */
    public final void start(TransportProtocol transport,
            ServiceState state, byte[] exchangeHash)
             throws ServiceOperationException {

        // Save the member variables
        this.transport = transport;
        this.state = state;
        this.exchangeHash = exchangeHash;

        transport.setServiceThread(this);

        // Call the abstract init method
        onServiceInit();

        // If were accepted (i.e. client) we will call onServiceAccept()
        if (state.getState() == ServiceState.SERVICE_ACCEPTED) {

            log.info(serviceName + " has been accepted");
            onServiceAccept();
        } else {
            // We've recevied a request instead
            log.info(serviceName + " has been requested");
            onServiceRequest();
        }

        // Start the message loop
        log.info("Starting " + serviceName + " thread");
        thread.start();

    }


    /**
     *  Gets the exchange hash produced from key exchange
     *
     *@return    The exchangeHash value
     */
    protected final byte[] getExchangeHash() {
        return exchangeHash;
    }


    /**
     *  Gets the transport mechanism for this service
     *
     *@return    The Transport layer object
     */
    protected final TransportProtocol getTransportLayer() {
        return transport;
    }


    /**
     *  <p>
     *
     *  Abstract method called when the service is initialized.</p> <p>
     *
     *  NOTE: The message loop is not started until this method has completed.
     *  </p>
     *
     *@exception  ServiceOperationException  This exception can be thrown by
     *      subclasses
     */
    protected abstract void onServiceInit() throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Abstract method called when the service has been accepted by the remote
     *  computer.</p> <p>
     *
     *  NOTE: the message loop does not start until this method has completed.
     *  </p>
     *
     *@exception  ServiceOperationException  This exception can be thrown by
     *      subclasses
     */
    protected abstract void onServiceAccept()
             throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Abstract method called when the service has been requested.</p>
     *
     *@exception  ServiceOperationException  This exception can be thrown by
     *      subclasses
     */
    protected abstract void onServiceRequest()
             throws ServiceOperationException;


    /**
     *  <p>
     *
     *  Abstract method that is called by the framework whenever an exception
     *  occurs in a child thread. This is to avoid any exception being written
     *  to System.out.</p>
     *
     *@param  e  The exception thrown
     */
    protected abstract void onThreadException(Exception e);


    /**
     *  <p>
     *
     *  Abstract method called by the framework when the remote computer
     *  disconnects.</p>
     *
     *@param  reason  The reason for the disconnection.
     */
    protected abstract void onDisconnect(String reason);


    /**
     *  <p>
     *
     *  Gets the service's message store.</p>
     *
     *@return    The messageStore value
     */
    protected final SshMessageStore getMessageStore() {
        return messageStore;
    }


    /**
     *  <p>
     *
     *  Gets or waits for a single message from the message store. If the
     *  message filter is null then the next available message is returned. To
     *  filter for specific messages then create an int array containing the
     *  message id's for the messages you are prepared to accept.</p>
     *
     *@param  messageIdFilter                 Description of the Parameter
     *@return
     *@exception  TransportProtocolException  Description of the Exception
     *@exception  ServiceOperationException   Description of the Exception
     */
    protected SshMessage waitForSingleMessage(int[] messageIdFilter)
             throws TransportProtocolException,
            ServiceOperationException {

        if (messageIdFilter == null) {
            return messageStore.getMessage();
        }

        SshMessage msg;

        while (true) {

            // Get the next message
            msg = messageStore.getMessage();

            // Determine whether its one of the filtered messages
            for (int i = 0; i < messageIdFilter.length; i++) {
                if (msg.getMessageId() == messageIdFilter[i]) {
                    return msg;
                }
            }

            // Not one of the filtered messages so process it through
            // the standard handlers
            onMessageReceived(msg);
        }

    }



    /**
     *  <p>
     *
     *  Provides the message loop.</p>
     */
    public final void run() {

        while (keepRunning) {

            log.info(serviceName + " is waiting for new messages");
            // Get the next message from the message store
            SshMessage msg = messageStore.getMessage();

            try {

                onMessageReceived(msg);
            } catch (TransportProtocolException tpe) {
                onThreadException(tpe);
                keepRunning = false;
            } catch (ServiceOperationException soe) {
                onThreadException(soe);
                keepRunning = false;
            }

        }

    }


    /**
     *  Stops the message loop and causes the thread to exit
     */
    public final void stop() {

        // Let the message loop know we want to stop
        keepRunning = false;

        // Interrupt the thread if its waiting for new messages
        thread.interrupt();

        // The thread should now exit
        log.info(serviceName + " thread is exiting");
    }


    /**
     *  <p>
     *
     *  Returns the name of the service.</p>
     *
     *@return    The serviceName value
     */
    public final String getServiceName() {
        return serviceName;
    }


    /**
     *  <p>
     *
     *  Abstract method called when a registered message ahs been received.</p>
     *
     *@param  msg                             The message received
     *@exception  ServiceOperationException   Subclasses can throw this
     *      exception
     *@exception  TransportProtocolException  Subclasses can throw this
     *      exception
     */
    protected abstract void onMessageReceived(SshMessage msg) throws ServiceOperationException, TransportProtocolException;

}
