/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.util.*;
import com.sshtools.j2ssh.authentication.AuthenticationProtocol;


/**
 *  <p>
 *
 *  The SshServiceFactory provides a mechanism so that the transport layer can
 *  create an instance of the requested service upon reciept of a service accept
 *  message.</p> <p>
 *
 *  This factory currently only supports one service "ssh-userauth". This
 *  service is usually requested to authenticate the user and to start a further
 *  service. The AuthenticaitonProtocol provides a means to start any service
 *  after authentication has completed.</p> <p>
 *
 *  NOTE: We need to be able to dynamically load additional services so that
 *  developers can implement non authenticated services/</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class ServiceFactory {

    /**
     *  <p>
     *
     *  The current services available to create.</p>
     */
    private static Map services = new HashMap();

    static {

        services.put("ssh-userauth", AuthenticationProtocol.class.getName());

    }


    /**
     *  <p>
     *
     *  Returns a new instance of the service specified.</p>
     *
     *@param  serviceName  The name of the service
     *@return              The Service instance
     */
    public final static Service getInstance(String serviceName) {

        try {
            String className = (String) services.get(serviceName);

            return (Service) Class.forName(className).newInstance();
        } catch (Exception e) {
            return null;
        }

    }


    /**
     *  <p>
     *
     *  Returns true if the service is supported.</p>
     *
     *@param  serviceName  The service name to query.
     *@return
     */
    public final static boolean supports(String serviceName) {

        return services.containsKey(serviceName);
    }

}
