/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;
import com.sshtools.j2ssh.transport.InvalidStateException;

/**
 *  <p>
 *
 *  When a service is requested the transport layer sends an
 *  SSH_MSG_SERVICE_REQUEST message to the remote computer. This state object is
 *  used to synchronize with the input stream thread so that the transport layer
 *  can synchronously return either a new instance of the requested service or
 *  deny the service request. The object is additionally used when starting the
 *  service to indicate whether in was instansiated due to a service request
 *  (i.e is in server mode) or by the completion of a service request and the
 *  subsequent receipt of a service accept.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class ServiceState {

    /**
     *  The service has been requested
     */
    public final static int SERVICE_REQUESTED = 1;

    /**
     *  The service has been accepted
     */
    public final static int SERVICE_ACCEPTED = 2;

    /**
     *  The internal state
     */
    private int state;


    /**
     *  <p>
     *
     *  Constructor for the SshServiceState object, defaults the state to
     *  service requested.</p>
     */
    public ServiceState() {

        state = SERVICE_REQUESTED;
    }


    /**
     *  <p>
     *
     *  Sets the state of the service and notifys any waiting threads.</p>
     *
     *@param  state                      The new state value
     *@exception  InvalidStateException  Thrown if an attempt is made to set the
     *      state to anything other than the public static values of this class
     */
    public final synchronized void setState(int state) throws InvalidStateException {

        if (state != SERVICE_REQUESTED && state != SERVICE_ACCEPTED) {
            throw new InvalidStateException("The state specified is not valid");
        }
        this.state = state;

        notifyAll();
    }


    /**
     *  <p>
     *
     *  Gets the current state of the service.</p>
     *
     *@return    The state value
     */
    public synchronized int getState() {
        return state;
    }


    /**
     *  <p>
     *
     *  Synchronized method that will wait on the state untill the state has
     *  changed from SERVICE_REQUEST to SERVICE_ACCEPT.<p>
     *
     *
     */
    public synchronized void waitForServiceAccept() {

        if (state == SERVICE_ACCEPTED) {
            return;
        }

        while (state != SERVICE_ACCEPTED) {

            try {
                wait();
            } catch (InterruptedException e) {

            }
        }

    }

}
