/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.IOException;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;
/**
 *  <p>
 *
 *  This abstract class should be subclassed to provide message implementations
 *  for the SSH2 protocol. The message object can be constructed either from a
 *  byte array or implementation specific types. The transport layer dynamically
 *  creates the class from the registered message details and calls <code>
 *  fromByteArray</code> to populate before routing the message to the
 *  registered message store.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public abstract class SshMessage {

    // The message Id of the message
    private int messageId;


    /**
     *  <p>
     *
     *  Default constructor for dynamic creation of a message from message data.
     *  </p>
     *
     *@param  data                         Description of the Parameter
     *@exception  InvalidMessageException  Description of the Exception
     */
    /*
     *  public SshMessage() { }
     */
    /**
     *  <p>
     *
     *  Constructs an SshMessage from a byte array. This method calls the
     *  abstract method constructMessage to allow subclasses classes to perform
     *  message specific construction.</p>
     *
     *@param  data                      The data being read.
     *@throws  InvalidMessageException  Thrown when the data cannot be read.
     */
    protected final void fromByteArray(ByteArrayReader data) throws InvalidMessageException {

        // Skip the first 5 bytes as this contains the packet length and payload
        // length fields
        data.skip(5);

        int id = data.read();

        if (id != messageId) {
            throw new InvalidMessageException("The message id " + String.valueOf(id) +
                    " is not the same as the message implementation id " + String.valueOf(messageId));
        }

        // Call abstract method for subclasses to extract the message specific data
        constructMessage(data);

    }


    /**
     *  <p>
     *
     *  Constructs an SshMessage used to send. Subclasses classes will set their
     *  own properties typically passed to its own constructor.</p>
     *
     *@param  messageId  The message Id of the message.
     */
    public SshMessage(int messageId) {

        // Save the message id
        this.messageId = messageId;

    }


    /**
     *@return    The message Id.
     */
    public final int getMessageId() {
        return messageId;
    }


    /**
     *  Gets the messageName attribute of the SshMessage object
     *
     *@return    The messageName value
     */
    public abstract String getMessageName();


    /**
     *  <p>
     *
     *  Converts the message into a byte array suitable for sending.</p>
     *
     *@return                              A byte array containing the message
     *      data.
     *@exception  InvalidMessageException  Thrown when the byte array cannot be
     *      written.
     */
    public final byte[] toByteArray() throws InvalidMessageException {

        // Create a writer object to construct the array
        ByteArrayWriter baw = new ByteArrayWriter();

        // Write the message id
        baw.write(messageId);

        // Call the abstract method so subclasses classes can add their data
        constructByteArray(baw);

        // Return the array
        return baw.toByteArray();
    }


    /**
     *  <p>
     *
     *  Abstract method which is called to construct the byte array returned in
     *  a call to toByteArray.</p>
     *
     *@param  baw                          The ByteArrayWriter instance
     *      constructing the array.
     *@exception  InvalidMessageException  Thrown when the array cannot be
     *      written.
     */
    protected abstract void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException;


    /**
     *  <p>
     *
     *  Abstract method which is called to construct the message from a byte
     *  array.</p>
     *
     *@param  bar                          The ByteArrayReader containing the
     *      message data.
     *@exception  InvalidMessageException  Thrown when the message cannot be
     *      constructed either because the data supplied is corrupt or there are
     *      insufficient bytes.
     */
    protected abstract void constructMessage(ByteArrayReader bar) throws InvalidMessageException;
}
