/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.util.*;
import com.sshtools.j2ssh.transport.SshMessage;


/**
 *  <p>
 *
 *  The message store allows the transport protocol to provide asynchronous
 *  messaging. When an object (such as Service) wishes to receive messages,
 *  it creates an instance of the message store and
 *  calls the TransportProtocol interface method registerMessage passing the
 *  message Id of the desired message and the message store object that will be
 *  called.</p> <p>
 *
 *  When a message is received the transport layer looks up the message store
 *  object for the incoming message Id and adds the message to it. Any waiting
 *  threads are notified so that the message can be handled appropriatley.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    26 August 2002
 *@version    1.0
 */

public final class SshMessageStore {

    // List to hold messages as they are received
    private List messages = new ArrayList();


    /**
     *  <p>
     *
     *  Constructs the message store object.</p>
     */
    public SshMessageStore() { }


    /**
     *  <p>
     *
     *  Adds a message to the message store and notify the waiting threads.</p>
     *
     *@param  msg  The message to add.
     */

    public synchronized void addMessage(SshMessage msg) {

        // Add the message
        messages.add(msg);

        // Notify the threads
        notifyAll();

    }


    /**
     *  <p>
     *  Get the next message available from the message store. If there are no
     *  messages available, the method will wait untill notified of a new
     *  message.</p>
     *
     *@return    The next available message.
     */
    public synchronized SshMessage getMessage() {

        // If there are no messages available then wait untill there are.
        while (messages.size() <= 0) {
            try {
                wait();
            } catch (InterruptedException e) {

            }
        }
        SshMessage msg = (SshMessage) messages.get(0);

        // Remove the message from the list
        messages.remove(0);

        // Return it
        return msg;
    }

}

