/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.IOException;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the transport protocol debug message.</p> <b> <p>
 *
 *  byte SSH_MSG_DEBUG</p> <p>
 *
 *  boolean always_display</p> <p>
 *
 *  string message [RFC2279]</p> <p>
 *
 *  string language tag [RFC1766]</p> </b> <p>
 *
 *  All implementations MUST understand this message, but they are allowed to
 *  ignore it. This message is used to pass the other side information that may
 *  help debugging. If always_display is TRUE, the message SHOULD be displayed.
 *  Otherwise, it SHOULD NOT be displayed unless debugging information has been
 *  explicitly requested by the user.</p> <p>
 *
 *  The message doesn't need to contain a newline. It is, however, allowed to
 *  consist of multiple lines separated by CRLF (Carriage Return - Line Feed)
 *  pairs.</p> <p>
 *
 *  If the message string is displayed, terminal control character filtering
 *  discussed in [SSH-ARCH] should be used to avoid attacks by sending terminal
 *  control characters.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class SshMsgDebug extends SshMessage {

    // Holds the always_display value
    private boolean alwaysDisplay;
    // Holds the message value
    private String message;
    // Holds the language_tag value
    private String langTag;

    /**
     *  The message id of this message
     */
    protected final static int SSH_MSG_DEBUG = 4;


    /**
     *  <p>
     *
     *  Constructs the message ready for sending.</p>
     *
     *@param  alwaysDisplay  Should the debug message be displayed?
     *@param  message        The message to be displayed.
     *@param  langTag        The language tag of the message.
     */
    public SshMsgDebug(boolean alwaysDisplay, String message, String langTag) {

        super(SSH_MSG_DEBUG);

        // Save the debug details
        this.alwaysDisplay = alwaysDisplay;
        this.message = message;
        this.langTag = langTag;

    }


    /**
     *  Gets the messageName attribute of the SshMsgDebug object
     *
     *@return    The messageName value
     */
    public String getMessageName() {
        return "SSH_MSG_DEBUG";
    }


    /**
     *  <p>
     *
     *  Default constructor.</p>
     */
    public SshMsgDebug() {
        super(SSH_MSG_DEBUG);
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to extract the message data from a byte
     *  array.</p>
     *
     *@param  bar                          The byte array to extract the message
     *      from.
     *@exception  InvalidMessageException  Thrown when the message cannot be
     *      constructed either because the data supplied is corrupt or there are
     *      insufficient bytes.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            // Extract the message information
            alwaysDisplay = (bar.read() == 0) ? false : true;
            message = bar.readString();
            langTag = bar.readString();

        } catch (IOException ioe) {
            throw new InvalidMessageException("Error reading message data: " + ioe.getMessage());
        }
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to construct a byte array suitable for
     *  sending.</p>
     *
     *@param  baw                          The byte array to store the message.
     *@exception  InvalidMessageException  Thrown when the array cannot be
     *      written to.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {

            // Write the data
            baw.write(alwaysDisplay ? 1 : 0);
            baw.writeString(message);
            baw.writeString(langTag);

        } catch (IOException ioe) {
            throw new InvalidMessageException("Error writing message data: " + ioe.getMessage());
        }
    }


    /**
     *  <p>
     *
     *  Gets the message attribute of the message.</p>
     *
     *@return    The debug message.
     */
    public String getMessage() {
        return message;
    }


    /**
     *  <p>
     *
     *  Gets the language tag attribute of the message.</p>
     *
     *@return    The language tag.
     */
    public String getLanguageTag() {
        return langTag;
    }


    /**
     *  <p>
     *
     *  Gets the display always attribute of the message.</p>
     *
     *@return    The display always value
     */
    public boolean getDisplayAlways() {
        return alwaysDisplay;
    }
}
