/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.IOException;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the disconnection Message.</p> <b><p>
 *
 *  byte SSH_MSG_DISCONNECT<br>
 *  uint32 reason code<br>
 *  string description [RFC2279]<br>
 *  string language tag [RFC1766]<br>
 *  </p> </b> <p>
 *
 *  This message causes immediate termination of the connection. All
 *  implementations MUST be able to process this message; they SHOULD be able to
 *  send this message.</p> <p>
 *
 *  The sender MUST NOT send or receive any data after this message, and the
 *  recipient MUST NOT accept any data after receiving this message. The
 *  description field gives a more specific explanation in a human- readable
 *  form. The error code gives the reason in a more machine- readable format
 *  (suitable for localization), and can have the following values:</p> <p>
 *
 *  #define SSH_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT 1<br>
 *  #define SSH_DISCONNECT_PROTOCOL_ERROR 2<br>
 *  #define SSH_DISCONNECT_KEY_EXCHANGE_FAILED 3<br>
 *  #define SSH_DISCONNECT_RESERVED 4<br>
 *  #define SSH_DISCONNECT_MAC_ERROR 5<br>
 *  #define SSH_DISCONNECT_COMPRESSION_ERROR 6<br>
 *  #define SSH_DISCONNECT_SERVICE_NOT_AVAILABLE 7<br>
 *  #define SSH_DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED 8<br>
 *  #define SSH_DISCONNECT_HOST_KEY_NOT_VERIFIABLE 9<br>
 *  #define SSH_DISCONNECT_CONNECTION_LOST 10<br>
 *  #define SSH_DISCONNECT_BY_APPLICATION 11<br>
 *  #define SSH_DISCONNECT_TOO_MANY_CONNECTIONS 12<br>
 *  #define SSH_DISCONNECT_AUTH_CANCELLED_BY_USER 13<br>
 *  #define SSH_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE 14<br>
 *  #define SSH_DISCONNECT_ILLEGAL_USER_NAME 15</p> <p>
 *
 *  If the description string is displayed, control character filtering
 *  discussed in [SSH-ARCH] should be used to avoid attacks by sending terminal
 *  control characters.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */
class SshMsgDisconnect extends SshMessage {

    /**
     *  The message id of this message
     */
    protected final static int SSH_MSG_DISCONNECT = 1;
    /**
     *  Host not allowed reason code
     */
    public final static int HOST_NOT_ALLOWED = 1;
    /**
     *  Protocol error reason code
     */
    public final static int PROTOCOL_ERROR = 2;
    /**
     *  Key exchange failed reason code
     */
    public final static int KEY_EXCHANGE_FAILED = 3;
    /**
     *  Reserved reason code
     */
    public final static int RESERVED = 4;
    /**
     *  Mac error reason code
     */
    public final static int MAC_ERROR = 5;
    /**
     *  Compression error reason code
     */
    public final static int COMPRESSION_ERROR = 6;
    /**
     *  Service not available reason code
     */
    public final static int SERVICE_NOT_AVAILABLE = 7;
    /**
     *  Protocol version not supported reason code
     */
    public final static int PROTOCOL_VERSION_NOT_SUPPORTED = 8;
    /**
     *  Host key not verifiable reason code
     */
    public final static int HOST_KEY_NOT_VERIFIABLE = 9;
    /**
     *  Connection lost reason code
     */
    public final static int CONNECTION_LOST = 10;
    /**
     *  The application disconnected reason code
     */
    public final static int BY_APPLICATION = 11;
    /**
     *  Too many connections reason code
     */
    public final static int TOO_MANY_CONNECTIONS = 12;
    /**
     *  Authentication cancelled by user reason code
     */
    public final static int AUTH_CANCELLED_BY_USER = 13;
    /**
     *  No more authentication methods available reason code
     */
    public final static int NO_MORE_AUTH_METHODS_AVAILABLE = 14;
    /**
     *  The user is illegal reason code
     */
    public final static int ILLEGAL_USER_NAME = 15;

    // Holds the reason for disconnection
    private int reasonCode;
    // The readble version of the disconneciton reason
    private String desc;
    // The language tag
    private String langTag;


    /**
     *  <p>
     *
     *  Constructs the disconnect message for sending.</p>
     *
     *@param  reasonCode  The reason code for disconnection
     *@param  desc        A readable version of the disconnection reason
     *@param  langTag     The language tag for the description
     */
    public SshMsgDisconnect(int reasonCode, String desc, String langTag) {
        super(SSH_MSG_DISCONNECT);

        // Store the message values
        this.reasonCode = reasonCode;
        this.desc = desc;
        this.langTag = langTag;
    }


    /**
     *  <p>
     *
     *  Default constructor</p>
     */
    public SshMsgDisconnect() {
        super(SSH_MSG_DISCONNECT);
    }


    /**
     *  Gets the messageName attribute of the SshMsgDisconnect object
     *
     *@return    The messageName value
     */
    public String getMessageName() {
        return "SSH_MSG_DISCONNECT";
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to construct the message from a byte
     *  array.</p>
     *
     *@param  bar                          The message data received.
     *@exception  InvalidMessageException  Thrown when the data cannot be read.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            // Save the values
            reasonCode = (int) bar.readInt();
            desc = bar.readString();
            langTag = bar.readString();

        } catch (IOException ioe) {
            throw new InvalidMessageException("Error reading message data: " + ioe.getMessage());
        }
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to construct a byte array containing the
     *  message data.</p>
     *
     *@param  baw                          The byte array storing the message
     *      data.
     *@exception  InvalidMessageException  Thrown when the data cannot be
     *      written.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {

            baw.writeInt(reasonCode);
            baw.writeString(desc);
            baw.writeString(langTag);

        } catch (IOException ioe) {
            throw new InvalidMessageException("Error writing message data: " + ioe.getMessage());
        }

    }


    /**
     *  Gets the reason code attribute of the message
     *
     *@return    The reason code
     */
    public int getReasonCode() {
        return reasonCode;
    }


    /**
     *  Gets the description attribute of message
     *
     *@return    The description
     */
    public String getDescription() {
        return desc;
    }


    /**
     *  Gets the language tag attribute of the message
     *
     *@return    The language tag
     */
    public String getLanguageTag() {
        return langTag;
    }
}
