/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.IOException;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the Ignored Data Message.</p> <b> <p>
 *
 *  byte SSH_MSG_IGNORE<br>
 *  string data</p> </b> <p>
 *
 *  All implementations MUST understand (and ignore) this message at any time
 *  (after receiving the protocol version). No implementation is required to
 *  send them. This message can be used as an additional protection measure
 *  against advanced traffic analysis techniques.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class SshMsgIgnore extends SshMessage {

    private String data;

    /**
     *  The message id for the message
     */
    protected final static int SSH_MSG_IGNORE = 2;


    /**
     *  <p>
     *
     *  Constructs the message ready for sending.</p>
     *
     *@param  data  A string of random data.
     */
    public SshMsgIgnore(String data) {
        super(SSH_MSG_IGNORE);
        this.data = data;
    }


    /**
     *  Gets the messageName attribute of the SshMsgIgnore object
     *
     *@return    The messageName value
     */
    public String getMessageName() {
        return "SSH_MSG_IGNORE";
    }


    /**
     *  <p>
     *
     *  Default constructor</p>
     */
    public SshMsgIgnore() {
        super(SSH_MSG_IGNORE);
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to constrcut the message from a byte
     *  array.</p>
     *
     *@param  bar                          The byte array containing the message
     *      data.
     *@exception  InvalidMessageException  Thrown when the data cannot be read.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {
            data = bar.readString();

        } catch (IOException ioe) {

            throw new InvalidMessageException("Error occurred reading message data: " + ioe.getMessage());
        }
    }


    /**
     *  <p>
     *
     *  Abstract method implementation to construct a byte array containing the
     *  message data.</p>
     *
     *@param  baw                          The byte array to store the message.
     *@exception  InvalidMessageException  Thrown when the data cannot be
     *      written.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {
            baw.writeString(data);
        } catch (IOException ioe) {
            throw new InvalidMessageException("Error occurred writing message data: " + ioe.getMessage());
        }
    }


    /**
     *  Gets the data attribute of the ignore message
     *
     *@return    The data value.
     */
    public String getData() {
        return data;
    }

}
