/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.IOException;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the service accept message.</p> <p>
 *
 *  If the server supports the service (and permits the client to use it), it
 *  MUST respond with the following:</p> <b><p>
 *
 *  byte SSH_MSG_SERVICE_ACCEPT<br>
 *  string service name</p> </b>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class SshMsgServiceAccept extends SshMessage {

    private String serviceName;

    /**
     *  The message id of the message.
     */
    protected final static int SSH_MSG_SERVICE_ACCEPT = 6;


    /**
     *  Constructs the message ready for sending.
     *
     *@param  serviceName  The service name that has been accepted.
     */
    public SshMsgServiceAccept(String serviceName) {
        super(SSH_MSG_SERVICE_ACCEPT);
        this.serviceName = serviceName;
    }


    /**
     *  Constructs the message from data received.
     */
    public SshMsgServiceAccept() {
        super(SSH_MSG_SERVICE_ACCEPT);
    }


    /**
     *  Gets the messageName attribute of the SshMsgServiceAccept object
     *
     *@return    The messageName value
     */
    public String getMessageName() {
        return "SSH_MSG_SERVICE_ACCEPT";
    }


    /**
     *  Abstract method implementation to construct the message from data
     *  received.
     *
     *@param  bar                          The data to be read.
     *@exception  InvalidMessageException  Thrown when the data cannot be read.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            serviceName = bar.readString();
        } catch (IOException ioe) {
            throw new InvalidMessageException("Error reading message data");
        }
    }


    /**
     *  Abstract method implementation to construct a byte array.
     *
     *@param  baw                          The byte array being written.
     *@exception  InvalidMessageException  Thrown when the data cannot be
     *      written.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {
        try {

            baw.writeString(serviceName);
        } catch (IOException ioe) {
            throw new InvalidMessageException("Error writing message data");
        }

    }


    /**
     *  Gets the service name that has been accepted.
     *
     *@return    The service name
     */
    public String getServiceName() {
        return serviceName;
    }
}
