/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.IOException;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  This class implements the service request message.</p> <p>
 *
 *  After the key exchange, the client requests a service. The service is
 *  identified by a name. The format of names and procedures for defining new
 *  names are defined in [SSH-ARCH].</p> <p>
 *
 *  Currently, the following names have been reserved:</p> <p>
 *
 *  ssh-userauth<br>
 *  ssh-connection</p> <p>
 *
 *  Similar local naming policy is applied to the service names, as is applied
 *  to the algorithm names; a local service should use the "servicename@domain"
 *  syntax.</p> <p>
 *
 *  <b>byte SSH_MSG_SERVICE_REQUEST<br>
 *  string service name</p> </b> <p>
 *
 *  If the server rejects the service request, it SHOULD send an appropriate
 *  SSH_MSG_DISCONNECT message and MUST disconnect.</p> <p>
 *
 *  When the service starts, it may have access to the session identifier
 *  generated during the key exchange.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class SshMsgServiceRequest extends SshMessage {

    private String serviceName;

    /**
     *  The message id of the message
     */
    protected final static int SSH_MSG_SERVICE_REQUEST = 5;


    /**
     *  Constructs the message ready for sending
     *
     *@param  serviceName  The service requested
     */
    public SshMsgServiceRequest(String serviceName) {
        super(SSH_MSG_SERVICE_REQUEST);
        this.serviceName = serviceName;
    }


    /**
     *  Gets the messageName attribute of the SshMsgServiceRequest object
     *
     *@return    The messageName value
     */
    public String getMessageName() {
        return "SSH_MSG_SERVICE_REQUEST";
    }


    /**
     *  Constructs the message from data received.
     */
    public SshMsgServiceRequest() {
        super(SSH_MSG_SERVICE_REQUEST);
    }


    /**
     *  Abstract method implementation to construct the message from a byte
     *  array.
     *
     *@param  bar                          The data being read.
     *@exception  InvalidMessageException  Thrown when the data cannot be read.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            serviceName = bar.readString();

        } catch (IOException ioe) {
            throw new InvalidMessageException("Error reading message data");
        }
    }


    /**
     *  Abstract method implementation to create a byte array containing the
     *  message.
     *
     *@param  baw                          The byte array being written.
     *@exception  InvalidMessageException  Thrown when the data cannot be read.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {

            baw.writeString(serviceName);

        } catch (IOException ioe) {
            throw new InvalidMessageException("Error writing message data");
        }
    }


    /**
     *  Gets the service name requested
     *
     *@return    The service name.
     */
    public String getServiceName() {
        return serviceName;
    }
}
