/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;
import java.io.*;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  An implementation MUST respond to all unrecognized messages with an
 *  SSH_MSG_UNIMPLEMENTED message in the order in which the messages were
 *  received. Such messages MUST be otherwise ignored. Later protocol versions
 *  may define other meanings for these message types.</p> <b><p>
 *
 *  byte SSH_MSG_UNIMPLEMENTED<br>
 *  uint32 packet sequence number of rejected message</p> </b>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class SshMsgUnimplemented extends SshMessage {

    /**
     *  The message id of the message
     */
    protected final static int SSH_MSG_UNIMPLEMENTED = 3;

    // The sequence no of the message
    private long sequenceNo;


    /**
     *  Constructs the message ready for sending
     *
     *@param  sequenceNo  The sequence no of the unimplemented message.
     */
    public SshMsgUnimplemented(long sequenceNo) {
        super(SSH_MSG_UNIMPLEMENTED);
        this.sequenceNo = sequenceNo;
    }


    /**
     *  Gets the messageName attribute of the SshMsgUnimplemented object
     *
     *@return    The messageName value
     */
    public String getMessageName() {
        return "SSH_MSG_UNIMPLEMENTED";
    }


    /**
     *  Constructs the message from data received.
     */
    public SshMsgUnimplemented() {
        super(SSH_MSG_UNIMPLEMENTED);
    }


    /**
     *  Abstract method implementation to construct the message from received
     *  data.
     *
     *@param  bar                          The data being read.
     *@exception  InvalidMessageException  Thrown when the data cannot be read.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {
            sequenceNo = bar.readInt();
        } catch (IOException ioe) {
            throw new InvalidMessageException("Error contructing SSH_MSG_UNIMPLEMENTED, expected int value");
        }

    }


    /**
     *  Abstract method implementation to construct a byte array containing the
     *  data
     *
     *@param  baw                          The byte array being written.
     *@exception  InvalidMessageException  Thrown when the data cannot be
     *      written.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {
            baw.writeInt(sequenceNo);
        } catch (IOException ioe) {
            throw new InvalidMessageException("Error extracting SSH_MSG_UNIMPLMENTED, expected int value");
        }
    }


    /**
     *  Gets the sequence number of the unimplemented message
     *
     *@return    The sequence number
     */
    public long getSequenceNo() {
        return sequenceNo;
    }

}
