/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;
import com.sshtools.j2ssh.transport.MessageAlreadyRegisteredException;
import com.sshtools.j2ssh.transport.MessageNotRegisteredException;
import com.sshtools.j2ssh.transport.Service;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.j2ssh.transport.SshMessageStore;

/**
 *  <p>
 *
 *  This interface provides external access to messaging services supplied by
 *  the SSH API.</p> <p>
 *
 *  When a party wants to receive an incoming message it simply registers its
 *  interest with the transport protocol through registerMessage(). Messages are
 *  then routed to the synchronized message store object supplied during
 *  registration where any waiting threads are notified. This framework for
 *  message routing is implemented by the abstract Service class.</p> <p>
 *
 *  Messages should be unregistered by services especially if the service is
 *  temporary, such as Authenticaiton services. Messages are sent through the
 *  transport layer by using sendMessage(). This requires an additional
 *  identification object (typically passed as 'this') so that the transport
 *  layer can filter messages whilst in key exchange. (only transport protocol
 *  messages are valid during the key exchange process.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public interface TransportProtocol {

    /**
     *  Registers a message id with the transport layer
     *
     *@param  messageId                              The id of the message being
     *      registered
     *@param  implementor                            The class of the message
     *      implementation.
     *@param  store                                  The message store to route
     *      the object to
     *@exception  MessageAlreadyRegisteredException  Thrown if the message id is
     *      already registered
     */
    public void registerMessage(Integer messageId, Class implementor, SshMessageStore store)
             throws MessageAlreadyRegisteredException;


    /**
     *  Unregisters a message from the transport layer
     *
     *@param  messageId                          The id of the message to
     *      unregister
     *@param  store                              The message store currently
     *      receiving notification
     *@exception  MessageNotRegisteredException  Thrown if the message is not
     *      registered
     */
    public void unregisterMessage(Integer messageId, SshMessageStore store)
             throws MessageNotRegisteredException;


    /**
     *  Sends an Message
     *
     *@param  ms                              The message to send
     *@param  sender                          The sender of the message
     *@exception  TransportProtocolException  Thrown if an error occurs
     */
    public void sendMessage(SshMessage ms, Object sender) throws TransportProtocolException;


    /**
     *  Closes the connection
     *
     *@param  description  A description of why the disconnect is occuring
     */
    public void disconnect(String description);


    /**
     *  Sets the serviceThread attribute of the TransportProtocol object
     *
     *@param  service  The new serviceThread value
     */
    public void setServiceThread(Service service);


    /**
     *  Description of the Method
     *
     *@param  service  Description of the Parameter
     */
    public void removeServiceThread(Service service);

}
