/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.net.Socket;
import java.math.BigInteger;
import java.util.*;
import com.sshtools.j2ssh.configuration.SshConnectionProperties;
import com.sshtools.j2ssh.SshException;
import com.sshtools.j2ssh.transport.kex.KeyExchangeException;
import com.sshtools.j2ssh.transport.kex.KeyExchangeState;
import com.sshtools.j2ssh.transport.publickey.InvalidHostKeyException;
import com.sshtools.j2ssh.transport.publickey.InvalidHostKeySignatureException;
import com.sshtools.j2ssh.transport.publickey.SshPublicKeyFactory;
import com.sshtools.j2ssh.transport.kex.SshKeyExchange;

/**
 *  <p>
 *
 *  Implements the client side of the SSH transport protocol. Specifically this
 *  class initiates client side key exchange operations and provides a mechanism
 *  to request SSH services from the remote server.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class TransportProtocolClient extends TransportProtocolCommon {

    private Map services = new HashMap();


    /**
     *  Constructor for the SshTransportProtocolClient object
     *
     *@param  socket                          The connected socket.
     *@param  properties                      The SSH connection properties for
     *      the current connection
     *@param  hosts                           Description of the Parameter
     *@exception  TransportProtocolException  Description of the Exception
     */
    public TransportProtocolClient(Socket socket, SshConnectionProperties properties, HostKeyVerification hosts)
             throws TransportProtocolException {

        super(socket, properties, hosts);

        try {
            // The client can request services so we need to register
            registerMessage(new Integer(SshMsgServiceAccept.SSH_MSG_SERVICE_ACCEPT), SshMsgServiceAccept.class, messageStore);

        } catch (MessageAlreadyRegisteredException mare) {
            throw new TransportProtocolException("Required messages were already registered to another party!");
        }
    }


    /**
     *  Sets the local sides identification string. This is sent initially to
     *  determine the protocol version. As the common layer cannot determine
     *  whether it is running as a client or server, this method allows the
     *  subclasses to correctly set the relevant protected member variable.
     */
    protected void setLocalIdent() {
        clientIdent = "SSH-" + PROTOCOL_VERSION + "-" + SOFTWARE_VERSION_COMMENTS;
    }


    /**
     *  Sets the remote sides identification string.
     *
     *@param  ident  The protocol version string received
     */
    protected void setRemoteIdent(String ident) {
        serverIdent = ident;
    }


    /**
     *  Returns the local protocol identification string. We return the relevant
     *  protected member variable for the subclass implementation.
     *
     *@return    The local identification string
     */
    protected String getLocalIdent() {
        return clientIdent;
    }


    /**
     *  Returns the remote computers protocol identification string.
     *
     *@return    The remote identificaiton string
     */
    protected String getRemoteIdent() {
        return serverIdent;
    }


    /**
     *  Called by the abstract super class to set the relevant kex init member
     *  variable according to the subclasses operation (client or server)
     *
     *@param  msg  The kex init message sent by the local machine
     */
    protected void setLocalKexInit(SshMsgKexInit msg) {
        log.debug(msg.toString());
        clientKexInit = msg;
    }


    /**
     *  Called by the abstract super class to set the remote sides kex init
     *  message.
     *
     *@param  msg  The kex init message received from the remote computer.
     */
    protected void setRemoteKexInit(SshMsgKexInit msg) {
        serverKexInit = msg;
    }


    /**
     *  Gets the local sides kex init message.
     *
     *@return    The local sides kex init message
     */
    protected SshMsgKexInit getLocalKexInit() {
        return clientKexInit;
    }


    /**
     *  Gets the remote computers kex init message
     *
     *@return    The remote computers kex init
     */
    protected SshMsgKexInit getRemoteKexInit() {
        return serverKexInit;
    }


    /**
     *  Returns the encryption algorithm to be used after key exchange. This
     *  method evaluates the supported algorithms of both sides and determines
     *  the correct algorithm.
     *
     *@return                                  The algortihm name
     *@exception  AlgorithmNotAgreedException  Thrown if an algorithm cannot be
     *      agreed
     */
    protected String getEncryptionAlgorithm()
             throws AlgorithmNotAgreedException {

        return determineAlgorithm(clientKexInit.getSupportedCSEncryption(),
                serverKexInit.getSupportedCSEncryption());
    }


    /**
     *  Returns the decryption algorithm to be used after key exchange. This
     *  method evaluates the supported algorithms of both sides and determines
     *  the correct algorithm.
     *
     *@return                                  The algorithm name.
     *@exception  AlgorithmNotAgreedException  Thrown if the algorithm cannot be
     *      agreed.
     */
    protected String getDecryptionAlgorithm()
             throws AlgorithmNotAgreedException {

        return determineAlgorithm(clientKexInit.getSupportedSCEncryption(),
                serverKexInit.getSupportedSCEncryption());
    }



    /**
     *  Returns the output stream MAC algorithm to be used after key exchange.
     *  This method evaluates the supported algorithms of both sides and
     *  determines the correct algorithm.
     *
     *@return                                  The algorithm name.
     *@exception  AlgorithmNotAgreedException  Thrown if an algorithm cannot be
     *      agreed.
     */
    protected String getOutputStreamMacAlgorithm()
             throws AlgorithmNotAgreedException {

        return determineAlgorithm(clientKexInit.getSupportedCSMac(),
                serverKexInit.getSupportedCSMac());
    }


    /**
     *  Returns the input stream MAC algorithm to be used after key exchange.
     *  This method evaluates the supported algorithms of both sides and
     *  determines the correct algorithm.
     *
     *@return                                  The algorithm name.
     *@exception  AlgorithmNotAgreedException  Thrown if an algorithm cannot be
     *      agreed.
     */
    protected String getInputStreamMacAlgorithm()
             throws AlgorithmNotAgreedException {

        return determineAlgorithm(clientKexInit.getSupportedSCMac(),
                serverKexInit.getSupportedSCMac());
    }


    /**
     *  Returns the output stream compression algorithm to be used after key
     *  exchange. This method evaluates the supported algorithms of both sides
     *  and determines the correct algorithm.
     *
     *@return                                  The algorithm name.
     *@exception  AlgorithmNotAgreedException  Thrown if an algorithm cannot be
     *      agreed.
     */
    protected String getOutputStreamCompAlgorithm()
             throws AlgorithmNotAgreedException {

        return determineAlgorithm(clientKexInit.getSupportedCSComp(),
                serverKexInit.getSupportedCSComp());
    }


    /**
     *  Returns the input stream compression algorithm to be used after key
     *  exchange. This method evaluates the supported algorithms of both sides
     *  and determines the correct algorithm.
     *
     *@return                                  The algorithm name.
     *@exception  AlgorithmNotAgreedException  Thrown if an algorithm cannot be
     *      agreed.
     */
    protected String getInputStreamCompAlgortihm()
             throws AlgorithmNotAgreedException {

        return determineAlgorithm(clientKexInit.getSupportedSCComp(),
                serverKexInit.getSupportedSCComp());
    }


    /**
     *  Requests that the remote computer start the specified service.
     *
     *@param  serviceName                     The service name to start
     *@return                                 The service instance.
     *@exception  TransportProtocolException  Thrown if a transport protocol
     *      exception occurs
     *@exception  ServiceOperationException   Thrown if the service does not
     *      initialize properly.
     */
    public Service requestService(String serviceName)
             throws TransportProtocolException, ServiceOperationException {

        // Make sure the service is supported
        if (!ServiceFactory.supports(serviceName)) {
            throw new ServiceOperationException("The service is not supported");
        }

        // Create a state object
        ServiceState state = new ServiceState();

        // Put the service on our list awaiting acceptance
        services.put(serviceName, state);

        // Create the message
        SshMsgServiceRequest msg = new SshMsgServiceRequest(serviceName);

        // Send the message on behalf of the service but only if were ready
        getState().waitForReadyState();

        sendMessage(msg, this);

        // Wait for the state of the service to be returned
        // If the service is denied the remote end will disconnect
        state.waitForServiceAccept();
        Service service = ServiceFactory.getInstance(serviceName);

        // Initialize the service
        service.start(this, state, exchangeHash);

        return service;
    }


    /**
     *  Verifies the server host key using the public key algorithm negotiated
     *  during key exchange.
     *
     *@return                                       The result of the
     *      verification
     *@exception  InvalidHostKeyException           Description of the Exception
     *@exception  InvalidHostKeySignatureException  Description of the Exception
     *@exception  AlgorithmNotAgreedException       Description of the Exception
     *@exception  AlgorithmNotSupportedException    Description of the Exception
     *@exception  InvalidHostFileException          Description of the Exception
     */
    protected boolean verifyHostKey() throws InvalidHostKeyException,
            InvalidHostKeySignatureException,
            AlgorithmNotAgreedException,
            AlgorithmNotSupportedException,
            InvalidHostFileException {

        // Determine the public key algorithm and obtain an instance
        pk = SshPublicKeyFactory.newInstance(
                determineAlgorithm(clientKexInit.getSupportedPublicKeys(),
                serverKexInit.getSupportedPublicKeys()));

        // Iniialize the public key instance
        pk.init(hostKey);

        // We have a valid key so verify it against the allowed hosts
        if (!hosts.verifyHost(properties.getHost(), pk.getFingerprint())) {
            return false;
        }

        // Verify the host key signauture
        return pk.verifySignature(signature, exchangeHash);
    }


    /**
     *  Called by the abstract super class when key exchange begins.
     *
     *@param  kex                             Description of the Parameter
     *@return                                 Description of the Return Value
     *@exception  TransportProtocolException  Can be thrown to indicate a fatal
     *      error (not key exchange failure)
     */
    protected boolean onBeginKeyExchange(SshKeyExchange kex)
             throws TransportProtocolException {

        try {

            // Start the key exchange instance
            kex.startClient();

            // Get the key exchange state
            KeyExchangeState state = kex.getState();

            // Wait for the key exchange to complete
            state.waitForCompletion();

            // If its failed then disconnect
            if (state.getState() == state.FAILED) {

                sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED, "Key exchange failed");
                return false;
            }

            // Record the output
            exchangeHash = state.getExchangeHash();
            hostKey = state.getHostKey();
            signature = state.getSignature();
            k = state.getSecret();

            // Verify the hoskey
            try {
                if (!verifyHostKey()) {

                    log.fatal("Sending disconnect: Host key invalid");

                    sendDisconnect(SshMsgDisconnect.HOST_KEY_NOT_VERIFIABLE,
                            "The host key supplied was not valid");

                    return false;
                }
            } catch (SshException e) {
                reportException(e);
            }

            // Send new keys
            sendNewKeys();

        } catch (KeyExchangeException ex) {
            reportException(ex);
        }

        return true;
    }


    /**
     *  Handles the SSH_MSG_SERVICE_ACCEPT message
     *
     *@param  msg  The message received
     */
    protected void onMsgServiceAccept(SshMsgServiceAccept msg) {
        ServiceState state = (ServiceState) services.get(msg.getServiceName());

        try {
            state.setState(ServiceState.SERVICE_ACCEPTED);
        } catch (InvalidStateException ise) {
            reportException(ise);
        }

    }


    /**
     *  Called by the framework when a registered message is received.
     *
     *@param  msg  The message received.
     */
    public void onMessageReceived(SshMessage msg) {

        switch (msg.getMessageId()) {

            case SshMsgServiceAccept.SSH_MSG_SERVICE_ACCEPT:
            {
                onMsgServiceAccept((SshMsgServiceAccept) msg);
                break;
            }
        }

    }

}
