/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.net.Socket;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.*;
import java.io.OutputStream;
import java.math.BigInteger;
import java.security.*;
import org.apache.log4j.Logger;
import com.sshtools.j2ssh.configuration.SshConnectionProperties;
import com.sshtools.j2ssh.transport.cipher.SshCipher;
import com.sshtools.j2ssh.transport.cipher.SshCipherFactory;
import com.sshtools.j2ssh.transport.hmac.SshHmac;
import com.sshtools.j2ssh.transport.hmac.SshHmacFactory;
import com.sshtools.j2ssh.transport.publickey.SshPublicKey;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;
import com.sshtools.util.Hash;
import com.sshtools.j2ssh.transport.kex.SshKeyExchange;
import com.sshtools.j2ssh.transport.kex.SshKeyExchangeFactory;

/**
 *  <p>
 *
 *  The main transport protocol implementation. This abstract class provides the
 *  common functionality of both client and server implementations.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public abstract class TransportProtocolCommon
         implements TransportProtocol, Runnable {

    // Object to synchronize key changing
    private Object keyLock = new Object();
    // The input stream for recieving data
    private TransportProtocolInputStream sshIn;
    // The output stream for sending data
    private TransportProtocolOutputStream sshOut;
    // The connected socket
    private Socket socket;
    // Message notification registry
    private Map messageNotifications = new HashMap();
    // Storage of messages whilst in key exchange
    private List messageStack = new ArrayList();
    // Key exchange lock for accessing the kix init messages
    private Object kexLock = new Object();
    // The state object
    private TransportProtocolState state = new TransportProtocolState();
    // The thread object
    private Thread thread = new Thread(this);

    private List serviceThreads = new ArrayList();

    // The allowed hosts
    /**
     *  Description of the Field
     */
    protected HostKeyVerification hosts;

    // The message store
    /**
     *  The transport layer's message store
     */
    protected SshMessageStore messageStore = new SshMessageStore();

    // Flag to keep on running
    private boolean keepRunning = true;

    /**
     *  The log4j log object
     */
    protected static Logger log = Logger.getLogger(TransportProtocolCommon.class);
    /**
     *  The software version comments that are sent during protocol negotiation
     */
    public final String SOFTWARE_VERSION_COMMENTS = "Sshtools.com Java SSH2 API (J2SSH) Version 1.0 alpha 1";
    /**
     *  The protocol version supported
     */
    public final String PROTOCOL_VERSION = "2.0";
    /**
     *  The identification string sent by the server
     */
    protected String serverIdent = null;
    /**
     *  The identification string sent by the client
     */
    protected String clientIdent = null;
    /**
     *  The key exchange init message sent by the client
     */
    protected SshMsgKexInit clientKexInit = null;
    /**
     *  The key exchange init message sent by the server
     */
    protected SshMsgKexInit serverKexInit = null;
    /**
     *  Indicates when either the remote or local side has completed key
     *  exchange
     */
    protected Boolean completeOnNewKeys = new Boolean(false);
    /**
     *  The exchange hash output from key exchange
     */
    protected byte[] exchangeHash = null;
    /**
     *  The servers host key data
     */
    protected byte[] hostKey = null;
    /**
     *  The servers signature supplied to verify the host key
     */
    protected byte[] signature = null;
    /**
     *  The public key object used in host key verification
     */
    protected SshPublicKey pk;
    /**
     *  The secret value k produced during key exchange
     */
    protected BigInteger k = null;
    /**
     *  The key exchange engine
     */
    protected SshKeyExchange kex = null;
    /**
     *  The connection properties for the current connection
     */
    protected SshConnectionProperties properties;


    /**
     *  Abstract method that requires a derived class to set value of the local
     *  identification string. If the class implementing this method is a client
     *  then it should set the clientIdent protected member variable, if the
     *  class is implementing a server it should set the protected member
     *  serverIdent.
     */
    protected abstract void setLocalIdent();


    /**
     *  Abstract method that requires a derived class to set value of the remote
     *  identification string. If the class implementing this method is a client
     *  then it should set the serverIdent protected member variable, if the
     *  class is implementing a server it should set the protected member
     *  clientIdent.
     *
     *@param  ident  The identifiaction string received from the remote host
     */
    protected abstract void setRemoteIdent(String ident);


    /**
     *  Abstract method to return the local identification string which is used
     *  in protocol negotiation and in computing the exchange hash.
     *  Implementations should return either the protected member variable
     *  clientIdent or serverIdent
     *
     *@return    The local computers idnetification string, used in protocol
     *      negotiation
     */
    protected abstract String getLocalIdent();


    /**
     *  Abstract method to return the remote identification string which is used
     *  in protocol negotiation and in computing the exchange hash.
     *  Implementations should return either the protected member variable
     *  clientIdent or serverIdent
     *
     *@return    The local computers idnetification string, used in protocol
     *      negotiation
     */
    protected abstract String getRemoteIdent();


    /**
     *  Abstract method to set the local kex init msg which is used in computing
     *  the exchange hash. Implementations should set the appropriate client or
     *  server member variable
     *
     *@param  msg  The local computers kex init message
     */
    protected abstract void setLocalKexInit(SshMsgKexInit msg);


    /**
     *  Abstract method to set the remote kex init msg which is used in
     *  computing the exchange hash. Implementations should set the appropriate
     *  client or server member variable
     *
     *@param  msg  The remote computers kex init message
     */
    protected abstract void setRemoteKexInit(SshMsgKexInit msg);


    /**
     *  Abstract method to get the local kex init msg which is used in computing
     *  the exchange hash. Implementations should return the appropriate client
     *  or server member variable
     *
     *@return    The local computers kex init message
     */
    protected abstract SshMsgKexInit getLocalKexInit();


    /**
     *  Abstract method to get the remote kex init msg which is used in
     *  computing the exchange hash. Implementations should return the
     *  appropriate client or server member variable
     *
     *@return    The local computers kex init message
     */
    protected abstract SshMsgKexInit getRemoteKexInit();


    /**
     *  Abstract method called when key exchange has begun
     *
     *@param  kex                             Description of the Parameter
     *@return                                 Description of the Return Value
     *@exception  TransportProtocolException  Description of the Exception
     */
    protected abstract boolean onBeginKeyExchange(SshKeyExchange kex) throws TransportProtocolException;


    /**
     *  Determines the correct key exchange algorithm to use
     *
     *@return                               A string containing the algorithm
     *      name i.e. "diffie-hellman-group1.sha1"
     *@throws  AlgorithmNotAgreedException  When no algorithm is agreed between
     *      the two parties an exception is thrown
     */
    protected String getKexAlgorithm() throws AlgorithmNotAgreedException {

        return determineAlgorithm(clientKexInit.getSupportedKex(),
                serverKexInit.getSupportedKex());
    }


    /**
     *  Abstract method to determine the correct encryption algorithm to use
     *  This is found by iterating through the clients supported algorithm and
     *  selecting the first supported encryption method that the server also
     *  supports. Client and server implementations should define this method
     *  using the determineAlgorithm method to pass either the CS or SC methods
     *  of the SshMsgKexInit object object
     *
     *@return                                  The encryption algorithm to use
     *      i.e. "3des-cbc"
     *@exception  AlgorithmNotAgreedException  When an algorithm cannot be
     *      agreed an exception is thrown
     */
    protected abstract String getEncryptionAlgorithm() throws AlgorithmNotAgreedException;


    /**
     *  Abstract method to determine the correct decryption algorithm to use
     *  This is found by iterating through the clients supported algorithm and
     *  selecting the first supported decryption method that the server also
     *  supports. Client and server implementations should define this method
     *  using the determineAlgorithm method to pass either the CS or SC methods
     *  of the SshMsgKexInit object.
     *
     *@return                                  The decryption algorithm to use
     *      i.e. "3des-cbc"
     *@exception  AlgorithmNotAgreedException  When an algorithm cannot be
     *      agreed an exception is thrown
     */
    protected abstract String getDecryptionAlgorithm() throws AlgorithmNotAgreedException;


    /**
     *  Abtract method for the client/server implmentations to determine the
     *  message authentication algorithm for the output stream. Client and
     *  server implementations should define this method using the
     *  determineAlgorithm method to pass either the CS or SC methods of the
     *  SshMsgKexInit object.
     *
     *@return                                  The mac algorithm to use i.e.
     *      "hmac-sha1"
     *@exception  AlgorithmNotAgreedException  When an algorithm cannot be
     *      agreed an exception is thrown
     */
    protected abstract String getOutputStreamMacAlgorithm() throws AlgorithmNotAgreedException;


    /**
     *  Abtract method for the client/server implmentations to determine the
     *  message authentication algorithm for the input stream. Client and server
     *  implementations should define this method using the determineAlgorithm
     *  method to pass either the CS or SC methods of the SshMsgKexInit object.
     *
     *@return                                  The mac algorithm to use i.e.
     *      "hmac-sha1"
     *@exception  AlgorithmNotAgreedException  When an algorithm cannot be
     *      agreed an exception is thrown
     */
    protected abstract String getInputStreamMacAlgorithm() throws AlgorithmNotAgreedException;


    /**
     *  Abtract method for the client/server implmentations to determine the
     *  compression algorithm for the output stream. Client and server
     *  implementations should define this method using the determineAlgorithm
     *  method to pass either the CS or SC methods of the SshMsgKexInit object.
     *
     *@return                                  The compression algorithm to use
     *      i.e. "zlib"
     *@exception  AlgorithmNotAgreedException  When an algorithm cannot be
     *      agreed an exception is thrown
     */
    protected abstract String getOutputStreamCompAlgorithm() throws AlgorithmNotAgreedException;


    /**
     *  Abtract method for the client/server implmentations to determine the
     *  compression algorithm for the input stream. Client and server
     *  implementations should define this method using the determineAlgorithm
     *  method to pass either the CS or SC methods of the SshMsgKexInit object.
     *
     *@return                                  The compression algorithm to use
     *      i.e. "zlib"
     *@exception  AlgorithmNotAgreedException  When an algorithm cannot be
     *      agreed an exception is thrown
     */
    protected abstract String getInputStreamCompAlgortihm() throws AlgorithmNotAgreedException;



    /**
     *  Constructor for the SshTransportProtocol object
     *
     *@param  socket                          A connected socket to the remote
     *      host
     *@param  properties                      The connection properties of the
     *      ssh connection
     *@param  hosts                           Description of the Parameter
     *@exception  TransportProtocolException  Description of the Exception
     */
    public TransportProtocolCommon(Socket socket, SshConnectionProperties properties,
            HostKeyVerification hosts) throws TransportProtocolException {

        // Save the connected socket for later use
        this.socket = socket;
        this.properties = properties;
        this.hosts = hosts;

        // Start the transport layer message loop
        log.info("Starting transport protocol");
        thread.start();

        state.waitForReadyState();
    }


    /**
     *  Gets the state attribute of the TransportProtocolCommon object
     *
     *@return    The state value
     */
    public TransportProtocolState getState() {
        return state;
    }


    /**
     *  Main processing method for the TransportProtocolCommon object
     */
    public void run() {

        try {

            state.setReadyState(false);

            log.info("Registering transport protocol messages with inputstream");

            // Create the input/output streams
            sshIn = new TransportProtocolInputStream(socket, this);
            sshOut = new TransportProtocolOutputStream(socket, this);

            // Register the transport layer messages that this class will handle
            registerMessage(new Integer(SshMsgDisconnect.SSH_MSG_DISCONNECT),
                    SshMsgDisconnect.class,
                    messageStore);

            registerMessage(new Integer(SshMsgIgnore.SSH_MSG_IGNORE),
                    SshMsgIgnore.class,
                    messageStore);

            registerMessage(new Integer(SshMsgUnimplemented.SSH_MSG_UNIMPLEMENTED),
                    SshMsgUnimplemented.class,
                    messageStore);

            registerMessage(new Integer(SshMsgDebug.SSH_MSG_DEBUG),
                    SshMsgDebug.class,
                    messageStore);

            registerMessage(new Integer(SshMsgKexInit.SSH_MSG_KEX_INIT),
                    SshMsgKexInit.class,
                    messageStore);

            registerMessage(new Integer(SshMsgNewKeys.SSH_MSG_NEWKEYS),
                    SshMsgNewKeys.class,
                    messageStore);

            // call abstract to initialise the local ident string
            setLocalIdent();

            // negotiate the protocol version
            negotiateVersion();

            // Send our Kex Init
            sendKeyExchangeInit();

            sshIn.start();

            processMessages();

        } catch (Exception e) {
            reportException(e);
        }

    }


    /**
     *  Stops the transport layer
     */
    protected final void stop() {

        // Close the input/output streams
        sshIn.stop();
        sshOut.stop();

        try {
            socket.close();
        } catch (IOException ioe) {}

        // Let the message loop know that we want to stop
        keepRunning = false;

        // Interrupt the message store if its waiting
        thread.interrupt();

        log.info("Transport protocol thread is exiting");
    }


    /**
     *  Implements the message loop
     *
     *@throws  InvalidMessageException
     *@throws  TransportProtocolException
     */
    private void processMessages() throws InvalidMessageException, TransportProtocolException {

        while (keepRunning) {
            SshMessage msg = messageStore.getMessage();

            switch (msg.getMessageId()) {

                case SshMsgKexInit.SSH_MSG_KEX_INIT:
                {
                    onMsgKexInit((SshMsgKexInit) msg);
                    break;
                }
                case SshMsgDisconnect.SSH_MSG_DISCONNECT:
                {
                    onMsgDisconnect((SshMsgDisconnect) msg);
                    break;
                }
                case SshMsgIgnore.SSH_MSG_IGNORE:
                {
                    onMsgIgnore((SshMsgIgnore) msg);
                    break;
                }
                case SshMsgUnimplemented.SSH_MSG_UNIMPLEMENTED:
                {
                    onMsgUnimplemented((SshMsgUnimplemented) msg);
                    break;
                }
                case SshMsgDebug.SSH_MSG_DEBUG:
                {
                    onMsgDebug((SshMsgDebug) msg);
                    break;
                }
                case SshMsgNewKeys.SSH_MSG_NEWKEYS:
                {
                    onMsgNewKeys((SshMsgNewKeys) msg);
                    break;
                }
                default:
                {
                    onMessageReceived(msg);
                }
            }

        }

    }



    /**
     *  When the protocol starts, both sides must send an identification string
     *  that identifies the protocol version supported as well as and additional
     *  software version comments field. The identification strings are saved
     *  for later use in computing the exchange hash
     *
     *@exception  TransportProtocolException  Description of the Exception
     */
    private void negotiateVersion() throws TransportProtocolException {

        byte[] buf;
        int len;
        String remoteVer = "";

        try {

            log.info("Negotiating protocol version");
            log.debug("Local identification: " + getLocalIdent());
            // Get the local ident string by calling the abstract method, this
            // way the implementations set the correct variables for computing the
            // exchange hash
            String data = getLocalIdent() + "\r\n";

            // Send our version string
            socket.getOutputStream().write(data.getBytes());

            // Now wait for a reply and evaluate the ident string
            buf = new byte[255];

            // Look for a string starting with "SSH-"
            while (!remoteVer.startsWith("SSH-")) {

                len = socket.getInputStream().read(buf);
                remoteVer = new String(buf, 0, len);
            }

            // Get the index of the seperators
            int l = remoteVer.indexOf("-");
            int r = remoteVer.indexOf("-", l + 1);

            // Call abstract method so the implementations can set the
            // correct member variable
            setRemoteIdent(remoteVer.trim());

            log.debug("Remote identification: " + getRemoteIdent());

            // Get the version
            String remoteVersion = remoteVer.substring(l + 1, r);

            // Evaluate the version, we only support 2.0
            if (!(remoteVersion.equals("2.0") || (remoteVersion.equals("1.99")))) {
                log.fatal("The remote computer does not support protocol version 2.0");
                throw new TransportProtocolException("The protocol version of the remote computer is not supported!");
            }

            log.info("Protocol negotiation complete");

        } catch (IOException ioe) {
            log.fatal(ioe);
            throw new TransportProtocolException("Remote Identification string invalid: " + remoteVer);
        }

    }


    /**
     *  Sets the transport layer up for performing the key exchange, this is
     *  called when either a SSH_MSG_KEXINIT message is received or sent by
     *  either party
     *
     *@exception  TransportProtocolException  Description of the Exception
     */
    protected void beginKeyExchange() throws TransportProtocolException {

        log.info("Starting key exchange");

        String kexAlgorithm = "";

        // We now have both kex inits, this is where client/server
        // implemtations take over so call abstract methods

        try {

            // Determine the key exchange algorithm
            kexAlgorithm = getKexAlgorithm();

            log.debug("Key exchange algorithm: " + kexAlgorithm);

            // Get an instance of the key exchange algortihm
            kex = SshKeyExchangeFactory.newInstance(kexAlgorithm);

            // Initialise with the ident strings and kex inits
            kex.init(this, clientIdent, serverIdent,
                    clientKexInit.toByteArray(),
                    serverKexInit.toByteArray());

            onBeginKeyExchange(kex);

        } catch (AlgorithmNotAgreedException e) {

            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED,
                    "No suitable key exchange algorithm was agreed");

            throw new TransportProtocolException("The connection was disconnected because" +
                    " no suitable key exchange algorithm could be agreed.");
        } catch (AlgorithmNotSupportedException anse) {

            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED,
                    "An error occured during key exchage");

            throw new TransportProtocolException("The connection was disconnected because" +
                    " the key exchange algorithm " + kexAlgorithm + " could not be loaded.");
        } catch (InvalidMessageException ime) {

            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED,
                    "An error occured during key exchage");

            throw new TransportProtocolException("The connection was disconnected because an invalid" +
                    " key exchange message as detected.");
        }

    }



    /**
     *  Sends the SSH_MSG_NEWKEYS message to indicate that new keys are now in
     *  operation
     *
     *@exception  TransportProtocolException  Description of the Exception
     */
    protected void sendNewKeys() throws TransportProtocolException {

        // Send new keys
        SshMsgNewKeys msg = new SshMsgNewKeys();
        sendMessage(msg, this);

        // If we have received the remote sides new keys message then
        // we will put the new keys into operation
        synchronized (completeOnNewKeys) {

            if (completeOnNewKeys.booleanValue()) {
                completeKeyExchange();
            } else {
                completeOnNewKeys = new Boolean(true);
            }

        }

    }


    /**
     *  Completes key exchange by creating keys from the exchange hash and puts
     *  them into use
     *
     *@exception  TransportProtocolException  Description of the Exception
     */
    protected void completeKeyExchange() throws TransportProtocolException {

        log.info("Completing key exchange");

        try {
            // Put the keys into operation
            synchronized (keyLock) {

                // Reset the state variables
                completeOnNewKeys = new Boolean(false);

                log.debug("Making keys from key exchange output");
                // Make the keys
                byte[] encryptionKey = makeSshKey('C');
                byte[] encryptionIV = makeSshKey('A');
                byte[] decryptionKey = makeSshKey('D');
                byte[] decryptionIV = makeSshKey('B');
                byte[] sendMac = makeSshKey('E');
                byte[] receiveMac = makeSshKey('F');

                log.debug("Creating algorithm objects");

                // Setup the encryption cipher
                SshCipher sshCipher = SshCipherFactory.newInstance(getEncryptionAlgorithm());
                sshCipher.init(SshCipher.ENCRYPT_MODE, encryptionIV, encryptionKey);
                sshOut.setCipher(sshCipher);

                // Setup the decryption cipher
                sshCipher = SshCipherFactory.newInstance(getDecryptionAlgorithm());
                sshCipher.init(SshCipher.DECRYPT_MODE, decryptionIV, decryptionKey);
                sshIn.setCipher(sshCipher);

                // Create and put our macs into operation
                SshHmac hmac = SshHmacFactory.newInstance(getOutputStreamMacAlgorithm());
                hmac.init(sendMac);
                sshOut.setMac(hmac);

                hmac = SshHmacFactory.newInstance(getInputStreamMacAlgorithm());
                hmac.init(receiveMac);
                sshIn.setMac(hmac);

                // Reset the key exchange
                clientKexInit = null;
                serverKexInit = null;
                kex = null;

                /*
                 *  Update our state, we can send all packets
                 *  }
                 */
                state.setReadyState(true);

                // Send any outstanding messages
                synchronized (messageStack) {

                    Iterator it = messageStack.iterator();

                    log.debug("Sending queued messages");

                    while (it.hasNext()) {
                        SshMessage msg = (SshMessage) it.next();

                        sendMessage(msg, this);

                    }

                    messageStack.clear();

                }

            }
        } catch (AlgorithmNotAgreedException anae) {
            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED, "Algorithm not agreed");
            throw new TransportProtocolException("The connection was disconnected because an algorithm could not be agreed");
        } catch (AlgorithmNotSupportedException anse) {
            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED, "Application error");
            throw new TransportProtocolException("The connection was disconnected because an algorithm class could not be loaded");
        } catch (AlgorithmOperationException aoe) {
            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED, "Algorithm operation error");
            throw new TransportProtocolException("The connection was disconnected because" +
                    " of an algorithm operation error");
        } catch (AlgorithmInitializationException aie) {
            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED, "Algorithm initialization error");
            throw new TransportProtocolException("The connection was disconnected because" +
                    " of an algorithm initialization error");
        }
    }


    /**
     *  Helper method to determine the first algorithm that appears in the
     *  client list that is also supported by the server
     *
     *@param  clientAlgorithms                 The list of client algorithms
     *@param  serverAlgorithms                 The list of server algorithms
     *@return                                  The determined algrithms
     *@exception  AlgorithmNotAgreedException  Thrown if the algorithm cannot be
     *      agreed
     */
    protected String determineAlgorithm(List clientAlgorithms, List serverAlgorithms)
             throws AlgorithmNotAgreedException {

        log.debug("Determine Algorithm");
        log.debug("Client Algorithms: " + clientAlgorithms.toString());
        log.debug("Server Algorithms: " + serverAlgorithms.toString());

        String algorithmClient;
        String algorithmServer;

        Iterator itClient = clientAlgorithms.iterator();

        while (itClient.hasNext()) {

            algorithmClient = (String) itClient.next();

            Iterator itServer = serverAlgorithms.iterator();

            while (itServer.hasNext()) {

                algorithmServer = (String) itServer.next();

                if (algorithmClient.equals(algorithmServer)) {

                    log.debug("Returning " + algorithmClient);

                    return algorithmClient;
                }
            }
        }

        throw new AlgorithmNotAgreedException("Could not agree algorithm");
    }


    /**
     *  Creates an Ssh key from the exchange hash and a literal character
     *
     *@param  chr                             The character used to create the
     *      key
     *@return                                 40 bytes of key data
     *@exception  TransportProtocolException  Description of the Exception
     */
    private byte[] makeSshKey(char chr) throws TransportProtocolException {

        try {

            // Create the first 20 bytes of key data
            ByteArrayWriter keydata = new ByteArrayWriter();
            byte[] data = new byte[20];

            Hash hash = new Hash("SHA");

            // Put the dh k value
            hash.putBigInteger(k);
            // Put in the exchange hash
            hash.putBytes(exchangeHash);
            // Put in the character
            hash.putByte((byte) chr);
            // Put the exchange hash in again
            hash.putBytes(exchangeHash);

            // Create the fist 20 bytes
            data = hash.doFinal();
            keydata.write(data);

            // Now do the next 20
            hash.reset();
            // Put the dh k value in again
            hash.putBigInteger(k);
            // And the exchange hash
            hash.putBytes(exchangeHash);
            // Finally the first 20 bytes of data we created
            hash.putBytes(data);

            data = hash.doFinal();

            // Put it all together
            keydata.write(data);

            // Return it
            return keydata.toByteArray();
        } catch (NoSuchAlgorithmException nsae) {
            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED, "Application error");
            throw new TransportProtocolException("SHA algorithm not supported");
        } catch (IOException ioe) {
            sendDisconnect(SshMsgDisconnect.KEY_EXCHANGE_FAILED, "Application error");
            throw new TransportProtocolException("Error writing key data");
        }

    }


    /**
     *  Send an SSH message, if the state doen't allow it because of key
     *  exchange then the message is stored and sent as soon as the state
     *  changes
     *
     *@param  msg                             The SshMessage to send
     *@param  sender                          Description of the Parameter
     *@exception  TransportProtocolException  Description of the Exception
     */
    public void sendMessage(SshMessage msg, Object sender)
             throws TransportProtocolException {

        // Send a message, if were in key exchange then add it to
        // the list unless of course it is a transport protocol or key
        // exchange message
        log.debug("Sending " + msg.getMessageName());

        //try {
        if (sender instanceof SshKeyExchange || sender instanceof TransportProtocolCommon) {
            //sshOut.sendMessage(msg);
            sshOut.sendMessage(msg);
            return;
        }

        if (!state.isReady()) {

            log.debug("Adding to message queue");
            synchronized (messageStack) {
                // Add this message to the end of the list
                messageStack.add(msg);
            }
        } else {

            sshOut.sendMessage(msg);
        }

    }


    /**
     *  This is called when a corrupt Mac has been received on the input stream.
     *  In this instance we will send a disconnect message.
     */
    protected void onCorruptMac() {

        log.fatal("Corrupt Mac on Input");

        // Send a disconnect message
        sendDisconnect(SshMsgDisconnect.MAC_ERROR, "Corrupt Mac on input");

    }


    /**
     *  Implements the TransportProtocol interface method to allow external SSH
     *  implementations to receive message notificaitons.
     *
     *@param  messageId                              The messageId of the
     *      registered message
     *@param  implementor                            The class that implements
     *      the message
     *@param  store                                  The message store to
     *      receive notificaiton
     *@exception  MessageAlreadyRegisteredException  Thrown if the message
     *      cannot be registered.
     */
    public void registerMessage(Integer messageId, Class implementor, SshMessageStore store)
             throws MessageAlreadyRegisteredException {

        log.debug("Registering message Id " + messageId.toString());

        if (!messageNotifications.containsKey(messageId)) {
            messageNotifications.put(messageId, new RegisteredMessage(implementor, store));
        } else {
            throw new MessageAlreadyRegisteredException(messageId);
        }

    }


    /**
     *  Implements the TransportProtocol method to allow external SSH
     *  implementations to unregister a message.
     *
     *@param  messageId                          The message id of the message
     *@param  store                              The message store receiving the
     *      notifications.
     *@exception  MessageNotRegisteredException  Thrown if the message is not
     *      registered.
     */
    public void unregisterMessage(Integer messageId, SshMessageStore store)
             throws MessageNotRegisteredException {

        log.debug("Unregistering message Id " + messageId.toString());

        if (!messageNotifications.containsKey(messageId)) {
            throw new MessageNotRegisteredException(messageId);
        }
        SshMessageStore actual = (SshMessageStore) messageNotifications.get(messageId);

        if (!store.equals(actual)) {
            throw new MessageNotRegisteredException(messageId, store);
        }

        messageNotifications.remove(messageId);

    }


    /**
     *  Sends a disconnect message
     *
     *@param  reason       The reason code.
     *@param  description  The readable reason description.
     */
    protected void sendDisconnect(int reason, String description) {

        SshMsgDisconnect msg = new SshMsgDisconnect(reason, description, "");

        try {
            sendMessage(msg, this);

        } catch (Exception e) {

        }

    }


    /**
     *  Disconnects the connection by sending a disconnect message with the
     *  BY_APPLICAITON reason.
     *
     *@param  description  The description of the reason
     */
    public void disconnect(String description) {

        log.debug("Disconnecting: " + description);

        SshMsgDisconnect msg = new SshMsgDisconnect(SshMsgDisconnect.BY_APPLICATION,
                description, "");

        try {

            sshIn.disconnecting = true;

            // Send the disconnect message
            sendMessage(msg, this);

            stop();

        } catch (Exception e) {

        }

    }


    /**
     *  Called by the framework when a new message is received
     *
     *@param  messageId                       Description of the Parameter
     *@param  bar                             Description of the Parameter
     *@exception  InvalidMessageException     Description of the Exception
     *@exception  TransportProtocolException  Description of the Exception
     */
    protected void onMessageData(int messageId, ByteArrayReader bar) throws InvalidMessageException,
            TransportProtocolException {

        try {

            // Get the registered message
            RegisteredMessage registered = (RegisteredMessage) messageNotifications.get(new Integer(messageId));

            if (registered == null) {
                SshMsgUnimplemented msg = new SshMsgUnimplemented(sshIn.getSequenceNo());
                sendMessage(msg, this);
                return;
            }
            SshMessage msg = (SshMessage) registered.getImplementor().newInstance();

            msg.fromByteArray(bar);

            SshMessageStore store = (SshMessageStore) registered.getMessageStore();

            if (store == null) {
                throw new TransportProtocolException("Message was registered but messate store is null!");
            }

            log.info("Received " + msg.getMessageName());

            store.addMessage(msg);

        } catch (IllegalAccessException iae) {
            throw new InvalidMessageException("Illegal access! Could not create message class: " +
                    iae.getMessage());
        } catch (InstantiationException ie) {
            throw new InvalidMessageException("Could not instansate class: " +
                    ie.getMessage());
        }

    }


    /**
     *  Called by the framework when a new message is received.
     *
     *@param  msg  The message recevied
     */
    protected abstract void onMessageReceived(SshMessage msg);



    /**
     *  Handles the new keys message
     *
     *@param  msg                             The message received
     *@exception  TransportProtocolException
     */
    private void onMsgNewKeys(SshMsgNewKeys msg) throws TransportProtocolException {

        // Determine whether we have completed our own
        log.debug("Received New Keys");

        synchronized (completeOnNewKeys) {

            if (completeOnNewKeys.booleanValue()) {
                completeKeyExchange();
            } else {
                completeOnNewKeys = new Boolean(true);
            }

        }
    }


    /**
     *  Handles the kex init message
     *
     *@param  msg                             Description of the Parameter
     *@exception  TransportProtocolException  Description of the Exception
     */
    private void onMsgKexInit(SshMsgKexInit msg) throws TransportProtocolException {

        log.debug("Received remote key exchange init message");
        log.debug(msg.toString());

        synchronized (kexLock) {

            setRemoteKexInit(msg);

            // As either party can initiate a key exchange then we
            // must check to see if we have sent our own
            if (getLocalKexInit() == null) {
                sendKeyExchangeInit();
            }

            beginKeyExchange();

        }
    }


    /**
     *  Sends the key exchange init message
     *
     *@exception  TransportProtocolException  Description of the Exception
     */
    protected void sendKeyExchangeInit() throws TransportProtocolException {

        setLocalKexInit(new SshMsgKexInit(properties));
        sendMessage(getLocalKexInit(), this);

    }


    /**
     *  Handles the ignore message
     *
     *@param  msg  Description of the Parameter
     */
    private void onMsgIgnore(SshMsgIgnore msg) {

        log.debug("SSH_MSG_IGNORE with " + String.valueOf(msg.getData().length()) + " bytes of data");

    }


    /**
     *  Handles a debug message
     *
     *@param  msg  Description of the Parameter
     */
    private void onMsgDebug(SshMsgDebug msg) {

        if (msg.getDisplayAlways()) {
            log.debug(msg.getMessage());
        }
    }


    /**
     *  Handles a disconnect message
     *
     *@param  msg                             Description of the Parameter
     *@exception  TransportProtocolException  Description of the Exception
     */
    private void onMsgDisconnect(SshMsgDisconnect msg) throws TransportProtocolException {

        reportDisconnect(msg.getDescription());

        stop();

    }


    /**
     *  Handles an unimplemented message
     *
     *@param  msg  Description of the Parameter
     */
    private void onMsgUnimplemented(SshMsgUnimplemented msg) {

        log.debug("The message with sequence no " + msg.getSequenceNo() + " was reported as unimplemented by the remote end.");

    }


    /**
     *  Called if an inputstream error occurs
     *
     *@param  e  Description of the Parameter
     */
    protected void onChildThreadException(Exception e) {

        reportException(e);

        if (keepRunning) {
            stop();
        }
    }


    /**
     *  <p>
     *
     *  Sets a service thread so that it receives notifications of exceptions.
     *  </p>
     *
     *@param  service
     */
    public void setServiceThread(Service service) {

        serviceThreads.add(service);

    }


    /**
     *  <p>
     *
     *  Removes thes service thread from the exception notification list.</p>
     *
     *@param  service
     */
    public void removeServiceThread(Service service) {

        serviceThreads.remove(service);

    }


    /**
     *  <p>
     *
     *  Reports an exception to service threads</p>
     *
     *@param  e
     */
    protected void reportException(Exception e) {

        Iterator it = serviceThreads.iterator();

        while (it.hasNext()) {

            Service service = (Service) it.next();

            service.onThreadException(e);

        }

    }


    /**
     *  <p>
     *
     *  Reports a disconnect to service threads.</p>
     *
     *@param  reason
     */
    protected void reportDisconnect(String reason) {

        Iterator it = serviceThreads.iterator();

        while (it.hasNext()) {

            Service service = (Service) it.next();

            service.onDisconnect(reason);

        }

    }

}

/**
 *  Stores the details of a registered message
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */
class RegisteredMessage {


    private Class implementor;
    private SshMessageStore messageStore;


    /**
     *  Constructor for the RegisteredMessage object
     *
     *@param  implementor   Description of the Parameter
     *@param  messageStore  Description of the Parameter
     */
    public RegisteredMessage(Class implementor, SshMessageStore messageStore) {

        this.implementor = implementor;
        this.messageStore = messageStore;

    }


    /**
     *  Gets the implementor attribute of the RegisteredMessage object
     *
     *@return    The implementor value
     */
    public Class getImplementor() {
        return implementor;
    }


    /**
     *  Gets the messageStore attribute of the RegisteredMessage object
     *
     *@return    The messageStore value
     */
    public SshMessageStore getMessageStore() {
        return messageStore;
    }

}
