/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.math.BigInteger;
import java.util.*;
import java.io.*;
import java.net.Socket;
import javax.crypto.*;
import java.security.*;
import org.apache.log4j.Logger;
import com.sshtools.j2ssh.transport.cipher.SshCipher;
import com.sshtools.j2ssh.transport.hmac.SshHmac;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;
/**
 *  <p>
 *
 *  Waits on the socket for data, decrypts and performs message authentication
 *  and then routes the message to the transport layer for processing.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class TransportProtocolInputStream implements Runnable {

    private Thread thread;
    private BufferedInputStream in;
    private Socket socket;
    private TransportProtocolCommon listener;
    private boolean keepRunning = true;
    private SshCipher cipher = null;
    private SshHmac hmac = null;
    private Object sequenceLock = new Object();
    private long sequenceNo = 0;
    private long sequenceWrapLimit = BigInteger.valueOf(2).pow(32).longValue();
    private static Logger log = Logger.getLogger(TransportProtocolInputStream.class);
    private Object keyLock = new Object();
    /**
     *  Description of the Field
     */
    public boolean disconnecting = false;


    /**
     *  Constructor for the TransportProtocolInputStream object
     *
     *@param  in        The socket input stream
     *@param  listener  The transport layer for routing messages
     */
    public TransportProtocolInputStream(Socket socket,
                                  TransportProtocolCommon listener)
                                                       throws IOException {

        this.socket = socket;
        this.in = new BufferedInputStream(socket.getInputStream());
        this.listener = listener;

    }


    /**
     *  <p>
     *
     *  Starts the inputstream.</p>
     */
    protected void start() {

        log.info("Starting TransportProtocolInputStream thread");
        thread = new Thread(this);
        thread.start();

    }


    /**
     *  <p>
     *
     *  Stops the Inputstream.</p>
     */
    public void stop() {

        try {
            keepRunning = false;
            in.close();
            thread.interrupt();

        } catch (IOException ioe) {

        }

        log.info("Transport protocol inputstream thread is exiting");

    }


    /**
     *  Main processing method for the SshInputStream object
     */
    public void run() {

        int msglen;
        int padlen;
        int read;
        int remaining;
        int cipherlen = 8;
        int maclen = 0;
        int messageId;

        // Create an input buffer about 1024 bytes but make sure that it
        // is always is a multiple of the block size
        byte[] buffer = new byte[128 * cipherlen];
        ByteArrayWriter message = new ByteArrayWriter();
        byte[] initial = new byte[cipherlen];
        byte[] data;

        while (keepRunning) {

            try {

                // Read the first block size bytes of this message
                read = in.read(initial);

                // Make sure we have not closed or reached eof
                if (read <= 0 && !disconnecting) {
                    listener.onChildThreadException(new Exception("Error reading from socket 1"));
                    return;
                }

                // The rest of the loop we cannot change keys
                synchronized (keyLock) {

                    // If the cipher object has been set then make sure
                    // we have the correct blocksize
                    if (cipher != null) {
                        cipherlen = cipher.getBlockSize();
                    } else {
                        cipherlen = 8;
                    }

                    // Make sure that our initial buffer size is the correct
                    if (initial.length != cipherlen) {
                        initial = new byte[cipherlen];
                    }

                    // Make sure that our buffer size is a multiple of blocksize
                    if ((buffer.length % cipherlen) != 0) {
                        buffer = new byte[128 * cipherlen];
                    }

                    // Record the mac length
                    if (hmac != null) {
                        maclen = hmac.getMacLength();
                    } else {
                        maclen = 0;
                    }

                    // Decrypt the data if we have a valid cipher
                    if (cipher != null) {
                        initial = cipher.transform(initial);
                    }

                    // Save the initial data
                    message.write(initial);

                    // Preview the message length
                    msglen = ByteArrayReader.previewInt(initial, 0);
                    padlen = initial[4];
                    messageId = initial[5];

                    // Read, decrypt and save the remaining data
                    remaining = (msglen - (cipherlen - 4));

                    // Loop collecting data untill we have the correct number of
                    // bytes
                    while (remaining > 0) {

                        // Read up to buffer.length or remaining whichever is lower
                        if (remaining > buffer.length) {
                            read = in.read(buffer);
                        } else {
                            read = in.read(buffer, 0, remaining);
                        }

                        // Check that nothing went wrong on the socket
                        if (read > 0) {

                            // Record how many bytes weve received
                            remaining -= read;

                            // Decrypt the data and/or write it to the message
                            if (cipher != null) {
                                message.write(cipher.transform(buffer, 0, read));
                            } else {
                                message.write(buffer, 0, read);
                            }

                        } else if (!disconnecting) {
                            listener.onChildThreadException(new Exception("Error reading from socket 2"));
                            return;
                        }
                    }
                    // End of while

                    synchronized (sequenceLock) {

                        if (hmac != null) {

                            read = in.read(buffer, 0, maclen);

                            message.write(buffer, 0, read);
                            // Verify the mac
                            if (!hmac.verify(sequenceNo, message.toByteArray())) {
                                listener.onCorruptMac();
                            }
                        }

                        // Increment the sequence no
                        if (sequenceNo < sequenceWrapLimit) {
                            sequenceNo++;
                        } else {
                            sequenceNo = 0;
                        }

                    }
                    // End of sequence no lock

                    // Trim the packet length, padding length and padding fields
                    ByteArrayReader msg = new ByteArrayReader(message.toByteArray());

                    // Reset the message for the next
                    message.reset();

                    // We now have a completed message
                    listener.onMessageData(messageId, msg);

                }
                // End of key lock synchronization

            } catch (Exception e) {
                if(keepRunning)
                  listener.onChildThreadException(e);

            }

        }

    }


    /**
     *  Gets the sequence no of the last message sent
     *
     *@return    The sequenceNo value
     */
    public synchronized long getSequenceNo() {

        return sequenceNo;
    }


    /**
     *  Sets the cipher
     *
     *@param  cipher  The new cipher value
     */
    public synchronized void setCipher(SshCipher cipher) {

        this.cipher = cipher;

    }


    /**
     *  Sets the mac
     *
     *@param  hmac  The new mac value
     */
    public synchronized void setMac(SshHmac hmac) {

        this.hmac = hmac;

    }

}
