/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport;

import java.io.*;
import java.math.BigInteger;
import java.util.*;
import org.apache.log4j.Logger;
import java.net.Socket;
import java.security.SecureRandom;
import com.sshtools.j2ssh.transport.cipher.SshCipher;
import com.sshtools.j2ssh.transport.compression.SshCompression;
import com.sshtools.j2ssh.transport.hmac.SshHmac;
import com.sshtools.util.ByteArrayWriter;
/**
 *  <p>
 *
 *  Sends messages through the socket first encrypting and then adding message
 *  authentication.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

class TransportProtocolOutputStream implements Runnable {

    private OutputStream out;
    private Socket socket;
    private SshCipher cipher;
    private SshHmac hmac;
    private SshCompression compression;
    private long sequenceNo = 0;
    private long sequenceWrapLimit = BigInteger.valueOf(2).pow(32).longValue();
    private Object keyLock = new Object();
    private static Logger log = Logger.getLogger(TransportProtocolOutputStream.class);
    private SshMessageStore messageStore = new SshMessageStore();
    private Thread thread = new Thread(this);
    private boolean keepRunning;
    private TransportProtocolCommon transport;


    /**
     *  Constructor for the TransportProtocolOutputStream object
     *
     *@param  out        The sockets outputstream
     *@param  transport  Description of the Parameter
     */
    public TransportProtocolOutputStream(Socket socket,
                          TransportProtocolCommon transport)
                              throws IOException {

        this.socket = socket;
        this.out = socket.getOutputStream();
        this.transport = transport;
        keepRunning = true;
        thread.start();

    }


    /**
     *  <p>
     *
     *  Sends a message</p>
     *
     *@param  msg
     */
    public void sendMessage(SshMessage msg) {
        messageStore.addMessage(msg);
    }


    /**
     *  <p>
     *
     *  Loops waiting on the message store for new messages to send</p>
     */
    public void run() {

        while (keepRunning) {

            SshMessage msg = messageStore.getMessage();

            try {
                send(msg);
            } catch (Exception e) {

                if (keepRunning) {
                    log.fatal("Exception in output stream!" + e.getMessage());
                    transport.onChildThreadException(e);
                }
            }

        }

    }


    /**
     *  <p>
     *
     *  Stops the thread.</p>
     */
    public void stop() {

        keepRunning = false;

        thread.interrupt();

        try {
            out.close();
        } catch (IOException ioe) {

        }

        log.info("The Transport protocol outputstream thread is exiting");
    }


    /**
     *  Sets the cipher
     *
     *@param  cipher  The new cipher value
     */
    public synchronized void setCipher(SshCipher cipher) {

        this.cipher = cipher;

    }


    /**
     *  Sets the mac
     *
     *@param  hmac  The new mac value
     */
    public synchronized void setMac(SshHmac hmac) {

        this.hmac = hmac;
    }


    /**
     *  sends a message
     *
     *@param  msg                              An SshMessage derived class
     *@exception  AlgorithmOperationException  Description of the Exception
     *@exception  IOException                  Description of the Exception
     *@exception  InvalidMessageException      Description of the Exception
     */
    private void send(SshMessage msg) throws AlgorithmOperationException, IOException,
            InvalidMessageException {

        synchronized (keyLock) {

            // Get the message payload data
            byte[] msgdata = msg.toByteArray();
            int padding = 4;
            int cipherlen = 8;

            // Determine the cipher length
            if (cipher != null) {
                cipherlen = cipher.getBlockSize();
            }

            // Determine the padding length
            padding += (cipherlen - (msgdata.length + 5 + padding) % cipherlen) % cipherlen;

            // Write the data into a byte array
            ByteArrayWriter message = new ByteArrayWriter();
            // Write the packet length field
            message.writeInt(msgdata.length + 1 + padding);
            // Write the padding length
            message.write(padding);
            // Write the message payload
            message.write(msgdata);

            // Create some random data for the padding
            byte[] pad = new byte[padding];
            SecureRandom r = new SecureRandom();
            r.nextBytes(pad);

            // Write the padding
            message.write(pad);

            // Get the unencrypted packet data
            byte[] packet = message.toByteArray();
            byte[] mac = null;

            // Generate the MAC
            if (hmac != null) {
                mac = hmac.generate(sequenceNo, packet, 0, packet.length);
            }

            // Do some compression
            if (compression != null) {
                packet = compression.transform(packet);
            }

            // Perfrom encrpytion
            if (cipher != null) {
                packet = cipher.transform(packet);
            }

            // Reset the message
            message.reset();

            // Write the packet data
            message.write(packet);

            // Combine the packet and MAC
            if (mac != null) {
                message.write(mac);
            }

            // Send!
            if(socket.isConnected())
              out.write(message.toByteArray());

            // Increment the sequence no
            if (sequenceNo < sequenceWrapLimit) {
                sequenceNo++;
            } else {
                sequenceNo = 0;
            }

        }
    }
}
