/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.cipher;
import com.sshtools.j2ssh.transport.AlgorithmOperationException;

/**
 *  <p>
 *
 *  Defines the interface for all SSH ciphers. Any cipher class that implements
 *  this interface can be used for encryption within the SSH protocol. Third
 *  party implementations can dynamically add new ciphers to existing J2SSH
 *  installations by inserting a CipherAlgorithm element into the SSH API
 *  configuration file ssh.xml. The algorithm name must follow the name@domain
 *  syntax.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public interface SshCipher {

    /**
     *  Declares the cipher as encrypting
     */
    public final int ENCRYPT_MODE = 0;
    /**
     *  Declares the cipher as decrypting
     */
    public final int DECRYPT_MODE = 1;


    /**
     *  <p>
     *
     *  Called to initiate the cipher after construction.</p>
     *
     *@param  mode                             The cipher mode ENCRYPT_MODE |
     *      DECRYPT_MODE
     *@param  iv                               The iv data output from key
     *      exchange
     *@param  keydata                          The key data output from key
     *      exchange
     *@exception  AlgorithmOperationException  Throw if an error occurs
     */
    public void init(int mode, byte[] iv, byte[] keydata) throws AlgorithmOperationException;


    /**
     *  <p>
     *
     *  Called to transform the supplied data according to the cipher mode.</p>
     *
     *@param  data                             The untransformed data
     *@return                                  The transformed data
     *@exception  AlgorithmOperationException  Throw if an error occurs
     */
    public byte[] transform(byte[] data) throws AlgorithmOperationException;


    /**
     *  <p>
     *
     *  Called to transform the supplied data according to the cipher mode.</p>
     *
     *@param  data                          The untransformed data
     *@param  offset                        The offset to start in the data
     *@param  len                           The length to transform
     *@return                               The transformed data
     *@throws  AlgorithmOperationException  Throw if an error occurs
     */
    public byte[] transform(byte[] data, int offset, int len) throws AlgorithmOperationException;


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    /*
     *  ( public byte[] doFinal(byte[] data) throws AlgorithmOperationException;
     */
    /**
     *  Gets the blockSize of the cipher
     *
     *@return    The blockSize value
     */
    public int getBlockSize();
}
