/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.cipher;

import java.util.*;
import org.apache.log4j.Logger;
import com.sshtools.j2ssh.configuration.CipherAlgorithm;
import com.sshtools.j2ssh.configuration.CipherConfiguration;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.configuration.SshAPIConfiguration;
import com.sshtools.j2ssh.transport.AlgorithmNotSupportedException;

/**
 *  <p>
 *
 *  Creates instances of all available ciphers. All the standard ciphers
 *  described within the SSH transport protocol are reserved for implementation
 *  by sshtools.com. Additional ciphers can be implemented by using the
 *  SshCipher interface and adding a CipherAlgorithm element into the
 *  CipherConfiguraton element of the SSH API configuraiton file ssh.xml.</p>
 *  <p>
 *
 *  All additional cipher names are required to follow the name@domain syntax as
 *  described in the SSH transport protocol. The factory will not register
 *  cipher names that do not follow the extensibility rules.</p> <p>
 *
 *  If a standard cipher is not currently available then email the sshtools
 *  development team or alternatvly implement it and send it to us! we will
 *  include your source and credit you.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshCipherFactory extends Exception {

    private static Map ciphers;
    private static String defaultCipher;
    private static Logger log = Logger.getLogger(SshCipherFactory.class);

    /**
     *  The standard TripleDes cipher
     */
    public final static String CIPHER_3DES = "3des-cbc";
    //public final static String CIPHER_BLOWFISH = "blowfish-cbc";

    static {

        ciphers = new HashMap();

        log.info("Loading supported cipher algorithms");
        ciphers.put(CIPHER_3DES, TripleDesCbc.class.getName());
        //ciphers.put(CIPHER_BLOWFISH, BlowfishCbc.class.getName());


        // Now load external ciphers from configuration file
        SshAPIConfiguration config = ConfigurationLoader.getAPIConfiguration();

        if (config != null) {

            CipherConfiguration cipherConfig = config.getCipherConfiguration();

            List addons = cipherConfig.getCipherAlgorithm();

            Iterator it = addons.iterator();

            // Add the ciphers to our supported list
            while (it.hasNext()) {

                CipherAlgorithm cipherAlgorithm = (CipherAlgorithm) it.next();

                String name = cipherAlgorithm.getAlgorithmName();

                // Make sure it follows the extensibility rules
                //if (name.lastIndexOf("@") > 0) {
                    ciphers.put(cipherAlgorithm.getAlgorithmName(),
                            cipherAlgorithm.getImplementationClass());

                    log.info(cipherAlgorithm.getAlgorithmName() + " has been added");

                //}
            }

            defaultCipher = cipherConfig.getDefaultAlgorithm();

            if (defaultCipher == null || !ciphers.containsKey(defaultCipher)) {
                defaultCipher = CIPHER_3DES;
            }
        }
    }


    /**
     *  Constructor for the SshCipherFactory object
     */
    protected SshCipherFactory() { }


    /**
     *  <p>
     *
     *  Returns the configurations default cipher.</p>
     *
     *@return
     */
    public static String getDefaultCipher() {
        return defaultCipher;
    }


    /**
     *  <p>
     *
     *  Returns the list of supported ciphers.</p>
     *
     *@return    The supported Ciphers
     */
    public static List getSupportedCiphers() {

        // Get the list of ciphers
        ArrayList list = new ArrayList(ciphers.keySet());

        // Remove the default from wherever it may be
        list.remove(defaultCipher);

        // Add it to the top of the list
        list.add(0, defaultCipher);

        // Return the list
        return list;
    }


    /**
     *  <p>
     *
     *  Creates a new instance of the algorithm.</p>
     *
     *@param  algorithmName                       The name of the algorithm to
     *      create
     *@return                                     The new instance
     *@exception  AlgorithmNotSupportedException  Thrown if the algorithm is not
     *      supported
     */
    public static SshCipher newInstance(String algorithmName)
             throws AlgorithmNotSupportedException {

        String className = "";

        log.info("Creating new " + algorithmName + " cipher instance");

        try {
            className = (String) ciphers.get(algorithmName);

            if (className != null) {
                return (SshCipher) Class.forName(className).newInstance();
            } else {
                throw new AlgorithmNotSupportedException(algorithmName + " is not supported");
            }
        } catch (Exception e) {
            throw new AlgorithmNotSupportedException("Could not locate " + className);
        }

    }
}
