/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.cipher;

import javax.crypto.*;
import javax.crypto.spec.*;
import java.security.*;
import java.security.spec.*;
import com.sshtools.j2ssh.transport.AlgorithmOperationException;

/**
 *  <p>
 *
 *  Implements the TripleDes algorithm for the SSH protocol</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

class TripleDesCbc implements SshCipher {

    Cipher cipher;

    /**
     *  The SSH algorithm name
     */
    protected static String algorithmName = "3des-cbc";


    /**
     *  Constructor for the Ssh3DesCbc object
     */
    public TripleDesCbc() { }


    /**
     *  <p>
     *
     *  Initiates the cipher object.</p>
     *
     *@param  mode                             The cipher mode
     *@param  iv                               The iv
     *@param  keydata                          The key
     *@exception  AlgorithmOperationException  Thrown if an error occurs
     */
    public void init(int mode, byte[] iv, byte[] keydata) throws AlgorithmOperationException {

        try {
            KeySpec keyspec;
            Key key;

            // Create the cipher according to its algorithm
            cipher = Cipher.getInstance("DESede/CBC/NoPadding");
            byte[] actualKey = new byte[24];
            System.arraycopy(keydata, 0, actualKey, 0, 24);
            keyspec = new DESedeKeySpec(actualKey);
            key = SecretKeyFactory.getInstance("DESede").generateSecret(keyspec);

            cipher.init((mode == ENCRYPT_MODE ? Cipher.ENCRYPT_MODE : Cipher.DECRYPT_MODE), key,
                    new IvParameterSpec(iv, 0, cipher.getBlockSize()));

        } catch (NoSuchPaddingException nspe) {
            throw new AlgorithmOperationException("Padding not supported");
        } catch (NoSuchAlgorithmException nsae) {
            throw new AlgorithmOperationException("Algorithm not supported");
        } catch (InvalidKeyException ike) {
            throw new AlgorithmOperationException("Invalid encryption key");
        } catch (InvalidKeySpecException ispe) {
            throw new AlgorithmOperationException("Invalid encryption key specification");
        } catch (InvalidAlgorithmParameterException ape) {
            throw new AlgorithmOperationException("Invalid algorithm parameter");
        }
    }


    /**
     *  <p>
     *
     *  Transform the data according to the cipher mode.</p>
     *
     *@param  data                             The untransformed data
     *@return                                  The transformed data
     *@exception  AlgorithmOperationException  Thrown if an error occurs
     */
    public byte[] transform(byte[] data) throws AlgorithmOperationException {

        return cipher.update(data);
    }


    /**
     *  <p>
     *
     *  Transform the data according to the cipher mode.</p>
     *
     *@param  data                          The untransformed data
     *@param  offset                        The offset to start
     *@param  len                           The length to transform
     *@return                               The transformed data
     *@throws  AlgorithmOperationException  Thrown if an error occurs
     */
    public byte[] transform(byte[] data, int offset, int len) throws AlgorithmOperationException {

        return cipher.update(data, offset, len);
    }


    /**
     *  Gets the blockSize of the cipher
     *
     *@return    The blockSize value
     */
    public int getBlockSize() {
        return cipher.getBlockSize();
    }
}
