/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.compression;
import java.util.*;
import com.sshtools.j2ssh.configuration.CompressionAlgorithm;
import com.sshtools.j2ssh.configuration.CompressionConfiguration;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.configuration.SshAPIConfiguration;
import com.sshtools.j2ssh.transport.AlgorithmNotSupportedException;
/**
 *  <p>
 *
 *  Creates new instances of SshCompression objects. Will load additional
 *  compression methods from the SSH API configuration file ssh.xml as long as
 *  they follow the name@domain syntax.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@version    1.0
 */

public class SshCompressionFactory {

    /**
     *  Defines the none method
     */
    public final static String COMP_NONE = "none";
    private static String defaultAlgorithm;
    private static Map comps;

    static {

        comps = new HashMap();

        comps.put(COMP_NONE, "");

        // Now load external compression from configuration file
        SshAPIConfiguration config = ConfigurationLoader.getAPIConfiguration();

        CompressionConfiguration addons = config.getCompressionConfiguration();

        if (addons != null) {

            List list = addons.getCompressionAlgorithm();

            if (list != null) {

                Iterator it = list.iterator();

                while (it.hasNext()) {

                    CompressionAlgorithm algorithm = (CompressionAlgorithm) it.next();

                    String name = algorithm.getAlgorithmName();

                    // Make sure it follows the extensibility rules
                    //if (name.lastIndexOf("@") > 0) {
                        comps.put(algorithm.getAlgorithmName(), algorithm.getImplementationClass());
                    //}
                }
            }

            defaultAlgorithm = addons.getDefaultAlgorithm();

            if (defaultAlgorithm == null || !comps.containsKey(defaultAlgorithm)) {
                defaultAlgorithm = COMP_NONE;
            }
        }
    }


    /**
     *  <p>
     *
     *  Returns the default compression method.</p>
     *
     *@return
     */
    public static String getDefaultCompression() {
        return defaultAlgorithm;
    }


    /**
     *  Constructor for the SshCompressionFactory object
     */
    protected SshCompressionFactory() { }


    /**
     *  <p>
     *
     *  Returns the supported compression methods available.</p>
     *
     *@return    The supported compressions.
     */
    public static List getSupportedCompression() {

        return new ArrayList(comps.keySet());
    }


    /**
     *  <p>
     *
     *  Create a new instance of the compression method.</p>
     *
     *@param  algorithmName                       The compression algorithm to
     *      create
     *@return                                     The new instance
     *@exception  AlgorithmNotSupportedException  Thrown if the algorithm is not
     *      supported
     */
    public static SshCompression newInstance(String algorithmName)
             throws AlgorithmNotSupportedException {

        String className = "";
        try {
            className = (String) comps.get(algorithmName);

            if ((className != null)) {
                if (className.equals("")) {
                    return null;
                } else {
                    return (SshCompression) Class.forName(className).newInstance();
                }
            } else {
                throw new AlgorithmNotSupportedException(algorithmName + " is not supported");
            }
        } catch (Exception e) {
            throw new AlgorithmNotSupportedException("Could not locate " + className);
        }

    }
}
