/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.hmac;

import javax.crypto.*;
import javax.crypto.spec.*;
import java.security.*;
import java.security.spec.*;
import com.sshtools.j2ssh.transport.AlgorithmInitializationException;

/**
 *  <p>
 *
 *  Implements the SHA Hash Message authentication algorithm</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class HmacSha implements SshHmac {

    private Mac mac;


    /**
     *  Constructor for the HmacSha object
     */
    public HmacSha() { }


    /**
     *  Initates the message authentication with the supplied key data
     *
     *@param  keydata                               Key data produced during key
     *      exchange
     *@exception  AlgorithmInitializationException  Thrown if the mac cannot be
     *      initialized
     */
    public void init(byte[] keydata) throws AlgorithmInitializationException {

        try {

            mac = Mac.getInstance("HmacSha1");
            byte[] key = new byte[20];
            System.arraycopy(keydata, 0, key, 0, 20);
            SecretKeySpec keyspec = new SecretKeySpec(key, "HmacSha1");

            mac.init(keyspec);

        } catch (NoSuchAlgorithmException nsae) {
            throw new AlgorithmInitializationException("No provider exists for the HmacSha1 algorithm");
        } catch (InvalidKeyException ike) {
            throw new AlgorithmInitializationException("Invalid key");
        }

    }


    /**
     *  Generates the mac
     *
     *@param  sequenceNo  The sequence no of the message
     *@param  data        The message data
     *@param  offset      Description of the Parameter
     *@param  len         Description of the Parameter
     *@return             The mac
     */
    public byte[] generate(long sequenceNo, byte[] data, int offset, int len) {

        // Write the sequence no
        byte[] sequenceBytes = new byte[4];
        sequenceBytes[0] = (byte) (sequenceNo >> 24);
        sequenceBytes[1] = (byte) (sequenceNo >> 16);
        sequenceBytes[2] = (byte) (sequenceNo >> 8);
        sequenceBytes[3] = (byte) (sequenceNo >> 0);

        mac.update(sequenceBytes);

        mac.update(data, offset, len);

        return mac.doFinal();
    }


    /**
     *  Verifies the mac received
     *
     *@param  sequenceNo  The sequence no of the message
     *@param  data        The message data
     *@return             The result of the verification
     */
    public boolean verify(long sequenceNo, byte[] data) {

        int len = mac.getMacLength();

        byte[] generated = generate(sequenceNo, data, 0, data.length - len);

        String compare1 = new String(generated);
        String compare2 = new String(data, data.length - len, len);

        return compare1.equals(compare2);
    }


    /**
     *  Gets the mac length
     *
     *@return    The mac length
     */
    public int getMacLength() {
        return mac.getMacLength();
    }
}

