/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.hmac;
import java.util.*;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.configuration.MacAlgorithm;
import com.sshtools.j2ssh.configuration.MacConfiguration;
import com.sshtools.j2ssh.configuration.SshAPIConfiguration;
import com.sshtools.j2ssh.transport.AlgorithmNotSupportedException;
/**
 *  <p>
 *
 *  This factory object creates instances of SshHmac objects. The standard
 *  algorithms are supported as well as additional classes described in the
 *  ssh.xml configuration file. These algorithms must follow the extensibility
 *  naming convention as described in the SSH protocol specification (i.e
 *  algorithm@company.com).</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class SshHmacFactory {

    /**
     *  The hmac-sha1 method
     */
    public final static String HMAC_SHA = "hmac-sha1";
    private static String defaultAlgorithm;
    private static Map macs;

    static {
        macs = new HashMap();

        macs.put(HMAC_SHA, HmacSha.class.getName());

        SshAPIConfiguration config = ConfigurationLoader.getAPIConfiguration();

        MacConfiguration addons = config.getMacConfiguration();

        if (addons != null) {

            List supported = addons.getMacAlgorithm();

            if (macs != null) {

                Iterator it = supported.iterator();

                while (it.hasNext()) {

                    MacAlgorithm mac = (MacAlgorithm) it.next();

                    String name = mac.getAlgorithmName();

                    // Make sure it follows the extensibility rules
                    //if (name.lastIndexOf("@") > 0) {
                        macs.put(mac.getAlgorithmName(), mac.getImplementationClass());
                    //}
                }
            }

            defaultAlgorithm = addons.getDefaultAlgorithm();

            if (defaultAlgorithm == null || !macs.containsKey(defaultAlgorithm)) {
                defaultAlgorithm = HMAC_SHA;
            }
        }
    }


    /**
     *  Gets the default message authentication method
     *
     *@return
     */
    public final static String getDefaultHmac() {
        return defaultAlgorithm;
    }



    /**
     *  Constructor for the SshHmacFactory object
     */
    protected SshHmacFactory() { }


    /**
     *  Gets the supported message authentication methods
     *
     *@return    The supported message authentication methods
     */
    public static List getSupportedMacs() {

        return new ArrayList(macs.keySet());
    }


    /**
     *  Creates a new instance of the message autentication method
     *
     *@param  methodName                          The method name
     *@return                                     The new instance
     *@exception  AlgorithmNotSupportedException  Thrown if the method is not
     *      supported
     */
    public static SshHmac newInstance(String methodName)
             throws AlgorithmNotSupportedException {

        String className = "";
        try {
            className = (String) macs.get(methodName);

            if (className != null) {
                return (SshHmac) Class.forName(className).newInstance();
            } else {
                throw new AlgorithmNotSupportedException(methodName + " is not supported");
            }
        } catch (Exception e) {
            throw new AlgorithmNotSupportedException("Could not locate " + className);
        }

    }
}
