/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.kex;
import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import org.apache.log4j.Logger;
import java.security.SecureRandom;
import com.sshtools.j2ssh.SshException;
import com.sshtools.j2ssh.transport.MessageAlreadyRegisteredException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.j2ssh.transport.TransportProtocol;
import com.sshtools.util.Hash;
/**
 *  <p>
 *
 *  Implements the diffie-hellman-group1-sha1 key exchange method as described
 *  in the transport protocol specification [SSH-TRANS]</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class DhGroup1Sha1 extends SshKeyExchange {

    private String clientId;
    private String serverId;
    private byte[] clientKexInit;
    private byte[] serverKexInit;
    private TransportProtocol transport;

    private byte[] hostKey;
    private static Logger log = Logger.getLogger(DhGroup1Sha1.class);

    private BigInteger e = null;
    /**
     *  The value f produced during diffie hellman key exchange
     */
    private BigInteger f = null;
    /**
     *  The secret value k produced during diffie hellman key exchange
     */
    private BigInteger k = null;
    /**
     *  The random value x used during diffie hellman key exchange
     */
    private BigInteger x = null;
    /**
     *  The value g used during diffie hellman key exchange
     */
    private BigInteger g = new BigInteger("2");
    /**
     *  The prime used in diffie hellman key exchange
     */
    private BigInteger p = new BigInteger("FFFFFFFFFFFFFFFFC90FDAA22168C234" +
            "C4C6628B80DC1CD129024E088A67CC74" +
            "020BBEA63B139B22514A08798E3404DD" +
            "EF9519B3CD3A431B302B0A6DF25F1437" +
            "4FE1356D6D51C245E485B576625E7EC6" +
            "F44C42E9A637ED6B0BFF5CB6F406B7ED" +
            "EE386BFB5A899FA5AE9F24117C4B1FE6" +
            "49286651ECE65381FFFFFFFFFFFFFFFF", 16);
    /**
     *  The value q used in diffie hellman key exchange
     */

    private BigInteger q = new BigInteger("7FFFFFFFFFFFFFFFE487ED5110B4611A" +
            "62633145C06E0E68948127044533E63A" +
            "0105DF531D89CD9128A5043CC71A026E" +
            "F7CA8CD9E69D218D98158536F92F8A1B" +
            "A7F09AB6B6A8E122F242DABB312F3F63" +
            "7A262174D31BF6B585FFAE5B7A035BF6" +
            "F71C35FDAD44CFD2D74F9208BE258FF3" +
            "24943323F67329C0FFFFFFFFFFFFFFFF", 16);


    /**
     *  Constructor for the DhGroup1Sha1 object
     */

    public DhGroup1Sha1() { }


    /**
     *  Called by the framework to initate the key exchange.
     *
     *@param  transport      The transport protocol object for sending/receiving
     *@param  clientId       The client identication string from protocol
     *      negotiation
     *@param  serverId       The servers identification string from protocol
     *      negotiation
     *@param  clientKexInit  The clients SSH_MSG_KEX_INIT payload
     *@param  serverKexInit  The servers SSH_MSG_KEX_INIT payload
     */
    public void init(TransportProtocol transport,
            String clientId, String serverId, byte[] clientKexInit, byte[] serverKexInit) {

        this.transport = transport;
        this.clientId = clientId;
        this.serverId = serverId;
        this.clientKexInit = clientKexInit;
        this.serverKexInit = serverKexInit;

    }


    /**
     *  Called by the framework to start the client side of the key exchange
     *  method.
     *
     *@exception  KeyExchangeException  Thrown if a key exchange error occurs
     */
    public void startClient() throws KeyExchangeException {

        log.info("Starting client side key exchange.");

        try {
            transport.registerMessage(new Integer(SshMsgKexDhReply.SSH_MSG_KEXDH_REPLY),
                    SshMsgKexDhReply.class,
                    messageStore);
        } catch (MessageAlreadyRegisteredException e) {
            throw new KeyExchangeException("SSH_MSG_KEXDH_REPLY msg already registered with SshMessageManager");
        }

        int minBits = g.bitLength();
        int maxBits = q.bitLength();

        SecureRandom rnd = new SecureRandom();

        // Generate a random bit count for the random x value
        int genBits = (int) (((maxBits - minBits + 1) * rnd.nextFloat()) + minBits);

        x = new BigInteger(genBits, rnd);

        // Calculate e
        e = g.modPow(x, p);

        // Prepare the message
        SshMsgKexDhInit msg = new SshMsgKexDhInit(e);

        // Send it
        try {
            transport.sendMessage(msg, this);
        } catch (SshException e) {

        }

    }


    /**
     *  Called by the framework to start the server side of the key exchange
     *
     *@exception  KeyExchangeException  Thrown if a key exchange error occurs
     */
    public void startServer() throws KeyExchangeException {

        try {
            transport.registerMessage(new Integer(SshMsgKexDhInit.SSH_MSG_KEXDH_INIT),
                    SshMsgKexDhInit.class,
                    messageStore);
        } catch (MessageAlreadyRegisteredException e) {
            throw new KeyExchangeException("SSH_MSG_KEXDH_INIT msg already registered with SshMessageManager");
        }

    }


    /**
     *  Called by the framework when a registered message is received.
     *
     *@param  msg  The message received
     */
    protected void onMessageReceived(SshMessage msg) {

        switch (msg.getMessageId()) {
            case SshMsgKexDhInit.SSH_MSG_KEXDH_INIT:
            {
                onMsgKexDhInit((SshMsgKexDhInit) msg);
                break;
            }
            case SshMsgKexDhReply.SSH_MSG_KEXDH_REPLY:
            {
                onMsgKexDhReply((SshMsgKexDhReply) msg);
                break;
            }
        }

    }


    /**
     *  Sent by the client, this message initiates the diffie hellman key
     *  exchange by providing the e value.
     *
     *@param  msg  Description of the Parameter
     */
    private void onMsgKexDhInit(SshMsgKexDhInit msg) { }


    /**
     *  The server sends this message with its host key and the signature to be
     *  verified.
     *
     *@param  msg  The message received
     */
    private void onMsgKexDhReply(SshMsgKexDhReply msg) {

        byte[] exchangeHash;
        byte[] hostKey = msg.getHostKey();
        byte[] signature = msg.getSignature();

        f = msg.getF();

        // Calculate diffe hellman k value
        k = f.modPow(x, p);

        // Calculate the exchange hash
        try {
            exchangeHash = calculateExchangeHash(hostKey);

        } catch (SshException e) {
            setFailed(e.getMessage());
            return;
        }

        // Verify the signature
        setComplete(exchangeHash, hostKey, signature, k);

    }



    /**
     *  This method is during key exchange to calculate the exchange hash The
     *  exchange hash is computed as the concatenation of the following: The
     *  clients identification string, The servers identification string, The
     *  payload of the clients SSH_MSG_KEXINIT, The payload of the servers
     *  SSH_MSG_KEX_INIT, The servers host key, The diffie hellman e value, The
     *  diffie hellman f value, The diffie hellman k value.
     *
     *@param  hostKey                   The servers host key
     *@return                           The exchange hash output
     *@exception  KeyExchangeException  Thrown if an error occurs
     */
    protected byte[] calculateExchangeHash(byte[] hostKey)
             throws KeyExchangeException {

        Hash hash;
        try {
            // Start a SHA hash
            hash = new Hash("SHA");
        } catch (NoSuchAlgorithmException nsae) {
            throw new KeyExchangeException("SHA algorithm not supported");
        }

        int i;

        // The local software version comments
        hash.putString(clientId);

        // The remote software version comments
        hash.putString(serverId);

        // The local kex init payload
        hash.putInt(clientKexInit.length);
        hash.putBytes(clientKexInit);

        // The remote kex init payload
        hash.putInt(serverKexInit.length);
        hash.putBytes(serverKexInit);

        // The host key
        hash.putInt(hostKey.length);
        hash.putBytes(hostKey);

        // The diffie hellman e value
        hash.putBigInteger(e);

        // The diffie hellman f value
        hash.putBigInteger(f);

        // The diffie hellman k value
        hash.putBigInteger(k);

        // Do the final output
        return hash.doFinal();
    }

}
