/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.kex;

import java.math.BigInteger;

/**
 *  <p>
 *
 *  The key exchange state object is synchronized so that a thread can wait for
 *  the key exchange to complete (or fail)</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class KeyExchangeState {

    /**
     *  The key exchange is currently in progress
     */
    public final static int IN_PROGRESS = 0;

    /**
     *  The key exchange completed
     */
    public final static int COMPLETE = 1;

    /**
     *  The key exchange failed. Use <code>getFailedReason</code> to examine the
     *  failure reason.
     */
    public final static int FAILED = 2;

    private int state = IN_PROGRESS;
    private byte[] exchangeHash;
    private byte[] hostKey;
    private byte[] signature;
    private BigInteger secret;
    private String reason;


    /**
     *  Constructs the KeyExchangeState
     */
    public KeyExchangeState() { }


    /**
     *  Gets the exchange hash output of the key exchange
     *
     *@return
     */
    public byte[] getExchangeHash() {
        return exchangeHash;
    }


    /**
     *  Gets the host key supplied by the server during key exchange
     *
     *@return
     */
    public byte[] getHostKey() {
        return hostKey;
    }


    /**
     *  Gets the signature supplied during key exchange
     *
     *@return
     */
    public byte[] getSignature() {
        return signature;
    }


    /**
     *  Gets the current state of the key exchange
     *
     *@return
     */
    public synchronized int getState() {
        return state;
    }


    /**
     *  Gets the secret value produced during key exchange
     *
     *@return
     */
    public BigInteger getSecret() {
        return secret;
    }


    /**
     *  Sets the key exchange to complete
     *
     *@param  exchangeHash  The hash output
     *@param  hostKey       The servers host key
     *@param  signature     The signature to verify the host key with
     *@param  secret        The secret key exchange value
     */
    public final synchronized void setComplete(byte[] exchangeHash, byte[] hostKey,
            byte[] signature, BigInteger secret) {

        this.exchangeHash = exchangeHash;
        this.hostKey = hostKey;
        this.signature = signature;
        this.secret = secret;
        state = COMPLETE;

        notifyAll();

    }


    /**
     *  Sets the state to failed
     *
     *@param  reason  The reason for failure
     */
    public final synchronized void setFailed(String reason) {

        this.reason = reason;

        state = FAILED;

        notifyAll();
    }


    /**
     *  A Thread can call this method to wait for the completetion of key
     *  exchange
     */
    public final synchronized void waitForCompletion() {

        while (state == IN_PROGRESS) {

            try {

                wait();
            } catch (InterruptedException e) {

            }

        }

    }


    /**
     *  Gets the failure reason
     *
     *@return
     */
    public synchronized String getFailureReason() {
        return reason;
    }

}
