/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.kex;
import java.math.BigInteger;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.j2ssh.transport.SshMessageStore;
import com.sshtools.j2ssh.transport.TransportProtocol;

/**
 *  <p>
 *
 *  This class provides a framework for implementing SSH protocol key exchange
 *  methods for the API.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public abstract class SshKeyExchange implements Runnable {

    /**
     *  The methods message store
     */
    protected SshMessageStore messageStore = new SshMessageStore();
    private Thread thread = new Thread(this);
    private boolean keepRunning = true;
    private KeyExchangeState state = new KeyExchangeState();


    /**
     *  Constructor for the KeyExchange object
     */
    public SshKeyExchange() {

        thread.start();
    }


    /**
     *  Sets the key exchanges state to complete
     *
     *@param  exchangeHash  The output of the key exchange
     *@param  hostKey       The servers host key
     *@param  signature     The signature
     *@param  secret        The secret value output of the key exchange
     */
    protected void setComplete(byte[] exchangeHash, byte[] hostKey, byte[] signature, BigInteger secret) {
        state.setComplete(exchangeHash, hostKey, signature, secret);
    }


    /**
     *  Sets the key exchanges state to failed
     *
     *@param  reason  The new failed value
     */
    protected void setFailed(String reason) {
        state.setFailed(reason);
    }


    /**
     *  Gets the state of the key exchange
     *
     *@return
     */
    public KeyExchangeState getState() {
        return state;
    }


    /**
     *  Main processing method for the KeyExchange object
     */
    public void run() {

        try {
            while (keepRunning) {

                SshMessage msg = messageStore.getMessage();

                onMessageReceived(msg);

            }
        } catch (KeyExchangeException ke) {
            setFailed(ke.getMessage());
        }
    }


    /**
     *  Stops the key exchange
     */
    public void stop() {
        keepRunning = false;

        thread.interrupt();
    }


    /**
     *  Initiates the key exchange object before starting
     *
     *@param  transport      The transport protocol for sending/receiving
     *@param  clientId       The clients identification string
     *@param  serverId       The servers identification string
     *@param  clientKexInit  The clients SSH_MSG_KEX_INIT payload
     *@param  serverKexInit  The servers SSH_MSG_KEX_INIT payload
     */
    public abstract void init(TransportProtocol transport,
            String clientId, String serverId, byte[] clientKexInit, byte[] serverKexInit);


    /**
     *  Starts the client side of the key exchange
     *
     *@exception  KeyExchangeException  Throw if a key exchange error occures
     */
    public abstract void startClient() throws KeyExchangeException;


    /**
     *  Starts the server side of the key exchange
     *
     *@exception  KeyExchangeException  Throw if a key exchange error occurs
     */
    public abstract void startServer() throws KeyExchangeException;


    /**
     *  Called by the framwork when a registered message is recieved.
     *
     *@param  msg                       The message received
     *@exception  KeyExchangeException  Description of the Exception
     */
    protected abstract void onMessageReceived(SshMessage msg) throws KeyExchangeException;

}
