/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.kex;

import java.util.*;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.configuration.KeyExchangeAlgorithm;
import com.sshtools.j2ssh.configuration.KeyExchangeConfiguration;
import com.sshtools.j2ssh.configuration.SshAPIConfiguration;
import com.sshtools.j2ssh.transport.AlgorithmNotSupportedException;

/**
 *  <p>
 *
 *  Creates new instances of SshKeyExchange objects. Will load additional key
 *  exchange methods from the SSH API configuration file ssh.xml as long as they
 *  follow the name@domain syntax.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class SshKeyExchangeFactory {

    /**
     *  The diffie-hellman-group1-sha1 method
     */
    public final static String DIFFIE_HELLMAN_GROUP1_SHA1 = "diffie-hellman-group1-sha1";

    private static Map kexs;
    private static String defaultAlgorithm;

    static {

        kexs = new HashMap();

        kexs.put(DIFFIE_HELLMAN_GROUP1_SHA1, DhGroup1Sha1.class.getName());

        // Now load external compression from configuration file
        SshAPIConfiguration config = ConfigurationLoader.getAPIConfiguration();

        KeyExchangeConfiguration addons = config.getKeyExchangeConfiguration();

        if (addons != null) {

            List list = addons.getKeyExchangeAlgorithm();

            if (list != null) {

                Iterator it = list.iterator();

                while (it.hasNext()) {

                    KeyExchangeAlgorithm algorithm = (KeyExchangeAlgorithm) it.next();

                    String name = algorithm.getAlgorithmName();

                    // Make sure it follows the extensibility rules
                    //if (name.lastIndexOf("@") > 0) {
                        kexs.put(algorithm.getAlgorithmName(), algorithm.getImplementationClass());
                    //}
                }
            }

            defaultAlgorithm = addons.getDefaultAlgorithm();

            if (defaultAlgorithm == null || !kexs.containsKey(defaultAlgorithm)) {
                defaultAlgorithm = DIFFIE_HELLMAN_GROUP1_SHA1;
            }
        }
    }


    /**
     *  Gets the default key exchange method name
     *
     *@return
     */
    public static String getDefaultKeyExchange() {
        return defaultAlgorithm;
    }


    /**
     *  Constructor for the SshKeyExchangeFactory object
     */
    protected SshKeyExchangeFactory() { }


    /**
     *  Gets the supported key exchange methods
     *
     *@return    The supported key exchanges
     */
    public static List getSupportedKeyExchanges() {

        return new ArrayList(kexs.keySet());
    }


    /**
     *  Creates a new instance of the key exchange method specified
     *
     *@param  methodName                          The key exchange method
     *@return                                     The new instance
     *@exception  AlgorithmNotSupportedException  Thrown if the method is not
     *      supported
     */
    public static SshKeyExchange newInstance(String methodName)
             throws AlgorithmNotSupportedException {

        String className = "";
        try {
            className = (String) kexs.get(methodName);

            if ((className != null)) {
                if (className.equals("")) {
                    return null;
                } else {
                    return (SshKeyExchange) Class.forName(className).newInstance();
                }
            } else {
                throw new AlgorithmNotSupportedException(methodName + " is not supported");
            }
        } catch (Exception e) {
            throw new AlgorithmNotSupportedException("Could not locate " + className);
        }

    }
}
