/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.kex;

import java.io.IOException;
import java.math.BigInteger;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  Implements the diffie-hellman-group1-sha1 key exchange init message.</p> <b>
 *  <p>
 *
 *  byte SSH_MSG_KEXDH_INIT<br>
 *  mpint e</p> </b> <p>
 *
 *  The Diffie-Hellman key exchange provides a shared secret that can not be
 *  determined by either party alone. The key exchange is combined with a
 *  signature with the host key to provide host authentication.</p> <p>
 *
 *  The client generates a random number x (1 < x < q) and computes e = g^x mod
 *  p. The client sends e to the server.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class SshMsgKexDhInit extends SshMessage {

    // Stores the e value
    private BigInteger e;

    /**
     *  The message id for this message
     */
    protected final static int SSH_MSG_KEXDH_INIT = 30;


    /**
     *  Constructs the message ready for sending
     *
     *@param  e  The diffie hellman e value
     */
    public SshMsgKexDhInit(BigInteger e) {
        super(SSH_MSG_KEXDH_INIT);
        this.e = e;
    }


    /**
     *  Gets the message name for debugging
     *
     *@return
     */
    public String getMessageName() {
        return "SSH_MSG_KEXDH_INIT";
    }


    /**
     *  Constructs the message from data received
     */
    public SshMsgKexDhInit() {
        super(SSH_MSG_KEXDH_INIT);
    }


    /**
     *  Abstract method implementation to construct the message from a byte
     *  array.
     *
     *@param  bar                          The byte array containing the message
     *      data.
     *@exception  InvalidMessageException  Thrown is the data cannot be read.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {
            e = bar.readBigInteger();
        } catch (IOException ioe) {

            throw new InvalidMessageException("Error reading message data: " + ioe.getMessage());
        }

    }


    /**
     *  Abstract method implementation to create a byte array containing the
     *  message.
     *
     *@param  baw                          The byte array to write to.
     *@exception  InvalidMessageException  Thrown if the data cannot be written.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {
        try {

            baw.writeBigInteger(e);

        } catch (IOException ioe) {

            throw new InvalidMessageException("Error writing message data: " + ioe.getMessage());
        }
    }


    /**
     *  Gets the diffie hellman e value
     *
     *@return    The e value
     */
    public BigInteger getE() {

        return e;
    }
}
