/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.kex;

import java.io.IOException;
import java.math.BigInteger;
import com.sshtools.j2ssh.transport.InvalidMessageException;
import com.sshtools.j2ssh.transport.SshMessage;
import com.sshtools.util.ByteArrayWriter;
import com.sshtools.util.ByteArrayReader;

/**
 *  <p>
 *
 *  The server responds to a SSH_MSG_KEXDH_INIT with the following:</p> <b><p>
 *
 *  byte SSH_MSG_KEXDH_REPLY<br>
 *  string server public host key and certificates (K_S)<br>
 *  mpint f<br>
 *  string signature of H</p> </b> <p>
 *
 *  The hash H is computed as the HASH hash of the concatenation of the
 *  following:</p> <p>
 *
 *  string V_C, the client's version string (CR and NL excluded)<br>
 *  string V_S, the server's version string (CR and NL excluded)<br>
 *  string I_C, the payload of the client's SSH_MSG_KEXINIT<br>
 *  string I_S, the payload of the server's SSH_MSG_KEXINIT<br>
 *  string K_S, the host key<br>
 *  mpint e, exchange value sent by the client<br>
 *  mpint f, exchange value sent by the server<br>
 *  mpint K, the shared secret</p> <p>
 *
 *  This value is called the exchange hash, and it is used to authenticate the
 *  key exchange. The exchange hash SHOULD be kept secret.</p> <p>
 *
 *  The signature algorithm MUST be applied over H, not the original data. Most
 *  signature algorithms include hashing and additional padding. For example,
 *  "ssh-dss" specifies SHA-1 hashing; in that case, the data is first hashed
 *  with HASH to compute H, and H is then hashed with SHA-1 as part of the
 *  signing operation.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class SshMsgKexDhReply extends SshMessage {

    // The host key data
    private byte[] hostKey;
    // The diffie hellman f value
    private BigInteger f;
    // The signature
    private byte[] signature;

    /**
     *  The message id of the message
     */
    protected final static int SSH_MSG_KEXDH_REPLY = 31;


    /**
     *  Constructs the message ready for sending.
     *
     *@param  hostKey    The servers host key data
     *@param  f          The diffie hellman f value
     *@param  signature  The signature to verify ownership of a private key
     */
    public SshMsgKexDhReply(byte[] hostKey, BigInteger f, byte[] signature) {
        super(SSH_MSG_KEXDH_REPLY);
        this.hostKey = hostKey;
        this.f = f;
        this.signature = signature;
    }


    /**
     *  Gets the messageName attribute of the SshMsgKexDhReply object
     *
     *@return    The messageName value
     */
    public String getMessageName() {
        return "SSH_MSG_KEXDH_REPLY";
    }


    /**
     *  Constructs the message from data received.
     */
    public SshMsgKexDhReply() {
        super(SSH_MSG_KEXDH_REPLY);
    }


    /**
     *  Abstract method implementation to construct the message from a byte
     *  array.
     *
     *@param  bar                          The byte array containing the data.
     *@exception  InvalidMessageException  Thrown if the data cannot be read.
     */
    protected void constructMessage(ByteArrayReader bar) throws InvalidMessageException {

        try {

            hostKey = bar.readBinaryString();
            f = bar.readBigInteger();
            signature = bar.readBinaryString();

        } catch (IOException ioe) {

            throw new InvalidMessageException("Error reading message data: " + ioe.getMessage());
        }
    }


    /**
     *  Abstract method implementation to construct a byte array containing the
     *  message.
     *
     *@param  baw                          The byte array being written to.
     *@exception  InvalidMessageException  Thrown if the data cannot be written.
     */
    protected void constructByteArray(ByteArrayWriter baw) throws InvalidMessageException {

        try {

            baw.writeBinaryString(hostKey);
            baw.writeBigInteger(f);
            baw.writeBinaryString(signature);
        } catch (IOException ioe) {

            throw new InvalidMessageException("Error writing message data: " + ioe.getMessage());
        }
    }


    /**
     *  Gets the hostKey attribute of the message
     *
     *@return    The hostKey value
     */
    public byte[] getHostKey() {
        return hostKey;
    }


    /**
     *  Gets the f attribute of the message
     *
     *@return    The f value
     */
    public BigInteger getF() {
        return f;
    }


    /**
     *  Gets the signature attribute of the message
     *
     *@return    The signature value
     */
    public byte[] getSignature() {
        return signature;
    }
}
