/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.publickey;

import java.security.spec.DSAPublicKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.PublicKey;
import java.security.KeyFactory;
import java.security.Signature;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import java.math.BigInteger;
import java.io.IOException;

import beecrypt.asn1.x957.*;
import beecrypt.asn1.*;
import org.apache.log4j.Logger;
import com.sshtools.util.ByteArrayReader;
import com.sshtools.util.Hash;

/**
 *  <p>
 *
 *  </p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class SshDss implements SshPublicKey {

    private String header;
    private DSAPublicKeySpec dsaKey;
    private PublicKey pk;
    private byte[] hostKey;


    /**
     *  Constructor for the SshDssPublicKey object
     *
     *@param  hostKey                      The host key data
     *@exception  InvalidHostKeyException  Thrown of the hopst key is invalid
     */
    public void init(byte[] hostKey) throws InvalidHostKeyException {

        try {

            this.hostKey = hostKey;

            // Extract the key information
            ByteArrayReader bar = new ByteArrayReader(hostKey);

            header = bar.readString();
            BigInteger p = bar.readBigInteger();
            BigInteger q = bar.readBigInteger();
            BigInteger g = bar.readBigInteger();
            BigInteger y = bar.readBigInteger();

            dsaKey = new DSAPublicKeySpec(y, p, q, g);

            try {
                KeyFactory kf = KeyFactory.getInstance("DSA");
                pk = kf.generatePublic(dsaKey);
            } catch (NoSuchAlgorithmException nsae) {
                throw new InvalidHostKeyException();
            } catch (InvalidKeySpecException ikpe) {
                throw new InvalidHostKeyException();
            }

        } catch (IOException ioe) {

            throw new InvalidHostKeyException();
        }
    }


    /**
     *  Called by the framework to verify the signature received during key
     *  exchange
     *
     *@param  signature                             The signature to verify
     *@param  exchangeHash                          The exchange hash produced
     *      during key exchange
     *@return                                       The result of the
     *      verification
     *@exception  InvalidHostKeySignatureException  Thrown if the signature is
     *      in an invalid format
     */
    public boolean verifySignature(byte[] signature, byte[] exchangeHash)
             throws InvalidHostKeySignatureException {

        try {

            ByteArrayReader bar = new ByteArrayReader(signature);

            byte[] data = bar.readBinaryString();

            if (data.length != 40) {

                // OpenSSH DSA Signature expected
                String header = new String(data);

                if (!header.equals("ssh-dss")) {
                    throw new InvalidHostKeySignatureException();
                }

                data = bar.readBinaryString();
            }

            bar = new ByteArrayReader(data);

            byte[] raw = new byte[20];

            bar.read(raw);
            BigInteger r = new BigInteger(raw);

            bar.read(raw);
            BigInteger s = new BigInteger(raw);

            DssSigVerification sshdss = new DssSigVerification(r,s);

            DEREncoder der = new DEREncoder();

            byte[] encoded = der.encode(sshdss);

            Signature sig = Signature.getInstance("SHA1withDSA");

            sig.initVerify(pk);

            sig.update(exchangeHash);

            return sig.verify(encoded);
        } catch (NoSuchAlgorithmException nsae) {
            throw new InvalidHostKeySignatureException();
        } catch (IOException ioe) {
            throw new InvalidHostKeySignatureException();
        } catch (SignatureException se) {
            throw new InvalidHostKeySignatureException();
        } catch (InvalidKeyException ike) {
            throw new InvalidHostKeySignatureException();
        }

    }


    /**
     *  Gets the keys fingerprint
     *
     *@return    The fingerprint
     */
    public String getFingerprint() {

        try {

            Hash md5 = new Hash("MD5");

            md5.putString("ssh-dss");

            md5.putBigInteger(dsaKey.getP());

            md5.putBigInteger(dsaKey.getQ());

            md5.putBigInteger(dsaKey.getG());

            md5.putBigInteger(dsaKey.getY());

            byte[] digest = md5.doFinal();

            String ret = String.valueOf(dsaKey.getP().bitLength());

            for (int i = 0; i < digest.length; i++) {

                ret += (i == 0 ? ":" : "") + " " + java.lang.Integer.toHexString(digest[i] & 0xFF);
            }

            return ret;
        } catch (NoSuchAlgorithmException nsae) {

            return "";
        }
    }


    /**
     *  Gets the formatted key
     *
     *@return    The formatted key
     */
    public String getFormattedKey() {
        return "";
    }
}


class DssSigVerification extends DssSigValue {

  DssSigVerification( BigInteger r, BigInteger s) {

    values = new beecrypt.asn1.Integer[2];

    values[0] = new beecrypt.asn1.Integer(r);
    values[1] = new beecrypt.asn1.Integer(s);

  }


}
