/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.j2ssh.transport.publickey;

import java.util.*;
import com.sshtools.j2ssh.configuration.ConfigurationLoader;
import com.sshtools.j2ssh.configuration.PublicKeyAlgorithm;
import com.sshtools.j2ssh.configuration.PublicKeyConfiguration;
import com.sshtools.j2ssh.configuration.SshAPIConfiguration;
import com.sshtools.j2ssh.transport.AlgorithmNotSupportedException;

/**
 *  <p>
 *
 *  This factory object creates instances of SshPublicKey objects. Furhter
 *  public key mechanisms can be configured by placing an entry in the SSH API
 *  configuration file ssh.xml. The method names must follow the extensibility
 *  naming convention as described in the SSH protocol specification (i.e
 *  algorithm@company.com).</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class SshPublicKeyFactory {

    private static Map pks;
    private static String defaultAlgorithm;

    /**
     *  The ssh-dss public key mechanism
     */
    public final static String SSH_DSS = "ssh-dss";

    static {

        pks = new HashMap();

        pks.put(SSH_DSS, "com.jssh.publickey.SshDss");

        // Now load external pks from configuration file
        SshAPIConfiguration config = ConfigurationLoader.getAPIConfiguration();

        PublicKeyConfiguration addons = config.getPublicKeyConfiguration();

        if (addons != null) {

            List list = addons.getPublicKeyAlgorithm();

            if (list != null) {

                Iterator it = list.iterator();

                while (it.hasNext()) {

                    PublicKeyAlgorithm algorithm = (PublicKeyAlgorithm) it.next();

                    String name = algorithm.getAlgorithmName();

                    // Make sure it follows the extensibility rules
                    //if (name.lastIndexOf("@") > 0) {
                        pks.put(algorithm.getAlgorithmName(), algorithm.getImplementationClass());
                    //}
                }
            }

            defaultAlgorithm = addons.getDefaultAlgorithm();

            if (defaultAlgorithm == null || !pks.containsKey(defaultAlgorithm)) {
                defaultAlgorithm = SSH_DSS;
            }
        }
    }


    /**
     *  Gets the default public key mechanism
     *
     *@return
     */
    public static String getDefaultPublicKey() {
        return defaultAlgorithm;
    }


    /**
     *  Constructor for the SshPublicKeyFactory object
     */
    protected SshPublicKeyFactory() { }


    /**
     *  Gets the supported public key mechanisms
     *
     *@return    The supported mechanisms
     */
    public static List getSupportedPublicKeys() {

        // Get the list of pks
        return new ArrayList(pks.keySet());
    }


    /**
     *  Creates a new instance of the public key method
     *
     *@param  methodName                          Description of the Parameter
     *@return                                     The new instance
     *@exception  AlgorithmNotSupportedException  Thrown if the method is not
     *      supported
     */
    public static SshPublicKey newInstance(String methodName)
             throws AlgorithmNotSupportedException {

        String className = "";
        try {
            className = (String) pks.get(methodName);

            if (className != null) {
                return (SshPublicKey) Class.forName(className).newInstance();
            } else {
                throw new AlgorithmNotSupportedException(methodName + " is not supported");
            }
        } catch (Exception e) {
            throw new AlgorithmNotSupportedException("Could not locate " + className);
        }

    }
}
