/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.sshterm;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import com.sshtools.j2ssh.authentication.*;
import org.apache.log4j.*;
import com.sshtools.j2ssh.connection.*;
import com.sshtools.j2ssh.configuration.*;
import com.borland.jbcl.layout.*;
import javax.swing.border.*;
import com.sshtools.j2ssh.transport.*;
import javax.swing.event.*;
import com.sshtools.util.*;
import com.sshtools.j2ssh.*;
import com.sshtools.j2ssh.client.*;
import java.io.*;
import com.sshtools.sshterm.emulation.TerminalPanel;
import com.sshtools.sshterm.emulation.TerminalEmulation;
import com.sshtools.sshterm.BannerDialog;
import com.sshtools.sshterm.NewAction;
import com.sshtools.sshterm.OpenAction;
import com.sshtools.sshterm.SaveAction;
import com.sshtools.sshterm.StandardAction;


/**
 *  <p>
 *
 *  This frame implements an SSH Terminal
 *
 *@author     unascribed
 *@created    13 August 2002
 *@version    1.0
 */

public class SshTerminalFrame extends JFrame implements ActionListener {

    // The status bar
    private JLabel jLabelStatus;

    // Our frame icon
    private final String TERMINAL_ICON = "/com/sshtools/sshterm/sshterm.gif";

    // The mouse handler
    private MouseHandler mouseHandler;

    // The shell client
    private SessionClient session;

    // The terminal
    private TerminalPanel terminal;

    // Host key verification
    private HostKeyVerification hostKeyVerification;


    /**
     *  The emulation object
     */
    protected TerminalEmulation emulation;

    // Actions
    private NewAction newAction;
    private OpenAction openAction;
    private SaveAction saveAction;
    private CloseAction closeAction;
    private ExitAction exitAction;
    private Vector actions;

    // The current connection properties
    private SshConnectionProperties current;

    // The current status bar text
    private String statusText;

    // The content pane
    JPanel contentPane;

    // Our borders
    TitledBorder titledBorder1;
    BorderLayout borderLayout1 = new BorderLayout();


    /**
     *  Constructor for the SshTerminalFrame object
     */
    public SshTerminalFrame() {

        // Load the SSHTerm icon from the resources
        ResourceIcon icon = new ResourceIcon(TERMINAL_ICON);

        setIconImage(icon.getImage());

        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     *  Initiates the frame
     *
     *@exception  Exception
     */
    private void jbInit() throws Exception {

        //setIconImage(Toolkit.getDefaultToolkit().createImage(ClientFrame.class.getResource("[Your Icon]")));

        // Set up the actions
        initActions();

        // Create our host key verification answer back instance
        hostKeyVerification = createHostKeyVerification();

        // Add a window listener so that we can detect if the window closes
        addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent evt) {
                    exitApplication();
                }
            });

        // Create the status bar
        jLabelStatus = createStatusBar();
        mouseHandler = new MouseHandler(jLabelStatus);

        // Create the application menu
        this.setJMenuBar(createMenu());

        // Create our terminal emulation object
        emulation = createEmulation();

        // Set up the frame
        contentPane = (JPanel) this.getContentPane();
        contentPane.setLayout(borderLayout1);

        this.setSize(new Dimension(800, 600));
        this.setTitle("SSHTerm");
        this.setResizable(true);

        // Create our swing terminal and add it to the main frame
        terminal = new TerminalPanel(emulation);

        contentPane.add(terminal);
        contentPane.add(jLabelStatus, BorderLayout.SOUTH);

        // Make sure that the swing terminal has focus
        terminal.requestFocus();

    }


    /**
     *  Fired when an action is performed
     *
     *@param  event  The action performed
     */
    public void actionPerformed(ActionEvent event) {

        // Get the name of the action command
        String command = event.getActionCommand();

        if (command.equals(newAction.getActionCommand())) {

            // Clear the screen
            emulation.clearScreen();

            // Force a repaint before the dialog shows

            this.getContentPane().repaint();

            try {

                // Create a new connection dialog
                ConnectionDialog connect = new ConnectionDialog(this);

                // Show the dialog and return the properties selected
                SshConnectionProperties props = connect.showConnectionDialog();

                // Make sure we didn't cancel

                if (props != null) {
                    connect(props);
                }
            } catch (SshException se) {

                // Display the exception
                showExceptionMessage("An unexpected error occured!\n\n" + se.getMessage(),
                        "Unexcpected Error");
            }

            return;
        }

        if (command.equals(closeAction.getActionCommand())) {

            closeSession(true);
        }

        if (command.equals(openAction.getActionCommand())) {

            // Create a file chooser with the current directory set to the
            // application home
            JFileChooser fileDialog = new JFileChooser(System.getProperty("jssh.home"));

            // Show it
            int ret = fileDialog.showOpenDialog(this);

            // If we've approved the selection then process
            if (ret == fileDialog.APPROVE_OPTION) {

                // Get the file
                File f = fileDialog.getSelectedFile();

                // Make sure its not invalid
                if (f != null) {

                    // Create a new connection properties object
                    SshConnectionProperties props = new SshConnectionProperties();

                    try {

                        // Open the file
                        props.open(f.getAbsolutePath());

                        // Connect with the new details
                        connect(props);

                    } catch (InvalidConnectionFileException fnfe) {
                        showExceptionMessage(fnfe.getMessage(), "Open Connection");
                    } catch (SshException e) {
                        showExceptionMessage("An unexpected error occured!", "Open Connection");
                    }
                } else {
                    showExceptionMessage("Invalid file specified", "Open Connection");
                }
            }
        }

        if (command.equals(saveAction.getActionCommand())) {

            // Make sure we dont have a null connection object
            if (current != null) {

                // Select the file to save
                JFileChooser fileDialog = new JFileChooser(System.getProperty("jssh.home"));

                int ret = fileDialog.showSaveDialog(this);

                if (ret == fileDialog.APPROVE_OPTION) {

                    try {

                        // Get the selected file
                        File f = fileDialog.getSelectedFile();

                        // Check to make sure its valid
                        if (f != null) {
                            // Save the connection details to file
                            current.save(f.getAbsolutePath());
                        } else {
                            showExceptionMessage("The file specified is invalid!", "Save Connection");
                        }
                    } catch (InvalidConnectionFileException e) {
                        showExceptionMessage(e.getMessage(), "Save Connection");
                    }
                }
            }
        }

        if(command.equals(exitAction.getActionCommand())) {

          if(session!=null)
            if(JOptionPane.showConfirmDialog(this,
                            "Close the current session and exit?",
                            "Exit Application",
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE) == JOptionPane.NO_OPTION)
                                    return;


          exitApplication();
        }

    }

    /**
     * Close the session and reset all parameters
     */
    private void closeSession(boolean doDisconnect) {

      if(doDisconnect) {
      // We should disconnect the session
       try {
            session.close();
        } catch (SshException se) {
            showExceptionMessage("An unexpected error occured!\n\n" +
                    se.getMessage(), "Disconnect");
        }
      }
        // Reset the menu items
        setFileMenuStatus(false);

        // Null the session and current properties
        session = null;
        current = null;
    }
    /**
     *  Connect to the remote host
     *
     *@param  props             The connection properties to use
     *@exception  SshException
     */
    private void connect(SshConnectionProperties props) throws SshException {

        // We need to connect
        session = createSession();

        // Set the current connection properties
        current = props;

        // Set the terminal type
        emulation.setTerminalType(current.getTerminalType());

        // Connect the session
        session.connectAsynchronously(current, hostKeyVerification);

        // Update the status bar
        jLabelStatus.setText("Connecting to " + current.getHost() + "....");

        // Make sure the terminal object has focus
        terminal.requestFocus();

    }


    /**
     *  Inializes the action objects
     */
    private void initActions() {

        actions = new Vector();

        newAction = new NewAction();
        registerAction(newAction);

        openAction = new OpenAction();
        registerAction(openAction);

        saveAction = new SaveAction();
        registerAction(saveAction);

        closeAction = new CloseAction();
        registerAction(closeAction);

        exitAction = new ExitAction();
        registerAction(exitAction);

        setFileMenuStatus(false);
    }


    /**
     *  <p>
     *
     *  Create a HostKeyVerification Object. This object loads the default hosts
     *  file JSSH_HOME/hosts.xml and verifies the servers host key against the
     *  list contained in the said file.</p> <p>
     *
     *  If the host key is not recognized then the transport layer will call
     *  <code>onUnknownHost</code> so that the user can determine the correct
     *  course of action. If the user accepts the host key then a callis made to
     *  <code>allowHost</code> and then the transport layer reverifies the host
     *  key at which point it will be allowed. The option exists for the host to
     *  be permantley updated in the hosts file</> <p>
     *
     *  Similairy a call to <code>onHostKeyMismatch</code> notifies the user
     *  that the current host key does not match the key supplied. The same
     *  process again is used to allow the host.</p>
     *
     *@return                               The HostKeyVerificaiton object
     *@exception  InvalidHostFileException
     */
    private HostKeyVerification createHostKeyVerification()
             throws InvalidHostFileException {

        return
            new HostKeyVerification() {

                /**
                 *  Overide the method so that we can handle any unknown host
                 *  keys supplied by connected servers.
                 *
                 *@param  host         The host name of the server
                 *@param  fingerprint  The fingerprint of the host key it
                 *      supplied
                 */
                public void onUnknownHost(String host, String fingerprint) {


                    // Set up the users options
                    Object[] options = {"Always", "Yes", "No"};

                    int res = JOptionPane.showOptionDialog(null,
                            "The host '" + host + "' is unknown.\nThe host key" +
                            " fingerprint is '" + fingerprint + "'.\nDo you want to allow this host?",
                            "Remote host authentication", JOptionPane.YES_NO_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE, null, options, options[0]);

                  try {
                    // Handle the choice
                    switch (res) {
                        case JOptionPane.YES_OPTION:
                        {
                            // Allow the host and save it to the host file
                            allowHost(host, fingerprint, true);
                            break;
                        }
                        case JOptionPane.NO_OPTION:
                        {
                            // Allow the host just this once
                            allowHost(host, fingerprint, false);
                            break;
                        }
                        case JOptionPane.CANCEL_OPTION:
                        {
                            // Do nothing we dont want to allow this host access
                            break;
                        }
                    }

                  } catch(InvalidHostFileException ihfe) {
                    showExceptionMessage(ihfe.getMessage(), "Host Verification");
                  }

                }


                /**
                 *  Overide this method so that we can handle a host key
                 *  mismatch
                 *
                 *@param  host                 The name of the host
                 *@param  recordedFingerprint  The fingerprint recorded in
                 *      allowed hosts file
                 *@param  actualFingerprint    The fingerprint the host supplied
                 */
                public void onHostKeyMismatch(String host, String recordedFingerprint,
                        String actualFingerprint) {

                    // Setup the options
                    Object[] options = {"Always", "Yes", "No"};

                    int res = JOptionPane.showOptionDialog(null,
                            "The host '" + host + "' has provided a different host key.\nThe host key" +
                            " fingerprint provided is '" + actualFingerprint + "'.\n" +
                            "The allowed host key fingerprint is " + recordedFingerprint + ".\nDo you want to allow this host?",
                            "Remote host authentication", JOptionPane.YES_NO_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE, null, options, options[0]);

                  try {
                    // Handle the reply
                    switch (res) {
                        case JOptionPane.YES_OPTION:
                        {
                            // Always allow the host with the new fingerprint
                            allowHost(host, actualFingerprint, true);
                            break;
                        }
                        case JOptionPane.NO_OPTION:
                        {
                            // Only allow the host this once
                            allowHost(host, actualFingerprint, false);
                            break;
                        }
                        case JOptionPane.CANCEL_OPTION:
                        {
                            // Deny the host
                            break;
                        }
                    }

                  } catch(InvalidHostFileException ihfe) {
                    showExceptionMessage(ihfe.getMessage(), "Host Verification");
                  }
                }


                /**
                 *  Overide the method to get notification of a denied host
                 *
                 *@param  host  The name of the host
                 */
                public void onDeniedHost(String host) {

                    // Show a message to the user to inform them that the host
                    // is denied
                    JOptionPane.showMessageDialog(null, "Access to '" + host + "' is denied.\n" +
                            "Verify the access granted/denied in the allowed hosts file.",
                            "Remote Host Authentication", JOptionPane.OK_OPTION);

                }

            };
    }


    /**
     *  Register an action
     *
     *@param  action  The action to register
     */
    private void registerAction(StandardAction action) {
        action.addActionListener(this);
        actions.addElement(action);
    }


    /**
     *  Creates the status bar.
     *
     *@return    The status bar
     */
    private JLabel createStatusBar() {

        // Set the text
        statusText = "Ready...";
        JLabel status = new JLabel(statusText);
        status.setBorder(BorderFactory.createEtchedBorder());
        return status;
    }

    /**
     * Enables the File menu's options
     * @param connected
     */
    private void setFileMenuStatus(boolean connected) {

      newAction.setEnabled(!connected);
      openAction.setEnabled(!connected);
      closeAction.setEnabled(connected);
      saveAction.setEnabled(connected);
      exitAction.setEnabled(true);

    }


    /**
     *  Creates the application menu bar.
     *
     *@return    The menu
     */
    private JMenuBar createMenu() {

        JMenuBar menuBar = new JMenuBar();

        JMenuItem menuItem;

        // Build the File menu
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic('F');

        // Add the New command to the File menu
        menuItem = fileMenu.add(newAction);
        menuItem.addMouseListener(mouseHandler);

        // Add the Open command to the File menu
        menuItem = fileMenu.add(openAction);
        menuItem.addMouseListener(mouseHandler);

        // Add the Close command to the File menu
        menuItem = fileMenu.add(closeAction);
        menuItem.addMouseListener(mouseHandler);

        // Add the Save command to the File menu
        menuItem = fileMenu.add(saveAction);
        menuItem.addMouseListener(mouseHandler);

        // Add a separator to the File menu
        fileMenu.addSeparator();

        // Add the Exit command to the File menu
        menuItem = fileMenu.add(exitAction);
        menuItem.addMouseListener(mouseHandler);

        // Add the connection Menue
        menuBar.add(fileMenu);

        return menuBar;
    }


    /**
     *  Create the terminal emulation object
     *
     *@return    Description of the Return Value
     */
    private TerminalEmulation createEmulation() {

        TerminalEmulation emulation =
            new TerminalEmulation(TerminalEmulation.VT320) {

                /**
                 *  Overide the write command so we can pick up terminal events
                 *  and send them through the session object
                 *
                 *@param  data  The data to send
                 */
                public void write(byte[] data) {
                    try {

                        // Send the data
                        SshTerminalFrame.this.session.sendSessionData(data);

                    } catch (WindowSpaceNotAvailableException wpna) {
                        SshTerminalFrame.this.showExceptionMessage("There is no window space available to send data", "Terminal");
                    /*} catch (ConnectionProtocolException cpe) {
                        SshTerminalFrame.this.showExceptionMessage("An unexpected error occured!\n\n" + cpe.getMessage(), "Terminal");
                    */} catch (TransportProtocolException tpe) {
                        SshTerminalFrame.this.showExceptionMessage("An unexpected error occured!\n\n" + tpe.getMessage(), "Terminal");
                    }
                }

            };

        return emulation;
    }


    /**
     *  Displays an exception message to the user
     *
     *@param  message  The message to display
     *@param  title    The message box title
     */
    protected void showExceptionMessage(String message, String title) {

        JOptionPane.showMessageDialog(this, message, title, JOptionPane.ERROR_MESSAGE);

    }

    /**
     * Exits the application
     */
    private void exitApplication() {

      if (session != null) {

        try {
                session.close();
            } catch (SshException se) {
                showExceptionMessage("An unexpected error occured!\n\n" +
                        se.getMessage(), "Closing Session");
            }
      }

      System.exit(0);

    }

    /**
     *  Creates the SSH session
     *
     *@return    The SessionClient object
     */
    private SessionClient createSession() {

        return
            new SessionClient() {

                /**
                 *  Overide this method to initiate the session in the desired
                 *  manner
                 */
                protected void initSession() {

                    try {
                        // Request a pseudo terminal
                        if (requestPseudoTerminal(emulation)) {

                            // Start the users shell
                            startShell();
                        }
                    } catch (Exception e) {
                        showExceptionMessage("An unexpected error occured\n\n" +
                                e.getMessage(), "Session");
                    }
                }


                /**
                 *  Called by the framework in the event that the session cannot
                 *  be started.
                 *
                 *@param  reason  The reason supplied by the remote host
                 */
                protected void onStartSessionFailure(String reason) {

                    showExceptionMessage("The session failed to start\n\n" + reason,
                            "Session");
                }


                /**
                 *  Called by the framework in the event of a thread exception,
                 *  this is done instead of printing out to System.err or
                 *  System.out
                 *
                 *@param  e  The exception thrown
                 */
                protected void onThreadException(Exception e) {

                    // Display the error to the user
                    showExceptionMessage("An unexpected error occured!\n\nDetails: " +
                            e.getMessage(), "Thread Exception");

                    closeSession(false);
                }


                /**
                 *  Called by the framework when the remote computer disconnects
                 *  the connection.
                 *
                 *@param  reason  The reason for the disconnection
                 */
                protected void onDisconnect(String reason) {

                    // Set the status bar to the reason
                    jLabelStatus.setText(reason);

                    // If we got an exit code then show that as well
                    Integer exitCode = getExitCode();
                    if (exitCode != null) {
                        jLabelStatus.setText(jLabelStatus.getText() + ": Exit code is " + exitCode.toString());
                    }

                    closeSession(false);


                }


                /**
                 *  Called when the connection has been completed and is ready
                 *  for authenticaiton
                 */
                protected void onConnectionComplete() {

                    // We should now authenticate
                    int result;

                    // Our authenticated flag
                    boolean authenticated = false;

                    // Did we receive a banner from the remote computer
                    if (session.getAuthenticationBanner() != null) {
                        // Yes we did so show it by creating a new instance of banner dialog
                        BannerDialog banner = new BannerDialog(SshTerminalFrame.this, session.getAuthenticationBanner());
                        banner.showCentered();
                    }

                    // Get the supported authentication methods
                    java.util.List supported = SshAuthenticationFactory.getSupportedMethods();

                    // Create a list for display that will contain only the
                    // supported and available methods
                    java.util.List display = new java.util.ArrayList();

                    // Get the available methods
                    java.util.List auths = getAvailableAuthMethods();

                    // Iterate through selecting only the supported and available
                    // authenticaiton methods
                    Iterator it = supported.iterator();

                    while (it.hasNext() && !authenticated) {

                        Object obj = it.next();

                        if (auths.contains(obj)) {
                            display.add(obj);
                        }
                    }

                    // First look to see if we have any authenticaiton methods available
                    // in the current properties object as this will overide a manual selection
                    java.util.List selected = current.getAuthenticationMethods();

                    // If we don't have any selected authentications then show
                    // a dialog for the user to choose from the available
                    if (selected.size() <= 0) {

                        // Create a new authenticaiton dialog
                        AuthenticationDialog dialog = new AuthenticationDialog(
                                SshTerminalFrame.this);

                        dialog.setLocation((SshTerminalFrame.this.getWidth() - dialog.getWidth() / 2),
                                (SshTerminalFrame.this.getHeight() - dialog.getHeight() / 2));

                        // Get the selected authenticaition methoss
                        selected = dialog.showAuthenticationMethods(display);
                    }

                    if (selected.size() > 0) {

                        it = selected.iterator();
                        String method;
                        SshAuthentication auth;

                        while (it.hasNext()) {

                            // Get the next method
                            method = (String) it.next();

                            // Get an instance of the method implementation
                            try {
                              auth = SshAuthenticationFactory.newInstance(method);

                            } catch(AlgorithmNotSupportedException anse) {
                                showExceptionMessage("The authentication method '" + method + "' is not supported!", "Authentication");
                                return;
                            }

                            // Loop untill the user either cancels or completes
                            boolean completed = false;

                            while (!completed) {

                                // Set the username
                                auth.setUsername(current.getUsername());

                                // Show the implementations dialog
                                if (auth.showAuthenticationDialog(SshTerminalFrame.this)) {

                                    try {

                                        // Authentication with the details supplied
                                        result = session.authenticate(auth);

                                        if (result == AuthenticationProtocolState.FAILED ) {

                                            // If we failed ask the user if they wish to try again
                                            // as they might have mistyped!
                                            if (JOptionPane.showConfirmDialog(SshTerminalFrame.this,
                                                    method + " authentication failed, try again?", method + " authenticaiton",
                                                    JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {

                                                // The user has decided to cancel so just return
                                                return;
                                            }
                                        }

                                        /*if (result == AUTH_NOT_SUPPORTED) {
                                            showExceptionMessage("The selected authentication method is not supported!", "Authentication");
                                            return;
                                        }*/

                                        // If the result returned partial success then continue
                                        if (result == AuthenticationProtocolState.PARTIAL) {
                                            // We succeeded so add to the connections authenticaiton
                                            // list and continue on to the next one
                                            current.addAuthenticationMethod(method);
                                        }

                                        if (result == AuthenticationProtocolState.COMPLETE ) {

                                            //If successfull add to the connections list so we can save later
                                            current.addAuthenticationMethod(method);

                                            // We are now connected
                                            jLabelStatus.setText("Connected.");

                                            // Setup the menu
                                            setFileMenuStatus(true);

                                            // Make sure the terminal has focus
                                            terminal.requestFocus();

                                            // Set the completed flag
                                            completed = true;
                                        }
                                    } catch (AuthenticationProtocolException ape) {
                                        showExceptionMessage("An unexpected error occured!\n\n" +
                                                ape.getMessage(), "User Authentication");
                                    } catch (ServiceOperationException sie) {
                                        showExceptionMessage("An unexpected error occured!\n\n" +
                                                sie.getMessage(), "User Authentication");
                                    } catch (TransportProtocolException sie) {
                                        showExceptionMessage("An unexpected error occured!\n\n" +
                                                sie.getMessage(), "User Authentication");
                                    }
                                    // end of try
                                }
                                else {

                                    // User has cancelled the authenticaiton
                                    closeSession(true);
                                    return;
                                }

                                // end of if
                            }
                            // end of while

                        }
                        // end of while

                    }
                    // end of if
                }


                /**
                 *  Called by the framework when session data arrives. We will
                 *  direct this to the terminal
                 *
                 *@param  data  The data received
                 */
                public void onSessionData(byte[] data) {

                    try {

                        // add it to the terminal emulation
                        SshTerminalFrame.this.emulation.putString(new String(data, 0, data.length, "latin1"));
                    } catch (Exception e) {
                        showExceptionMessage("An unexpected error occured!\n\n" + e.getMessage(), "Session");
                    }

                }


                /**
                 *  Called by the framework when data on ext arrives
                 *
                 *@param  data
                 */
                public void onSessionExtData(byte[] data) {

                    try {
                        SshTerminalFrame.this.emulation.putString(new String(data, 0, data.length, "latin1"));
                    } catch (Exception e) {
                        showExceptionMessage("An unexpected error occured!\n\n" + e.getMessage(), "On Session Extended Data");
                    }

                }

            };
    }



    /**
     *  This adapter is constructed to handle mouse over component events.
     *
     *@author     lee
     *@created    22 August 2002
     */
    private class MouseHandler extends MouseAdapter {

        private JLabel label;


        /**
         *  Constructor for the handler
         *
         *@param  label  the JLabel which will recieve value of the
         *      Action.LONG_DESCRIPTION key.
         */
        public MouseHandler(JLabel label) {
            setLabel(label);
        }


        /**
         *  Sets the label attribute of the MouseHandler object
         *
         *@param  label  The new label value
         */
        public void setLabel(JLabel label) {
            this.label = label;
        }


        /**
         *  When the mouse enters the action we update the status bar
         *
         *@param  evt  The mouse event
         */
        public void mouseEntered(MouseEvent evt) {
            if (evt.getSource() instanceof AbstractButton) {
                AbstractButton button = (AbstractButton) evt.getSource();
                Action action = button.getAction();
                // getAction is new in JDK 1.3
                if (action != null) {
                    String message = (String) action.getValue(Action.LONG_DESCRIPTION);
                    label.setText(message);
                } else {
                    label.setText(SshTerminalFrame.this.statusText);
                }
            }
        }
    }

}

