/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.util;

import java.math.BigInteger;
import java.io.ByteArrayInputStream;
import java.io.IOException;

/**
 *  <p>
 *
 *  Provides a reader type interface into the ByteArrayInputStream.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class ByteArrayReader extends ByteArrayInputStream {

    /**
     *  Constructor for the ByteArrayReader object
     *
     *@param  data  The byte array to be read.
     */
    public ByteArrayReader(byte[] data) {
        super(data);
    }


    /**
     *  <p>
     *
     *  Reads an integer from the byte array.</p> <p>
     *
     *  NOTE: This currently returns a long but only extracts 4 bytes. The
     *  reason behind this is that this class has been designed for SSH byte
     *  arrays; SSH uses unsigned integers but in java we do not have an
     *  unsigned integer type. To safely return the non negative value we must
     *  extract the byte information into a long to preserve the unsigned value.
     *  THIS WILL BE UPDATED SOON WITH THE UnsignedInteger class.</p>
     *
     *@return                  Description of the Return Value
     *@exception  IOException  Description of the Exception
     */
    public long readInt() throws IOException {

        byte[] raw = new byte[4];
        read(raw);

        int ret = ((raw[0] & 0xFF) << 24)
                | ((raw[1] & 0xFF) << 16)
                | ((raw[2] & 0xFF) << 8)
                | ((raw[3] & 0xFF) << 0);

        return ret;
    }


    /**
     *  Previews an integer in the byte array provided. The method reads 4 bytes
     *  from the position in the array specified by start and returns the
     *  integer value
     *
     *@param  data   The byte array to preview
     *@param  start  The start position in the array
     *@return        The int value
     */
    public static int previewInt(byte[] data, int start) {

        int ret = ((data[start] & 0xFF) << 24)
                | ((data[start + 1] & 0xFF) << 16)
                | ((data[start + 2] & 0xFF) << 8)
                | ((data[start + 3] & 0xFF) << 0);

        return ret;
    }



    /**
     *  Reads a string from the byte array, the format of the string will be 4
     *  bytes representing an int value for the lengh of the string and then len
     *  bytes of data.
     *
     *@return                  A the string read from the byte array
     *@exception  IOException  Thrown of the data cannot be read
     */
    public String readString() throws IOException {

        long len = readInt();
        byte[] raw = new byte[(int) len];
        read(raw);

        return new String(raw);
    }


    /**
     *  Returns a binary string from the byte array. The method first reads an
     *  integer to obtain the length of the data and then reads the data which
     *  is returned in the byte array
     *
     *@return                  The binary data read
     *@exception  IOException  Thrown if the data cannot be read
     */
    public byte[] readBinaryString() throws IOException {

        long len = readInt();
        byte[] raw = new byte[(int) len];
        read(raw);

        return raw;
    }



    /**
     *  Returns a BigInteger from the byte array. The methos first reads the
     *  length of the BigInteger data and then reads and creates a BigInteger
     *  from the data read.
     *
     *@return                  The BigInteger value read
     *@exception  IOException  Thrown if the data cannot be read
     */
    public BigInteger readBigInteger() throws IOException {

        int len = (int) readInt();

        byte[] raw = new byte[len];

        read(raw);

        return new BigInteger(raw);
    }

}
