/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.util;

import java.math.BigInteger;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 *  <p>
 *
 *  Provides a reader type interface into the ByteArrayOutputStream</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class ByteArrayWriter extends ByteArrayOutputStream {

    /**
     *  Constructor for the ByteArrayWriter object
     */
    public ByteArrayWriter() { }


    /**
     *  <p>
     *
     *  Writes an integer into the byte array.</p> <p>
     *
     *  NOTE: This will be replaced with the UnsignedInteger class</p>
     *
     *@param  i                A long value to write (will be truncated if
     *      greater than max integer size)
     *@exception  IOException  Thrown if the array cannot be written to
     */
    public void writeInt(long i) throws IOException {

        byte[] raw = new byte[4];

        raw[0] = (byte) (i >> 24);
        raw[1] = (byte) (i >> 16);
        raw[2] = (byte) (i >> 8);
        raw[3] = (byte) (i);

        write(raw);

    }


    /**
     *  <p>
     *
     *  Writes an integer into the byte array.</p> <p>
     *
     *  NOTE: This will be replaced with the UnsignedInteger class</p>
     *
     *@param  i                The integer value
     *@exception  IOException  Thrown if the array cannot be written to
     */
    public void writeInt(int i) throws IOException {

        byte[] raw = new byte[4];

        raw[0] = (byte) (i >> 24);
        raw[1] = (byte) (i >> 16);
        raw[2] = (byte) (i >> 8);
        raw[3] = (byte) (i);

        write(raw);
    }


    /**
     *  Writes a string to the byte array. The method first writes the length of
     *  the string as an integer and then the string data.
     *
     *@param  str              The string to write
     *@exception  IOException  Thrown if the array cannot be written to
     */
    public void writeString(String str) throws IOException {

        writeInt(str.length());

        write(str.getBytes());

    }


    /**
     *  Writes a binary string to the byte array. The method first writes the
     *  length of the data as an integer and then the data itself.
     *
     *@param  data             The data to write
     *@exception  IOException  Thrown if the array cannot be written to
     */
    public void writeBinaryString(byte[] data) throws IOException {

        writeInt(data.length);

        write(data);
    }


    /**
     *  Writes a BigInteger to the array. The method first writes the length of
     *  the encoded data and then the data itself.
     *
     *@param  bi               The BigInteger to write
     *@exception  IOException  Thrown if the array cannot be written to
     */
    public void writeBigInteger(BigInteger bi) throws IOException {

        byte[] raw = bi.toByteArray();

        writeInt(raw.length);

        write(raw);

    }


    /**
     *  Writes an integer into an array, starting at the position specified
     *
     *@param  array         The array to write to
     *@param  pos           The starting position
     *@param  value         The integer value
     *@throws  IOException  Thrown if there is not at least 4 bytes of data to
     *      write to from pos to the end of the array
     */
    public static void writeIntToArray(byte[] array, int pos, int value)
             throws IOException {

        if (array.length - pos < 4) {
            throw new IOException("Not enough data in array to write integer at position " +
                    String.valueOf(pos));
        }

        array[pos] = (byte) (value >> 24);
        array[pos + 1] = (byte) (value >> 16);
        array[pos + 2] = (byte) (value >> 8);
        array[pos + 3] = (byte) (value);
    }

}
