/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.util;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.math.BigInteger;
import java.io.IOException;
/**
 *  <p>
 *
 *  Template helper class for Hash alogorithms, wraps the MessageDigest class
 *  from the java.security package</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class Hash {

    private MessageDigest hash;


    /**
     *  Constructor for the Hash object
     *
     *@param  algorithm                     The algorithm to use
     *@exception  NoSuchAlgorithmException  Thrown if the algorithm is not
     *      supported
     */
    public Hash(String algorithm) throws NoSuchAlgorithmException {

        hash = MessageDigest.getInstance(algorithm);

    }


    /**
     *  Puts a string into the hash. Writes the length as an integer and then
     *  the string data
     *
     *@param  str  The string to hash
     */
    public void putString(String str) {

        putInt(str.length());

        hash.update(str.getBytes());

    }


    /**
     *  Resets the hash;
     */
    public void reset() {
        hash.reset();
    }


    /**
     *  Puts an integer into the hash.
     *
     *@param  i  The integer value
     */
    public void putInt(int i) {

        ByteArrayWriter baw = new ByteArrayWriter();
        try {
            baw.writeInt(i);
        } catch (IOException ioe) {

        }
        hash.update(baw.toByteArray());

    }


    /**
     *  Puts an array of bytes into the hash
     *
     *@param  data  The byte array to hash
     */
    public void putBytes(byte[] data) {

        hash.update(data);

    }


    /**
     *  Puts a byte into the hash
     *
     *@param  b  The byte to hash
     */
    public void putByte(byte b) {
        hash.update(b);
    }


    /**
     *  Puts a BigInteger into the hash. Writes the length as an integer and
     *  then the BigInteger data
     *
     *@param  bi  The BigInteger to hash
     */
    public void putBigInteger(BigInteger bi) {

        byte[] data = bi.toByteArray();

        putInt(data.length);
        hash.update(data);

    }


    /**
     *  Puts the data into a new instance of the algorithm message digest and
     *  returns the output.
     *
     *@param  data                          The data to hash
     *@param  algorithm                     The algorithm to use
     *@return                               The hash output
     *@exception  NoSuchAlgorithmException  Thrown of the algorithm is not
     *      supported
     */
    public static byte[] simple(byte[] data, String algorithm) throws NoSuchAlgorithmException {

        MessageDigest simp = MessageDigest.getInstance(algorithm);

        simp.update(data);

        return simp.digest();
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public byte[] doFinal() {

        return hash.digest();
    }

}
