/*
* Copyright (c) 2002 Sshtools.com All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* Redistributions of source code must retain the above copyright notice, this list
* of conditions and the following disclaimer.
*
* Redistributions in binary form must reproduce the above copyright notice, this
* list of conditions and the following disclaimer in the documentation and/or other
* materials provided with the distribution.
*
* Neither the name of the Sshtools.com nor the names of its contributors may be
* used to endorse or promote products derived from this software without specific
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/

package com.sshtools.util;

/**
 *  <p>
 *
 *  Implements a true unsigned integer type storing the value as a series of
 *  bytes. This is helpful to the SSH API because there is no unsigned integer
 *  type in Java and all SSH integers are unsigned we need to be sure that we
 *  correclty interpret the integers send and received.</p>
 *
 *@author     Lee David Painter (<A HREF="mailto:lee@sshtools.com">
 *      lee@sshtools.com</A> )
 *@created    31 August 2002
 *@version    1.0
 */

public class UnsignedInteger extends Number {

    byte[] bytes = new byte[4];


    /**
     *  Constructor for the UnsignedInteger32 object
     *
     *@param  value  Description of the Parameter
     */
    public UnsignedInteger(int value) {

        bytes[0] = (byte) (value >> 24);
        bytes[1] = (byte) (value >> 16);
        bytes[2] = (byte) (value >> 8);
        bytes[3] = (byte) (value);

    }


    /**
     *  Constructor for the UnsignedInteger32 object
     *
     *@param  value  Description of the Parameter
     */
    public UnsignedInteger(long value) {

        bytes[0] = (byte) (value >> 24);
        bytes[1] = (byte) (value >> 16);
        bytes[2] = (byte) (value >> 8);
        bytes[3] = (byte) (value);
    }


    /**
     *  Constructor for the UnsignedInteger32 object
     *
     *@param  value  Description of the Parameter
     */
    public UnsignedInteger(String value) {
        this(Long.valueOf(value).longValue());
    }


    /**
     *  Constructor for the UnsignedInteger32 object
     *
     *@param  value  Description of the Parameter
     */
    public UnsignedInteger(Integer value) {
        this(value.intValue());
    }


    /**
     *  Constructor for the UnsignedInteger32 object
     *
     *@param  value  Description of the Parameter
     */
    public UnsignedInteger(byte[] value) {

        for (int i = 0; i < value.length && i < 4; i++) {
            bytes[i] = value[i];
        }

    }


    /**
     *  Constructor for the UnsignedInteger32 object
     *
     *@param  value  Description of the Parameter
     */
    public UnsignedInteger(Long value) {
        this(value.longValue());

    }



    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public int intValue() {

        int ret = ((bytes[0] & 0xFF) << 24)
                | ((bytes[1] & 0xFF) << 16)
                | ((bytes[2] & 0xFF) << 8)
                | ((bytes[3] & 0xFF) << 0);

        return ret;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public float floatValue() {

        return 1;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public double doubleValue() {
        return 1;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public long longValue() {

        long ret = ((bytes[0] & 0xFF) << 24)
                | ((bytes[1] & 0xFF) << 16)
                | ((bytes[2] & 0xFF) << 8)
                | ((bytes[3] & 0xFF) << 0);

        return ret;
    }


    /**
     *  Description of the Method
     *
     *@param  value  Description of the Parameter
     *@return        Description of the Return Value
     */
    public boolean equals(UnsignedInteger value) {

        return bytes[0] == value.bytes[0] &&
                bytes[1] == value.bytes[1] &&
                bytes[2] == value.bytes[2] &&
                bytes[3] == value.bytes[3];
    }


    /**
     *  Description of the Method
     *
     *@param  value  Description of the Parameter
     *@return        Description of the Return Value
     */
    public boolean equals(int value) {

        return equals((long) value);
    }


    /**
     *  Description of the Method
     *
     *@param  value  Description of the Parameter
     *@return        Description of the Return Value
     */
    public boolean equals(long value) {
        return bytes[0] == (value >> 24 & 0xFF) &&
                bytes[1] == (value >> 16 & 0xFF) &&
                bytes[2] == (value >> 8 & 0xFF) &&
                bytes[3] == (value & 0xFF);
    }


    /**
     *  Returns a decimal formatted string
     *
     *@return    Description of the Return Value
     */
    public String toString() {

        int workspace[];
        int offset = 0;
        int carry;
        char base = '0';
        char index;
        boolean isMore;
        String formatted = "";
        StringBuffer output = new StringBuffer();

        // Determine the number of bytes we need
        for (int i = 0; i < bytes.length; i++) {
            if (bytes[i] != 0) {
                offset = i;
                break;
            }
        }

        // Create a workspace
        workspace = new int[bytes.length - offset];

        // Copy the data into the workspace
        for (int i = 0; i < bytes.length - offset; i++) {
            workspace[i] = bytes[offset + i] & 0xFF;
        }
        // Remember to get rid of the sign

        do {

            carry = 0;

            for (int i = 0; i < workspace.length; i++) {
                carry = (carry << 8) + workspace[i];
                workspace[i] = (carry / 10);
                carry %= 10;

            }

            // Write the character to our buffer
            output.insert(0, (char) (base + carry));

            // Reset the state and determin if we have more data to process
            isMore = false;

            for (int i = 0; i < workspace.length; i++) {
                if (workspace[i] != 0) {
                    isMore = true;
                    break;
                }
            }

        } while (isMore);

        return output.toString();
    }


    /**
     *  Description of the Method
     *
     *@param  value  Description of the Parameter
     *@return        Description of the Return Value
     */
    public UnsignedInteger add(UnsignedInteger value) {

        /*for (int i = 0; i < this.bytes.length; i++) {
            System.out.print(this.bytes[i] + " ");
        }
        System.out.println();
        for (int i = 0; i < value.bytes.length; i++) {
            System.out.print(value.bytes[i] + " ");
        }
        System.out.println();
        */

        UnsignedInteger result = new UnsignedInteger(0);

        int workspace;
        int carry = 0;
        for (int i = this.bytes.length - 1; i >= 0; i--) {

            workspace = carry + (this.bytes[i] & 0xFF) + (value.bytes[i] & 0xFF);

            carry = workspace >> 8;

            result.bytes[i] = ((byte) (workspace % 256));

        }

        /*for (int i = 0; i < result.bytes.length; i++) {
            System.out.print(result.bytes[i] + " ");
        }
        System.out.println();
        */
        return result;
    }


    /**
     *  Description of the Method
     *
     *@param  value  Description of the Parameter
     *@return        Description of the Return Value
     */
    public UnsignedInteger subtract(UnsignedInteger value) {

        /*for (int i = 0; i < this.bytes.length; i++) {
            System.out.print(this.bytes[i] + " ");
        }
        System.out.println();
        for (int i = 0; i < value.bytes.length; i++) {
            System.out.print(value.bytes[i] + " ");
        }
        System.out.println();
        */

        UnsignedInteger result = new UnsignedInteger(0);

        int workspace;
        int carry = 0;
        for (int i = this.bytes.length - 1; i >= 0; i--) {

            workspace = (this.bytes[i] & 0xFF) - (value.bytes[i] & 0xFF) - carry;

            carry = workspace >> 8;

            result.bytes[i] = ((byte) (workspace % 256));

        }

       /* for (int i = 0; i < result.bytes.length; i++) {
            System.out.print(result.bytes[i] + " ");
        }
        System.out.println();
        */
        return result;
    }


    /**
     *  Description of the Method
     *
     *@param  value  Description of the Parameter
     *@return        Description of the Return Value
     */
    public UnsignedInteger multiply(UnsignedInteger value) {

        /*
         *  for(int i=0;i<this.bytes.length;i++)
         *  System.out.print(this.bytes[i] + " ");
         *  System.out.println();
         *  for(int i=0;i<value.bytes.length;i++)
         *  System.out.print(value.bytes[i] + " ");
         *  System.out.println();
         *  UnsignedInteger result = new UnsignedInteger(0);
         *  int workspace;
         *  int carry = 0;
         *  for(int i=this.bytes.length-1;i>=0;i--) {
         *  workspace = (this.bytes[i] & 0xFF);
         *  carry = 0;
         *  for(int j=value.bytes.length-1;j>=0;j--) {
         *  carry += workspace * (value.bytes[i] & 0xFF);
         *  carry += (result.bytes[i] & 0xFF);
         *  result.bytes[i+j+1] = (byte)carry;
         *  carry = carry >> 8;
         *  }
         *  result.bytes[i] = ((byte)carry);
         *  }
         *  for(int i=0;i<result.bytes.length;i++)
         *  System.out.print(result.bytes[i] + " ");
         *  System.out.println();
         *  return result;
         */
        return null;
    }


    /**
     *  Description of the Method
     *
     *@param  args  Description of the Parameter
     */
    public static void main(String args[]) {
        UnsignedInteger x = new UnsignedInteger(11020);
        UnsignedInteger y = new UnsignedInteger(2058);
        UnsignedInteger result = x.add(y);
        System.out.println("x = " + x.toString());
        System.out.println("y = " + y.toString());
        System.out.println("x + y = " + result.toString());

        result = x.subtract(y);

        System.out.println("x - y = " + result.toString());

        result = x.multiply(y);

        System.out.println("x * y = " + result.toString());

        byte v[] = new byte[4];
        v[0] = 1;
        v[1] = 90;
        v[2] = 14;
        v[3] = 120;

        int val = (v[0] & 0xFF) << 24 |
                (v[1] & 0xFF) << 16 |
                (v[2] & 0xFF) << 8 |
                v[3] & 0xFF;

        System.out.println(val);
    }

}
